Package: DT2
Type: Package
Title: 'DataTables' 2.x for R
Version: 0.1.2
Date: 2026-06-14
Authors@R: c(
    person("Andre", "Leite", email = "leite@castlab.org", role = c("aut", "cre")),
    person("Marcos", "Wasilew", email = "marcos.wasilew@gmail.com", role = "aut"),
    person("Hugo", "Vasconcelos", email = "hugo.vasconcelos@ufpe.br", role = "aut"),
    person("Carlos", "Amorin", email = "carlos.agaf@ufpe.br", role = "aut"),
    person("Diogo", "Bezerra", email = "diogo.bezerra@ufpe.br", role = "aut"))
Description: A modern R binding for 'DataTables' V2 with modular
    extension loading, 'Bootstrap 5' styling, 'Shiny' integration (proxy, events,
    inline inputs), server-side processing helpers, and standalone (non-Shiny)
    support. Configure 'DataTables' options directly via R lists, a 1:1 mapping
    to the 'JavaScript' API.
License: MIT + file LICENSE
URL: https://github.com/StrategicProjects/DT2,
        https://strategicprojects.github.io/DT2/
Encoding: UTF-8
Depends: R (>= 4.1)
Imports: htmlwidgets (>= 1.6.0), htmltools, jsonlite, cli
Suggests: shiny, rlang, dplyr, lubridate, tibble, bslib, knitr,
        rmarkdown, testthat (>= 3.1.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-14 00:53:34 UTC; leite
Author: Andre Leite [aut, cre],
  Marcos Wasilew [aut],
  Hugo Vasconcelos [aut],
  Carlos Amorin [aut],
  Diogo Bezerra [aut]
Maintainer: Andre Leite <leite@castlab.org>
Repository: CRAN
Date/Publication: 2026-06-14 05:20:09 UTC
Built: R 4.5.2; ; 2026-06-14 07:12:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,dt2_theme)
export(dt2)
export(dt2_bind_server)
export(dt2_buttons)
export(dt2_check_updates)
export(dt2_col_button)
export(dt2_col_checkbox)
export(dt2_col_template)
export(dt2_cols_align)
export(dt2_cols_escape)
export(dt2_cols_hide)
export(dt2_cols_html)
export(dt2_cols_render_js)
export(dt2_cols_render_orthogonal)
export(dt2_cols_width)
export(dt2_draw)
export(dt2_extensions)
export(dt2_format_datetime)
export(dt2_format_number)
export(dt2_format_number_abbrev)
export(dt2_format_time_format)
export(dt2_format_time_relative)
export(dt2_language)
export(dt2_length_menu)
export(dt2_order)
export(dt2_output)
export(dt2_proxy)
export(dt2_proxy_order)
export(dt2_proxy_page)
export(dt2_proxy_search)
export(dt2_register_renderer)
export(dt2_replace_data)
export(dt2_search_global)
export(dt2_select_rows)
export(dt2_ssp_handler)
export(dt2_state)
export(dt2_theme)
export(dt2_update_libs)
export(dt2_use_buttons)
export(dt2_use_renderer)
export(observe_dt2_events)
export(render_dt2)
importFrom(htmltools,htmlDependency)

# DT2 <img src="man/figures/logo.svg" align="right" height="139" />

<!-- badges: start -->
[![R-CMD-check](https://github.com/StrategicProjects/DT2/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/StrategicProjects/DT2/actions/workflows/R-CMD-check.yaml) 
![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/DT2) 
![CRAN Downloads](https://cranlogs.r-pkg.org/badges/grand-total/DT2) 
![License](https://img.shields.io/badge/license-MIT-darkviolet.svg) 
![Devel Badge](https://img.shields.io/badge/devel%20version-0.1.1-blue.svg)
<!-- badges: end -->

> **DataTables v2 for R** — modular, lightweight, works with or without Shiny.

DT2 is a modern `htmlwidgets` binding for
[DataTables.net v2](https://datatables.net/) with:

- **1:1 R ↔ JS mapping** — configure DataTables using plain R lists
- **Modular extensions** — load only Buttons, Select, Responsive, etc. when needed
- **Auto-detection** — DT2 scans your options and loads required extensions
- **Bootstrap 5** styling (or plain DataTables CSS)
- **Standalone** use in R Markdown, Quarto, and the Viewer (no Shiny required)
- **Full Shiny** integration: proxy, events, inline inputs, server-side processing

## Credits & Acknowledgments

DT2 is an **R wrapper** around the excellent
[**DataTables**](https://datatables.net/) JavaScript library created by
[**SpryMedia Ltd**](https://sprymedia.co.uk/) (Allan Jardine).
All table rendering, sorting, searching, pagination, and extensions are
powered entirely by the DataTables engine — DT2 simply provides the
bridge between R / htmlwidgets and the JavaScript API.

| Component | Author | License |
|---|---|---|
| [DataTables](https://datatables.net/) core + extensions | SpryMedia Ltd | MIT |
| [Bootstrap 5](https://getbootstrap.com/) integration | DataTables / Bootstrap | MIT |
| DT2 R package | André Leite, Marcos Wasilew, Hugo Vasconcelos, Carlos Amorin, Diogo Bezerra | MIT |

This package takes inspiration from the original
[DT](https://rstudio.github.io/DT/) package by Yihui Xie (RStudio / Posit),
which wraps DataTables v1.x. DT2 was written from scratch to support the
DataTables v2 API, the modern `layout` system, and new extensions like
ColumnControl.

## Installation

```r
# From CRAN
install.packages("DT2")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("StrategicProjects/DT2")
```

## Quick Start

### Standalone (no Shiny)

```r
library(DT2)

# Minimal
dt2(iris)

# With styling
dt2(mtcars,
    compact = TRUE, striped = TRUE, hover = TRUE, font_scale = 0.85,
    options = list(pageLength = 10))

# With export buttons
dt2(iris, options = list(
  pageLength = 10,
  buttons    = list("copy", "csv", "excel"),
  layout     = list(topEnd = "buttons")
))

# Save as standalone HTML
htmlwidgets::saveWidget(dt2(iris), "table.html")
```

### In Shiny (with bslib)

```r
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  card(
    card_header("DT2 in Shiny"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    dt2(iris,
        compact = TRUE, striped = TRUE, hover = TRUE,
        options = list(
          pageLength = 10,
          buttons    = list("copy", "csv", "excel"),
          layout     = list(topEnd = "buttons")
        ))
  })
}

shinyApp(ui, server)
```

## Extension System

DT2 ships with 15 DataTables extensions. See what's available:

```r
dt2_extensions()
#>            name version          dir
#> 1       Buttons   3.2.5      buttons
#> 2    ColReorder   2.1.1    colreorder
#> 3 ColumnControl   1.1.0 columncontrol
#> ...
```

Extensions are loaded **automatically** when DT2 detects them in your options,
or **explicitly** via the `extensions` argument:

```r
# Auto-detected (buttons in layout → loads Buttons extension)
dt2(iris, options = list(
  layout = list(topEnd = list(buttons = list("copy", "csv")))
))

# Explicit
dt2(iris, extensions = c("Buttons", "Select"))
```

## Translating DataTables.net Examples to R

Any example from [datatables.net](https://datatables.net/examples/) translates
directly to R lists:

| JavaScript | R |
|---|---|
| `{ pageLength: 25 }` | `list(pageLength = 25)` |
| `[1, 2, 3]` | `c(1, 2, 3)` |
| `{ layout: { topEnd: "buttons" } }` | `list(layout = list(topEnd = "buttons"))` |
| `function(d) { ... }` | `htmlwidgets::JS("function(d) { ... }")` |

See `vignette("js-config")` for a complete translation guide.


## Internationalization (i18n)

DataTables renders all interface text (pagination labels, search placeholder,
info line, button captions, etc.) from the `language` option.  Override any
term by passing a named list.

### Minimal — just the essentials

```r
dt2(iris, options = list(
  language = list(
    search       = "Buscar:",
    lengthMenu   = "Mostrar _MENU_ registros",
    info         = "Mostrando _START_ a _END_ de _TOTAL_",
    infoEmpty    = "Nenhum registro",
    zeroRecords  = "Nenhum resultado encontrado",
    paginate     = list(
      first = "«", previous = "‹", `next` = "›", last = "»"
    )
  )
))
```

### Complete pt-BR example

Below is a comprehensive Portuguese (Brazil) translation that covers the
core table, Buttons extension, and ColumnControl extension:

```r
lang_ptbr <- list(
  # ── Core table ──────────────────────────────────────────────
  lengthMenu     = "Mostrar _MENU_",
  search         = "Buscar",
  info           = "Mostrando _START_ a _END_ de _TOTAL_ registros",
  infoEmpty      = "Mostrando 0 a 0 de 0 registros",
  infoFiltered   = "(filtrado de _MAX_ registros no total)",
  zeroRecords    = "Nenhum registro encontrado",
  emptyTable     = "Nenhum dado disponível",
  loadingRecords = "Carregando dados...",
  decimal        = ",",
  thousands      = ".",
  infoThousands  = ".",
  paginate       = list(
    first = "«", previous = "‹", `next` = "›", last = "»"
  ),
  # Rótulos do menu "entries per page"
  # (mapeia o valor numérico → texto exibido no select)
  lengthLabels = list(
    `10` = "10", `25` = "25", `50` = "50", `-1` = "Todas"
  ),

  # ── Buttons extension ──────────────────────────────────────
  buttons = list(
    copyTitle   = "Copiado para a área de transferência",
    copyKeys    = paste0(
      "Pressione <i>Ctrl</i> ou <i>\u2318</i> + <i>C</i> para copiar. ",
      "Pressione Esc para cancelar."
    ),
    copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
  ),

  # ── ColumnControl extension ────────────────────────────────
  columnControl = list(
    colVis         = "Visibilidade da coluna",
    colVisDropdown = "Visibilidade da coluna",
    dropdown       = "Mostrar mais...",
    orderAsc       = "Ordem crescente",
    orderDesc      = "Ordem decrescente",
    orderClear     = "Remover ordenação",
    orderRemove    = "Remover ordenação",
    searchClear    = "Limpar pesquisa",
    searchDropdown = "Pesquisar",
    reorder        = "Reordenar",
    reorderLeft    = "Mover para a esquerda",
    reorderRight   = "Mover para a direita",
    list = list(
      all = "Todos", empty = "Vazio", none = "Nenhum",
      search = "Pesquisar..."
    ),
    search = list(
      text = list(
        contains = "Contém", empty = "Vazio", ends = "Termina em",
        equal = "Igual a", notContains = "Não contém",
        notEmpty = "Não está vazio", notEqual = "Diferente de",
        starts = "Começa por"
      ),
      number = list(
        empty = "Vazio", equal = "Igual a",
        greater = "Maior que", greaterOrEqual = "Maior ou igual a",
        less = "Menor que", lessOrEqual = "Menor ou igual a",
        notEmpty = "Não está vazio", notEqual = "Diferente de"
      ),
      datetime = list(
        empty = "Vazio", equal = "Igual a",
        greater = "Posterior a", less = "Anterior a",
        notEmpty = "Não está vazio", notEqual = "Diferente de"
      )
    )
  )
)

# Use it
dt2(iris, options = list(pageLength = 10, language = lang_ptbr))
```

### Using DataTables CDN translation files

DataTables provides ready-made translation files for 70+ languages.
You can load one directly via the `language.url` option:

```r
dt2(iris, options = list(
  language = list(
    url = "https://cdn.datatables.net/plug-ins/2.3.3/i18n/pt-BR.json"
  )
))
```

> **Note:** the CDN file covers the core table terms.  Extension-specific
> terms (Buttons, ColumnControl) must still be set manually via the
> `language` list as shown above.

Browse all available languages at:
<https://datatables.net/plug-ins/i18n/>


## Updating JS Libraries

All library versions are centralized in `tools/get-dt2-libs.sh`:

```bash
# Edit version variables at the top, then run:
bash tools/get-dt2-libs.sh
```

## Vignettes

| Vignette | Description |
|---|---|
| `vignette("getting-started")` | Standalone usage, themes, layout, pagination, buttons, scroller |
| `vignette("shiny-integration")` | Proxy, events, server-side processing (11 complete apps) |
| `vignette("extensions-guide")` | Buttons, Select, Responsive, ColumnControl, etc. |
| `vignette("formatting")` | Number/date formatting, custom JS renderers |
| `vignette("js-config")` | Translating DataTables.net JS examples to R |

A complete example app with ColumnControl, flags, custom renderers, and
full pt-BR translation is included at
[`inst/examples/app_complete.R`](inst/examples/app_complete.R).

## License

MIT — see [LICENSE](LICENSE) file.

DT2 bundles DataTables and extensions which are also MIT-licensed.
See [DataTables license](https://datatables.net/license/) for details.

# zzz.R — DT2 internal state
.DT2_STATE <- new.env(parent = emptyenv())

#' Lock Bootstrap mode per document (prevents mixing core/bootstrap5)
#' @keywords internal
.dt2_lock_mode <- function(bs) {
  if (!exists("bs_mode", envir = .DT2_STATE, inherits = FALSE)) {
    .DT2_STATE$bs_mode <- bs
  } else if (!identical(.DT2_STATE$bs_mode, bs)) {
    warning(sprintf(
      "DT2: this document already locked bs=\"%s\"; ignoring request for bs=\"%s\".",
      .DT2_STATE$bs_mode, bs
    ), call. = FALSE)
    bs <- .DT2_STATE$bs_mode
  }
  bs
}

.onLoad <- function(libname, pkgname) {
  # Reset state on package load
  if (exists("bs_mode", envir = .DT2_STATE, inherits = FALSE)) {
    rm("bs_mode", envir = .DT2_STATE)
  }
}

#' Create a proxy for a DT2 table
#' @param id Widget id used in [dt2_output()].
#' @param session Shiny session.
#' @return A `"DT2Proxy"` object.
#' @export
dt2_proxy <- function(id, session = shiny::getDefaultReactiveDomain()) {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("Package 'shiny' is required for dt2_proxy().", call. = FALSE)
  }
  structure(list(id = id, session = session), class = "DT2Proxy")
}

#' Replace all data in the table (proxy)
#' @param proxy [dt2_proxy()].
#' @param data New data.frame (will be serialized).
#' @return The `proxy` object, returned invisibly.
#' @export
dt2_replace_data <- function(proxy, data) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "replaceData", data = data))
  invisible(proxy)
}

#' Redraw the table (proxy)
#' @param proxy [dt2_proxy()].
#' @return The `proxy` object, returned invisibly.
#' @export
dt2_draw <- function(proxy) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "draw"))
  invisible(proxy)
}

#' Order the table (proxy)
#' @param proxy [dt2_proxy()].
#' @param ... Vectors `c(col, "asc"/"desc")`. If you pass `columns`, names will be resolved to indices.
#' @param columns Optional character vector of column names to resolve names to indices.
#' @return The `proxy` object, returned invisibly.
#' @export
dt2_proxy_order <- function(proxy, ..., columns = NULL) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  ord <- lapply(list(...), function(x) {
    if (!is.null(columns) && is.character(x[[1]])) x[[1]] <- match(x[[1]], columns)
    x
  })
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "order", args = list(ord)))
  invisible(proxy)
}

#' Global search (proxy)
#' @param proxy A [dt2_proxy()] object.
#' @param value Search string.
#' @param regex Logical; treat `value` as a regular expression? Default: `FALSE`.
#' @param smart Logical; use DataTables smart search? Default: `TRUE`.
#' @param caseInsensitive Logical; case-insensitive search? Default: `TRUE`.
#' @return The proxy, invisibly.
#' @export
dt2_proxy_search <- function(proxy, value, regex = FALSE, smart = TRUE, caseInsensitive = TRUE) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "search", args = list(value, regex, smart, caseInsensitive)))
  invisible(proxy)
}

#' Page navigation (proxy)
#' @param proxy A [dt2_proxy()] object.
#' @param page Navigation action: `"first"`, `"previous"`, `"next"`, `"last"`,
#'   or `"number"` (go to a specific page).
#' @param number Page number (1-based). Only used when `page = "number"`.
#' @return The proxy, invisibly.
#' @export
dt2_proxy_page <- function(proxy, page = c("first","previous","next","last","number"), number = NULL) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  page <- match.arg(page)
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "page", args = list(page, number)))
  invisible(proxy)
}

#' Select rows (proxy; Select extension)
#' @param proxy [dt2_proxy()].
#' @param indexes 1-based row indices.
#' @param reset If TRUE, clear selection before selecting.
#' @return The `proxy` object, returned invisibly.
#' @export
dt2_select_rows <- function(proxy, indexes, reset = TRUE) {
  stopifnot(inherits(proxy, "DT2Proxy"))
  proxy$session$sendCustomMessage(paste0(proxy$id, "_proxy"),
                                  list(cmd = "selectRows", args = list(as.integer(indexes), isTRUE(reset))))
  invisible(proxy)
}

#' Define initial ordering (option `order`)
#' @param options Options list.
#' @param ... Vectors like `c(col, "asc"/"desc")`. `col` may be name or 1-based index.
#' @return Updated `options`.
#' @export
#' @examples
#' opts <- list(columns = names(mtcars))
#' opts <- dt2_order(opts, c("mpg", "desc"))
#' dt2(mtcars, options = opts)
dt2_order <- function(options = list(), ...) {
  ord <- lapply(list(...), function(x) {
    idx <- .dt2_name_to_idx(x[[1]], options)
    list(idx - 1L, x[[2]])
  })
  options$order <- ord
  options
}

#' Set global search (option `search`)
#' @param options Options list.
#' @param value Text.
#' @param regex,smart,caseInsensitive Search flags.
#' @return Updated `options`.
#' @export
#' @examples
#' opts <- dt2_search_global(list(), value = "Toyota")
#' dt2(mtcars, options = opts)
dt2_search_global <- function(options = list(), value, regex = FALSE, smart = TRUE, caseInsensitive = TRUE) {
  options$search <- list(value = value, regex = regex, smart = smart, caseInsensitive = caseInsensitive)
  options
}

#' Enable Buttons (extension) and define buttons
#'
#' Uses the modern DataTables 2.x `layout` API (not the deprecated `dom`).
#'
#' @param options Options list.
#' @param buttons Vector of button ids (e.g., c("copy","csv","excel","print","colvis")).
#' @param position Where to place buttons in the layout.
#'   One of \code{"topEnd"} (default), \code{"topStart"}, \code{"bottomEnd"},
#'   \code{"bottomStart"}.
#' @param button_class CSS class for buttons (e.g., `"btn btn-sm btn-primary"`).
#'   If `NULL`, uses the theme default (`"btn btn-sm btn-outline-secondary"`).
#'   Applied per-button via `className`.
#' @return Updated `options`.
#' @seealso [dt2_buttons()] for a lower-level variant that takes full button
#'   objects and can relocate the buttons container to a custom CSS selector.
#' @export
#' @examples
#' opts <- dt2_use_buttons(list(), buttons = c("copy", "csv", "excel"))
#' dt2(mtcars, options = opts)
dt2_use_buttons <- function(options = list(),
                            buttons = c("copy","csv","excel","print"),
                            position = "topEnd",
                            button_class = NULL) {
  if (!is.null(button_class)) {
    # Apply className to each button
    options$buttons <- lapply(buttons, function(b) {
      list(extend = b, className = button_class)
    })
  } else {
    options$buttons <- as.list(buttons)
  }
  if (is.null(options$layout)) options$layout <- list()
  options$layout[[position]] <- "buttons"
  options
}

#' Language helper (either list or JSON url)
#' @param options Options list.
#' @param lang_list Named list of language keys.
#' @param lang_url URL to a JSON translation file.
#' @return Updated `options`.
#' @export
#' @examples
#' # Inline translation
#' opts <- dt2_language(list(), lang_list = list(search = "Buscar:"))
#' dt2(iris, options = opts)
#'
#' # Or load a ready-made translation file from the DataTables CDN
#' opts <- dt2_language(list(),
#'   lang_url = "https://cdn.datatables.net/plug-ins/2.3.3/i18n/pt-BR.json")
dt2_language <- function(options = list(), lang_list = NULL, lang_url = NULL) {
  if (!is.null(lang_url)) {
    options$language <- list(url = lang_url)
  } else if (!is.null(lang_list) && is.list(lang_list)) {
    options$language <- lang_list
  }
  options
}

#' Column widths (CSS)
#' @param options Options list.
#' @param map_named Named character vector: c(Col="120px", ...).
#' @return Updated `options`.
#' @export
dt2_cols_width <- function(options = list(), map_named) {
  options$columnDefs <- c(options$columnDefs %||% list(),
                          lapply(names(map_named), function(nm) {
                            i <- .dt2_name_to_idx(nm, options)
                            list(targets = i-1L, width = unname(map_named[[nm]]))
                          })
  )
  options
}

#' Column align (Bootstrap 5 classes)
#' @param options Options list.
#' @param cols Names or 1-based indices.
#' @param align "left","center","right".
#' @return Updated `options`.
#' @export
dt2_cols_align <- function(options = list(), cols, align = c("left","center","right")) {
  align <- match.arg(align)
  idx <- .dt2_name_to_idx(cols, options)
  cls <- switch(align, left="text-start", center="text-center", right="text-end")
  options$columnDefs <- c(options$columnDefs %||% list(),
                          lapply(idx, function(i) list(targets = i-1L, className = cls))
  )
  options
}

#' Hide columns
#' @param options Options list.
#' @param cols Names or 1-based indices.
#' @return Updated `options`.
#' @export
dt2_cols_hide <- function(options = list(), cols) {
  idx <- .dt2_name_to_idx(cols, options)
  options$columnDefs <- c(options$columnDefs %||% list(),
                          lapply(idx, function(i) list(targets = i-1L, visible = FALSE))
  )
  options
}

#' Escape/unescape columns content
#'
#' Controls whether cell content is HTML-escaped before display.
#' @param options Options list.
#' @param cols Names or indices.
#' @param escape If `TRUE` (default), HTML special characters are escaped so the
#'   raw text is shown. If `FALSE`, the content is rendered as raw HTML
#'   (use with care; only for trusted content).
#' @return Updated `options`.
#' @export
dt2_cols_escape <- function(options = list(), cols, escape = TRUE) {
  idx <- .dt2_name_to_idx(cols, options)
  render <- if (escape) {
    htmlwidgets::JS(
      "function(d,t){ if(t!=='display'||d==null) return d;",
      "  return String(d)",
      "    .replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;')",
      "    .replace(/\"/g,'&quot;').replace(/'/g,'&#39;'); }"
    )
  } else {
    htmlwidgets::JS("function(d,t){return d;}")
  }
  options$columnDefs <- c(options$columnDefs %||% list(),
                          lapply(idx, function(i) list(targets = i - 1L, render = render))
  )
  options
}

#' Length menu helper
#'
#' Configures the entries-per-page dropdown.
#'
#' @param options Options list.
#' @param values Numeric vector of page lengths (e.g., `c(10, 25, 50, -1)`).
#'   Use `-1` for "show all".
#' @param labels Optional character vector of labels. If `NULL`, numeric
#'   values are used as-is and `-1` becomes `"All"` automatically via
#'   `language.lengthLabels`.
#' @return Updated `options`.
#' @export
#'
#' @examples
#' opts <- dt2_length_menu(values = c(5, 10, 25, -1))
#' dt2(iris, options = opts)
#'
#' opts <- dt2_length_menu(values = c(10, 50, 100), labels = c("10", "50", "100"))
#' dt2(iris, options = opts)
dt2_length_menu <- function(options = list(), values = c(10, 25, 50, -1),
                            labels = NULL) {
  if (!is.null(labels) && length(labels) == length(values)) {
    # DT 2.x format: array of integers or {label, value} objects
    menu <- mapply(function(v, l) {
      if (as.character(v) == l) {
        v  # plain integer when label matches value
      } else {
        list(label = l, value = v)
      }
    }, values, labels, SIMPLIFY = FALSE, USE.NAMES = FALSE)
    options$lengthMenu <- menu
  } else {
    # Simple integer array — DT 2.x handles labels automatically.
    # For -1, set language.lengthLabels so it shows "All".
    options$lengthMenu <- as.list(as.integer(values))
    if (-1L %in% values) {
      if (is.null(options$language)) options$language <- list()
      if (is.null(options$language$lengthLabels)) options$language$lengthLabels <- list()
      options$language$lengthLabels[["-1"]] <- "All"
    }
  }
  options
}

# Tests for column-name resolution (the options$columns "footgun") and the
# automatic injection of options$columns by dt2().

test_that(".dt2_name_to_idx resolves names against options$columns", {
  expect_equal(.dt2_name_to_idx(c("b", "c"), list(columns = c("a", "b", "c"))),
               c(2L, 3L))
})

test_that(".dt2_name_to_idx passes 1-based numeric indices through", {
  expect_equal(.dt2_name_to_idx(c(2, 3), list()), c(2L, 3L))
})

test_that(".dt2_name_to_idx warns (not silent NA) when options$columns is unset", {
  expect_warning(idx <- .dt2_name_to_idx("x", list()), "options\\$columns")
  expect_true(all(is.na(idx)))
})

test_that(".dt2_name_to_idx warns on unknown column names", {
  expect_warning(idx <- .dt2_name_to_idx("zzz", list(columns = c("a", "b"))),
                 "Unknown column")
  expect_true(is.na(idx))
})

test_that("name-based helpers warn instead of failing silently", {
  expect_warning(dt2_cols_hide(list(), cols = "Species"),  "options\\$columns")
  expect_warning(dt2_cols_align(list(), "Species"),        "options\\$columns")
})

test_that("name-based helpers resolve correctly when columns are set", {
  o <- dt2_cols_hide(list(columns = c("a", "b", "c")), cols = c("b", "c"))
  expect_equal(vapply(o$columnDefs, function(d) d$targets, integer(1)), c(1L, 2L))
})

test_that("dt2() injects options$columns from the data when absent", {
  w <- dt2(head(iris, 2))
  expect_equal(w$x$options$columns, names(iris))
})

test_that("dt2() does not override user-provided options$columns", {
  cols <- c("A", "B", "C", "D", "E")
  w <- dt2(head(iris, 2), options = list(columns = cols))
  expect_equal(w$x$options$columns, cols)
})

# Regression tests for dt2_cols_escape().
# Bug: both branches of `if (escape)` returned the identical identity function
# `function(d,t){return d;}`, so `escape` was a no-op and the default
# (escape = TRUE) rendered raw HTML instead of escaping it.

test_that("dt2_cols_escape() escapes HTML special chars when escape = TRUE", {
  o <- dt2_cols_escape(list(columns = "x"), 1, escape = TRUE)
  r <- as.character(o$columnDefs[[1]]$render)
  expect_true(grepl("&lt;",   r, fixed = TRUE))
  expect_true(grepl("&gt;",   r, fixed = TRUE))
  expect_true(grepl("&amp;",  r, fixed = TRUE))
  expect_true(grepl("&quot;", r, fixed = TRUE))
})

test_that("dt2_cols_escape() renders raw HTML when escape = FALSE", {
  o <- dt2_cols_escape(list(columns = "x"), 1, escape = FALSE)
  r <- as.character(o$columnDefs[[1]]$render)
  expect_false(grepl("&lt;", r, fixed = TRUE))
})

test_that("dt2_cols_escape() TRUE and FALSE yield different renderers", {
  t_render <- as.character(
    dt2_cols_escape(list(columns = "x"), 1, escape = TRUE)$columnDefs[[1]]$render
  )
  f_render <- as.character(
    dt2_cols_escape(list(columns = "x"), 1, escape = FALSE)$columnDefs[[1]]$render
  )
  expect_false(identical(t_render, f_render))
})

# Null-coalescing operator (available in base R >= 4.4; provided here for R >= 4.1)
#' @keywords internal
`%||%` <- function(a, b) if (is.null(a)) b else a

# logging helpers (english logs)
.dt2_info  <- function(..., .envir = parent.frame()) cli::cli_inform(c("i" = ...), .envir = .envir)
.dt2_warn  <- function(..., .envir = parent.frame()) cli::cli_warn(c("!" = ...), .envir = .envir)
.dt2_abort <- function(..., .envir = parent.frame()) cli::cli_abort(c("x" = ...), .envir = .envir)

# Resolve column names OR 1-based indices to 1-based integer indices.
# Unlike a bare match(), this warns loudly instead of silently returning NA
# when `options$columns` is unset or a name does not exist, so the common
# "forgot to set options$columns" footgun is visible rather than silent.
#' @keywords internal
.dt2_name_to_idx <- function(cols, options) {
  if (is.numeric(cols)) return(as.integer(cols))
  if (!is.character(cols)) {
    .dt2_abort("Columns must be column names (character) or 1-based indices (numeric).")
  }
  if (is.null(options$columns)) {
    .dt2_warn(paste(
      "Column names were passed but {.code options$columns} is unset, so they",
      "cannot be resolved. Set {.code options$columns <- names(data)} before the",
      "column helpers, or pass 1-based indices instead."
    ))
    return(rep(NA_integer_, length(cols)))
  }
  idx <- match(cols, options$columns)
  if (anyNA(idx)) {
    .dt2_warn("Unknown column name{?s}: {.val {cols[is.na(idx)]}}.")
  }
  idx
}

# Render an R scalar as a safe JS string literal (properly quoted and escaped)
# for interpolation into generated JS, instead of sprintf("'%s'", x) which
# breaks when `x` contains a quote. `NULL` becomes `null_as` (e.g. "null"/"undefined").
#' @keywords internal
.dt2_js_str <- function(x, null_as = "null") {
  if (is.null(x)) return(null_as)
  as.character(jsonlite::toJSON(x, auto_unbox = TRUE))
}

# internal env to hold named renderers
.dt2_renderers <- new.env(parent = emptyenv())

#' Register a named JS renderer
#'
#' @param name Unique name (character scalar).
#' @param js A \code{htmlwidgets::JS()} function or a JSON helper expression.
#'
#' @return Invisibly, the name.
#' @export
dt2_register_renderer <- function(name, js) {
  stopifnot(is.character(name), length(name) == 1)
  if (!inherits(js, "JS_EVAL")) {
    stop("`js` must be created with htmlwidgets::JS(...).", call. = FALSE)
  }
  assign(name, js, envir = .dt2_renderers)
  invisible(name)
}

#' Use a named JS renderer on columns
#'
#' @param options Options list (returned modified).
#' @param col_specs Column names or indices.
#' @param name Name used in \code{dt2_register_renderer()}.
#'
#' @return Modified \code{options}.
#' @export
dt2_use_renderer <- function(options = list(), col_specs, name) {
  col_specs <- .dt2_name_to_idx(col_specs, options)
  js <- get0(name, envir = .dt2_renderers, inherits = FALSE)
  if (is.null(js)) stop(sprintf("Renderer '%s' is not registered.", name), call. = FALSE)

  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("DT2 in Shiny"),
#     card_body(dt2_output("my_table"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$my_table <- render_dt2({
#     dt2(iris, options = list(pageLength = 10))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Table State"),
#     card_body(dt2_output("tbl"))
#   ),
#   card(
#     card_header("Current state"),
#     card_body(verbatimTextOutput("state_info"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
#     dt2(iris, options = list(pageLength = 10))
#   })
# 
#   output$state_info <- renderPrint({
#     state <- input$tbl_state
#     if (is.null(state)) return("(interact with the table)")
#     cat("Reason:", state$reason, "\n")
#     cat("Page:",   state$page$page + 1, "of", state$page$pages, "\n")
#     cat("Search:", if (nzchar(state$search)) state$search else "(none)", "\n")
#     cat("Order:",  paste(state$order, collapse = ", "), "\n")
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_sidebar(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   title = "Proxy Demo",
#   sidebar = sidebar(
#     actionButton("refresh",  "Refresh data", class = "btn-sm btn-outline-primary w-100 mb-2"),
#     actionButton("go_last",  "Go to last page", class = "btn-sm btn-outline-primary w-100 mb-2"),
#     actionButton("sort_sl",  "Sort Sepal.Length \u2193", class = "btn-sm btn-outline-primary w-100 mb-2"),
#     hr(),
#     textInput("search_text", "Search", placeholder = "e.g. setosa"),
#     actionButton("search_btn", "Apply", class = "btn-sm btn-primary w-100")
#   ),
#   card(
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
#     dt2(iris, options = list(pageLength = 10))
#   })
# 
#   proxy <- dt2_proxy("tbl")
# 
#   observeEvent(input$refresh, {
#     new_data <- iris[sample(nrow(iris), 50), ]
#     dt2_replace_data(proxy, new_data)
#   })
# 
#   observeEvent(input$go_last, {
#     dt2_proxy_page(proxy, "last")
#   })
# 
#   observeEvent(input$sort_sl, {
#     dt2_proxy_order(proxy, c("Sepal.Length", "desc"),
#                     columns = names(iris))
#   })
# 
#   observeEvent(input$search_btn, {
#     dt2_proxy_search(proxy, input$search_text)
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Styled Buttons"),
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
#     opts <- dt2_use_buttons(
#       buttons = c("copy", "csv", "excel", "print"),
#       position = "topEnd",
#       button_class = "btn btn-sm btn-primary"
#     )
#     dt2(mtcars, options = opts)
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Scroller \u2014 10 000 rows"),
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   big <- data.frame(
#     id    = 1:10000,
#     value = round(rnorm(10000), 3),
#     group = sample(LETTERS[1:5], 10000, replace = TRUE)
#   )
# 
#   output$tbl <- render_dt2({
#     dt2(big, options = list(
#       scroller    = TRUE,
#       scrollY     = 400,
#       deferRender = TRUE,
#       buttons     = list("copy", "csv"),
#       layout      = list(
#         topStart = list(search = list(placeholder = "Filter...")),
#         topEnd   = "buttons"
#       )
#     ))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Custom Layout"),
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
#     dt2(iris, options = list(
#       pageLength = 10,
#       layout = list(
#         topStart  = "pageLength",
#         topEnd    = list(
#           buttons = list("copy", "csv", "excel"),
#           search  = list(placeholder = "Search...")
#         ),
#         bottomStart = "info",
#         bottomEnd   = "paging"
#       )
#     ))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   layout_columns(
#     col_widths = c(6, 6),
#     card(
#       card_header("Simple (prev / next only)"),
#       card_body(dt2_output("tbl_simple"))
#     ),
#     card(
#       card_header("Full (first / 1 2 3 / last)"),
#       card_body(dt2_output("tbl_full"))
#     )
#   ),
#   card(
#     card_header("No pagination"),
#     card_body(dt2_output("tbl_none"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl_simple <- render_dt2({
#     dt2(iris, options = list(
#       pageLength = 5,
#       layout = list(bottomEnd = list(paging = list(
#         type = "simple"
#       )))
#     ))
#   })
#   output$tbl_full <- render_dt2({
#     dt2(iris, options = list(
#       pageLength = 5,
#       layout = list(bottomEnd = list(paging = list(
#         firstLast = TRUE, previousNext = TRUE, numbers = TRUE
#       )))
#     ))
#   })
#   output$tbl_none <- render_dt2({
#     dt2(iris[1:15, ], options = list(paging = FALSE))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Inline Inputs"),
#     card_body(dt2_output("tbl"))
#   ),
#   card(
#     card_header("Events"),
#     card_body(verbatimTextOutput("events"))
#   )
# )
# 
# server <- function(input, output, session) {
#   df <- data.frame(
#     name   = c("Alice", "Bob", "Carol", "Dave"),
#     score  = c(95, 82, 91, 78),
#     select = NA,
#     action = NA
#   )
# 
#   output$tbl <- render_dt2({
#     opts <- list(columns = names(df), pageLength = 10)
#     opts <- dt2_col_checkbox(opts, "select")
#     opts <- dt2_col_button(opts, "action", label = "View")
#     dt2(df, options = opts)
#   })
# 
#   event_log <- reactiveVal("")
# 
#   observeEvent(input$tbl_row_check, {
#     info <- input$tbl_row_check
#     msg <- paste0("Checkbox row ", info$row, ": ", info$value, "\n")
#     event_log(paste0(event_log(), msg))
#   })
# 
#   observeEvent(input$tbl_row_button, {
#     info <- input$tbl_row_button
#     msg <- paste0("Button clicked row ", info$row, "\n")
#     event_log(paste0(event_log(), msg))
#   })
# 
#   output$events <- renderText(event_log())
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   card(
#     card_header("Server-Side Processing \u2014 100k rows"),
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   large_data <- data.frame(
#     id    = seq_len(100000),
#     value = round(rnorm(100000), 4),
#     group = sample(LETTERS[1:5], 100000, replace = TRUE)
#   )
# 
#   dt2_bind_server("tbl", large_data)
# 
#   output$tbl <- render_dt2({
#     dt2(large_data, options = list(
#       columns      = names(large_data),
#       pageLength   = 25,
#       server_side  = TRUE,
#       language     = list(info = "Showing _START_ to _END_ of _TOTAL_ rows")
#     ))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# ui <- page_sidebar(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   title = "DT2 + bslib",
#   sidebar = sidebar(
#     selectInput("dataset", "Dataset",
#                 c("iris", "mtcars", "airquality")),
#     selectInput("theme_preset", "Theme preset",
#                 c("default", "clean", "minimal", "compact"))
#   ),
#   card(
#     card_header("Table"),
#     card_body(dt2_output("tbl"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
#     data <- switch(input$dataset,
#       iris       = iris,
#       mtcars     = mtcars,
#       airquality = airquality
#     )
#     dt2(data,
#         theme = input$theme_preset,
#         options = list(pageLength = 10))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(shiny)
# library(bslib)
# library(DT2)
# 
# my_theme <- dt2_theme("clean",
#   compact      = TRUE,
#   font_scale   = 0.85,
#   button_class = "btn btn-sm btn-outline-dark"
# )
# 
# ui <- page_fillable(
#   theme = bs_theme(version = 5, bootswatch = "flatly"),
#   padding = "1rem",
#   layout_columns(
#     col_widths = c(6, 6),
#     card(
#       card_header("iris"),
#       card_body(dt2_output("tbl1"))
#     ),
#     card(
#       card_header("mtcars"),
#       card_body(dt2_output("tbl2"))
#     )
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl1 <- render_dt2({
#     dt2(iris, theme = my_theme, options = dt2_use_buttons(
#       buttons = c("copy", "csv")
#     ))
#   })
# 
#   output$tbl2 <- render_dt2({
#     dt2(mtcars, theme = my_theme, options = list(pageLength = 15))
#   })
# }
# 
# shinyApp(ui, server)

## -----------------------------------------------------------------------------
# library(jsonlite)
# library(dplyr)
# library(tibble)
# library(lubridate)
# 
# # ── Sample data (57 employees) ────────────────────────────────────────────────
# json_url <- "https://raw.githubusercontent.com/StrategicProjects/DT2/main/inst/examples/employees.json"
# # For offline use, the same JSON is bundled in inst/examples/employees.json
# 
# json_txt <- '{
#   "data": [
#     {"name":"Tiger Nixon","position":"System Architect","salary":"320800","start_date":"2011-04-25","office":"Edinburgh","extn":"5421"},
#     {"name":"Garrett Winters","position":"Accountant","salary":"170750","start_date":"2011-07-25","office":"Tokyo","extn":"8422"},
#     {"name":"Ashton Cox","position":"Junior Technical Author","salary":"86000","start_date":"2009-01-12","office":"San Francisco","extn":"1562"},
#     {"name":"Cedric Kelly","position":"Senior JavaScript Developer","salary":"433060","start_date":"2012-03-29","office":"Edinburgh","extn":"6224"},
#     {"name":"Airi Satou","position":"Accountant","salary":"162700","start_date":"2008-11-28","office":"Tokyo","extn":"5407"},
#     {"name":"Brielle Williamson","position":"Integration Specialist","salary":"372000","start_date":"2012-12-02","office":"New York","extn":"4804"},
#     {"name":"Herrod Chandler","position":"Sales Assistant","salary":"137500","start_date":"2012-08-06","office":"San Francisco","extn":"9608"},
#     {"name":"Rhona Davidson","position":"Integration Specialist","salary":"327900","start_date":"2010-10-14","office":"Tokyo","extn":"6200"},
#     {"name":"Colleen Hurst","position":"JavaScript Developer","salary":"205500","start_date":"2009-09-15","office":"San Francisco","extn":"2360"},
#     {"name":"Sonya Frost","position":"Software Engineer","salary":"103600","start_date":"2008-12-13","office":"Edinburgh","extn":"1667"}
#   ]
# }'
# 
# df <- fromJSON(json_txt, flatten = TRUE)$data %>%
#   as_tibble() %>%
#   mutate(
#     salary     = as.numeric(salary),
#     extn       = as.integer(extn),
#     start_date = ymd(start_date)
#   )
# 
# # ── Shiny App ─────────────────────────────────────────────────────────────────
# library(shiny)
# library(bslib)
# library(DT2)
# library(htmlwidgets)
# 
# ui <- page_sidebar(
#   theme = bs_theme(version = 5, bootswatch = "spacelab"),
#   title = "DT2 — Complete Example",
#   sidebar = sidebar(
#     h5("Features"),
#     tags$ul(
#       tags$li("ColumnControl (order + search dropdowns)"),
#       tags$li("Export buttons with spacer"),
#       tags$li("Custom JS renderers"),
#       tags$li("pt-BR translation")
#     )
#   ),
#   # Flag sprites
#   tags$head(
#     tags$link(
#       rel = "stylesheet", type = "text/css",
#       href = "https://cdn.jsdelivr.net/gh/lafeber/world-flags-sprite/stylesheets/flags32-both.css"
#     ),
#     tags$style(HTML("
#       .f32 .flag { display:inline-block; width:32px; height:32px;
#                     vertical-align:middle; margin-right:6px; }
#       table.dataTable tbody td { vertical-align: middle; }
#     "))
#   ),
#   card(
#     card_header("Employee Table"),
#     card_body(dt2_output("tbl", height = "auto"))
#   )
# )
# 
# server <- function(input, output, session) {
#   output$tbl <- render_dt2({
# 
#     # ── JS Renderers ──────────────────────────────────────────
#     office_js <- JS("
#       function(data, type) {
#         if (type !== 'display') return data;
#         var cc = {Argentina:'ar', Edinburgh:'_Scotland', London:'_England',
#                   'New York':'us', 'San Francisco':'us', Sydney:'au', Tokyo:'jp'};
#         var flag = cc[data] || '';
#         return '<span class=\"flag ' + flag + '\"></span> ' + data;
#       }
#     ")
# 
#     salary_js <- JS("
#       (function() {
#         var nfmt = DataTable.render.number('.', ',', 2, 'R$ ');
#         return function(data, type) {
#           var txt = nfmt.display(data);
#           if (type !== 'display') return txt;
#           var c = data < 250000 ? 'red' : data < 500000 ? 'orange' : 'green';
#           return '<span style=\"color:' + c + '\">' + txt + '</span>';
#         };
#       })()
#     ")
# 
#     extn_js <- JS("
#       function(data, type) {
#         return type === 'display'
#           ? '<progress value=\"' + data + '\" max=\"9999\"></progress>'
#           : data;
#       }
#     ")
# 
#     # ── Options ───────────────────────────────────────────────
#     opts <- list(
#       pageLength = 10,
#       lengthMenu = c(10, 25, -1),
#       columns    = names(df),
# 
#       layout = list(
#         topStart = "pageLength",
#         topEnd   = list(
#           buttons = list(
#             list(extend = "copyHtml5", text = "Copiar"),
#             list(extend = "csvHtml5"),
#             list(extend = "excelHtml5"),
#             list(extend = "spacer", style = "bar"),
#             list(extend = "colvis", text = "Colunas")
#           ),
#           search = list(placeholder = "")
#         ),
#         bottomEnd = "paging"
#       ),
# 
#       columnControl = list(
#         target  = 0,
#         content = list("order", "searchDropdown", list(
#           list(extend = "orderAsc", text = "Ordem crescente"),
#           list(extend = "orderDesc", text = "Ordem decrescente"),
#           "spacer",
#           list(extend = "colVisDropdown", text = "Selecionar colunas")
#         ))
#       ),
#       ordering = list(indicators = FALSE, handler = FALSE),
# 
#       columnDefs = list(
#         list(targets = which(names(df) == "office") - 1L,
#              className = "f32", render = office_js),
#         list(targets = which(names(df) == "salary") - 1L,
#              className = "dt-body-right", render = salary_js),
#         list(targets = which(names(df) == "extn") - 1L,
#              render = extn_js)
#       ),
# 
#       language = list(
#         lengthMenu   = "Mostrar _MENU_",
#         search       = "Buscar",
#         info         = "Mostrando _START_ a _END_ de _TOTAL_ registros",
#         infoEmpty    = "Nenhum registro",
#         zeroRecords  = "Nenhum registro encontrado",
#         emptyTable   = "Nenhum dado disponível",
#         decimal      = ",", thousands = ".", infoThousands = ".",
#         lengthLabels = list(`10` = "10", `25` = "25", `-1` = "Todas"),
#         paginate     = list(first="«", previous="‹", `next`="›", last="»"),
#         buttons = list(
#           copyTitle   = "Copiado!",
#           copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
#         ),
#         columnControl = list(
#           orderAsc = "Crescente", orderDesc = "Decrescente",
#           searchDropdown = "Pesquisar", colVisDropdown = "Colunas",
#           searchClear = "Limpar",
#           search = list(
#             text   = list(contains="Contém", starts="Começa por",
#                           ends="Termina em", equal="Igual a"),
#             number = list(greater="Maior que", less="Menor que",
#                           equal="Igual a")
#           )
#         )
#       )
#     )
# 
#     dt2(df,
#         compact = TRUE, striped = TRUE, hover = TRUE,
#         font_scale = 0.85, responsive = FALSE,
#         options = opts)
#   })
# }
# 
# shinyApp(ui, server)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 25,
  ordering   = TRUE,
  language   = list(
    search     = "Filter:",
    lengthMenu = "Show _MENU_ entries"
  ),
  columnDefs = list(
    list(targets = 0, visible = FALSE),
    list(targets = c(1, 2), className = "text-center")
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart = list(search = list(placeholder = "Filter...")),
    topEnd   = "pageLength"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 10,
  searching = FALSE,
  layout = list(
    topStart  = NULL,      # no page length selector
    topEnd    = NULL,      # no search box
    bottomStart = NULL,    # no info
    bottomEnd = "paging"   # only pagination
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart  = list("pageLength", "info"),
    topEnd    = "search",
    bottomEnd = "paging"
  )
))

## -----------------------------------------------------------------------------
# Buttons on the left
dt2(iris[1:15, ], options = list(
  buttons = list("copy", "csv", "excel"),
  layout = list(
    topStart  = "buttons",
    topEnd    = "search",
    bottomEnd = "paging"
  )
))

## -----------------------------------------------------------------------------
# Buttons on the bottom
dt2(iris[1:15, ], options = list(
  buttons = list("copy", "csv"),
  layout = list(
    topEnd      = "search",
    bottomStart = "buttons",
    bottomEnd   = "paging"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topEnd = list(search = list(
      placeholder = "Type to filter...",
      text = "Search:"
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(mtcars[1:20, ], options = list(
  pageLength = 10,
  buttons = list(
    list(extend = "collection", text = "Export \u25BC",
         buttons = list("copyHtml5", "csvHtml5", "excelHtml5")),
    list(extend = "colvis", text = "Columns")
  ),
  layout = list(
    topStart    = "buttons",
    topEnd      = list(search = list(placeholder = "Filter...")),
    bottomStart = "info",
    bottomEnd   = "paging"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  createdRow = htmlwidgets::JS("
    function(row, data, dataIndex) {
      if (data['Sepal.Length'] > 5) {
        row.style.backgroundColor = '#fff3cd';
      }
    }
  ")
))

## -----------------------------------------------------------------------------
dt2(mtcars[1:10, c("mpg", "hp", "wt")], options = list(
  columnDefs = list(
    list(
      targets = 0,
      render = htmlwidgets::JS("DataTable.render.number('.', ',', 1, '', ' mpg')")
    ),
    list(
      targets = 1,
      render = htmlwidgets::JS("DataTable.render.number(',', '.', 0, '', ' hp')")
    )
  )
))

## -----------------------------------------------------------------------------
progress_render <- htmlwidgets::JS("
  function(data, type, row, meta) {
    if (type !== 'display') return data;
    var pct = Math.min(100, Math.max(0, parseFloat(data)));
    var color = pct > 70 ? '#198754' : (pct > 40 ? '#ffc107' : '#dc3545');
    return '<div style=\"background:#eee;border-radius:4px;overflow:hidden\">' +
           '<div style=\"width:' + pct + '%;background:' + color +
           ';height:14px;border-radius:4px\"></div></div>';
  }
")

df <- data.frame(
  task     = c("Design", "Backend", "Testing", "Deploy"),
  progress = c(85, 60, 30, 95)
)

dt2(df, options = list(
  pageLength = 10,
  columnDefs = list(
    list(targets = 1, render = progress_render)
  )
))

## -----------------------------------------------------------------------------
dt2(iris[1:10, ], options = list(
  pageLength = 5,
  language = list(
    search       = "Buscar:",
    lengthMenu   = "Mostrar _MENU_ registros",
    info         = "Mostrando _START_ a _END_ de _TOTAL_",
    paginate     = list(
      first    = "<<",
      previous = "<",
      `next`   = ">",
      last     = ">>"
    ),
    zeroRecords  = "Nenhum registro encontrado",
    emptyTable   = "Tabela vazia"
  )
))

library(jsonlite)
library(dplyr)
library(tibble)
library(lubridate)

# ── Dados ─────────────────────────────────────────────────────────────────────
json_txt <- '{
  "data": [
    {"name":"Tiger Nixon","position":"System Architect","salary":"320800","start_date":"2011-04-25","office":"Edinburgh","extn":"5421"},
    {"name":"Garrett Winters","position":"Accountant","salary":"170750","start_date":"2011-07-25","office":"Tokyo","extn":"8422"},
    {"name":"Ashton Cox","position":"Junior Technical Author","salary":"86000","start_date":"2009-01-12","office":"San Francisco","extn":"1562"},
    {"name":"Cedric Kelly","position":"Senior JavaScript Developer","salary":"433060","start_date":"2012-03-29","office":"Edinburgh","extn":"6224"},
    {"name":"Airi Satou","position":"Accountant","salary":"162700","start_date":"2008-11-28","office":"Tokyo","extn":"5407"},
    {"name":"Brielle Williamson","position":"Integration Specialist","salary":"372000","start_date":"2012-12-02","office":"New York","extn":"4804"},
    {"name":"Herrod Chandler","position":"Sales Assistant","salary":"137500","start_date":"2012-08-06","office":"San Francisco","extn":"9608"},
    {"name":"Rhona Davidson","position":"Integration Specialist","salary":"327900","start_date":"2010-10-14","office":"Tokyo","extn":"6200"},
    {"name":"Colleen Hurst","position":"JavaScript Developer","salary":"205500","start_date":"2009-09-15","office":"San Francisco","extn":"2360"},
    {"name":"Sonya Frost","position":"Software Engineer","salary":"103600","start_date":"2008-12-13","office":"Edinburgh","extn":"1667"},
    {"name":"Jena Gaines","position":"Office Manager","salary":"90560","start_date":"2008-12-19","office":"London","extn":"3814"},
    {"name":"Quinn Flynn","position":"Support Lead","salary":"342000","start_date":"2013-03-03","office":"Edinburgh","extn":"9497"},
    {"name":"Charde Marshall","position":"Regional Director","salary":"470600","start_date":"2008-10-16","office":"San Francisco","extn":"6741"},
    {"name":"Haley Kennedy","position":"Senior Marketing Designer","salary":"313500","start_date":"2012-12-18","office":"London","extn":"3597"},
    {"name":"Tatyana Fitzpatrick","position":"Regional Director","salary":"385750","start_date":"2010-03-17","office":"London","extn":"1965"},
    {"name":"Michael Silva","position":"Marketing Designer","salary":"198500","start_date":"2012-11-27","office":"London","extn":"1581"},
    {"name":"Paul Byrd","position":"Chief Financial Officer (CFO)","salary":"725000","start_date":"2010-06-09","office":"New York","extn":"3059"},
    {"name":"Gloria Little","position":"Systems Administrator","salary":"237500","start_date":"2009-04-10","office":"New York","extn":"1721"},
    {"name":"Bradley Greer","position":"Software Engineer","salary":"132000","start_date":"2012-10-13","office":"London","extn":"2558"},
    {"name":"Dai Rios","position":"Personnel Lead","salary":"217500","start_date":"2012-09-26","office":"Edinburgh","extn":"2290"},
    {"name":"Jenette Caldwell","position":"Development Lead","salary":"345000","start_date":"2011-09-03","office":"New York","extn":"1937"},
    {"name":"Yuri Berry","position":"Chief Marketing Officer (CMO)","salary":"675000","start_date":"2009-06-25","office":"New York","extn":"6154"},
    {"name":"Caesar Vance","position":"Pre-Sales Support","salary":"106450","start_date":"2011-12-12","office":"New York","extn":"8330"},
    {"name":"Doris Wilder","position":"Sales Assistant","salary":"85600","start_date":"2010-09-20","office":"Sydney","extn":"3023"},
    {"name":"Angelica Ramos","position":"Chief Executive Officer (CEO)","salary":"1200000","start_date":"2009-10-09","office":"London","extn":"5797"},
    {"name":"Gavin Joyce","position":"Developer","salary":"92575","start_date":"2010-12-22","office":"Edinburgh","extn":"8822"},
    {"name":"Jennifer Chang","position":"Regional Director","salary":"357650","start_date":"2010-11-14","office":"Argentina","extn":"9239"},
    {"name":"Brenden Wagner","position":"Software Engineer","salary":"206850","start_date":"2011-06-07","office":"San Francisco","extn":"1314"},
    {"name":"Fiona Green","position":"Chief Operating Officer (COO)","salary":"850000","start_date":"2010-03-11","office":"San Francisco","extn":"2947"},
    {"name":"Shou Itou","position":"Regional Marketing","salary":"163000","start_date":"2011-08-14","office":"Tokyo","extn":"8899"},
    {"name":"Michelle House","position":"Integration Specialist","salary":"95400","start_date":"2011-06-02","office":"Sydney","extn":"2769"},
    {"name":"Suki Burks","position":"Developer","salary":"114500","start_date":"2009-10-22","office":"London","extn":"6832"},
    {"name":"Prescott Bartlett","position":"Technical Author","salary":"145000","start_date":"2011-05-07","office":"London","extn":"3606"},
    {"name":"Gavin Cortez","position":"Team Leader","salary":"235500","start_date":"2008-10-26","office":"San Francisco","extn":"2860"},
    {"name":"Martena Mccray","position":"Post-Sales support","salary":"324050","start_date":"2011-03-09","office":"Edinburgh","extn":"8240"},
    {"name":"Unity Butler","position":"Marketing Designer","salary":"85675","start_date":"2009-12-09","office":"San Francisco","extn":"5384"},
    {"name":"Howard Hatfield","position":"Office Manager","salary":"164500","start_date":"2008-12-16","office":"San Francisco","extn":"7031"},
    {"name":"Hope Fuentes","position":"Secretary","salary":"109850","start_date":"2010-02-12","office":"San Francisco","extn":"6318"},
    {"name":"Vivian Harrell","position":"Financial Controller","salary":"452500","start_date":"2009-02-14","office":"San Francisco","extn":"9422"},
    {"name":"Timothy Mooney","position":"Office Manager","salary":"136200","start_date":"2008-12-11","office":"London","extn":"7580"},
    {"name":"Jackson Bradshaw","position":"Director","salary":"645750","start_date":"2008-09-26","office":"New York","extn":"1042"},
    {"name":"Olivia Liang","position":"Support Engineer","salary":"234500","start_date":"2011-02-03","office":"Argentina","extn":"2120"},
    {"name":"Bruno Nash","position":"Software Engineer","salary":"163500","start_date":"2011-05-03","office":"London","extn":"6222"},
    {"name":"Sakura Yamamoto","position":"Support Engineer","salary":"139575","start_date":"2009-08-19","office":"Tokyo","extn":"9383"},
    {"name":"Thor Walton","position":"Developer","salary":"98540","start_date":"2013-08-11","office":"New York","extn":"8327"},
    {"name":"Finn Camacho","position":"Support Engineer","salary":"87500","start_date":"2009-07-07","office":"San Francisco","extn":"2927"},
    {"name":"Serge Baldwin","position":"Data Coordinator","salary":"138575","start_date":"2012-04-09","office":"Argentina","extn":"8352"},
    {"name":"Zenaida Frank","position":"Software Engineer","salary":"125250","start_date":"2010-01-04","office":"New York","extn":"7439"},
    {"name":"Zorita Serrano","position":"Software Engineer","salary":"115000","start_date":"2012-06-01","office":"San Francisco","extn":"4389"},
    {"name":"Jennifer Acosta","position":"Junior JavaScript Developer","salary":"75650","start_date":"2013-02-01","office":"Edinburgh","extn":"3431"},
    {"name":"Cara Stevens","position":"Sales Assistant","salary":"145600","start_date":"2011-12-06","office":"New York","extn":"3990"},
    {"name":"Hermione Butler","position":"Regional Director","salary":"356250","start_date":"2011-03-21","office":"London","extn":"1016"},
    {"name":"Lael Greer","position":"Systems Administrator","salary":"103500","start_date":"2009-02-27","office":"London","extn":"6733"},
    {"name":"Jonas Alexander","position":"Developer","salary":"86500","start_date":"2010-07-14","office":"San Francisco","extn":"8196"},
    {"name":"Shad Decker","position":"Regional Director","salary":"183000","start_date":"2008-11-13","office":"Edinburgh","extn":"6373"},
    {"name":"Michael Bruce","position":"JavaScript Developer","salary":"183000","start_date":"2011-06-27","office":"Argentina","extn":"5384"},
    {"name":"Donna Snider","position":"Customer Support","salary":"112000","start_date":"2011-01-25","office":"New York","extn":"4226"}
  ]
}'

df <- fromJSON(json_txt, flatten = TRUE)$data %>%
  as_tibble() %>%
  mutate(
    salary     = as.numeric(salary),
    extn       = as.integer(extn),
    start_date = ymd(start_date)
  )

# ── App ───────────────────────────────────────────────────────────────────────
library(shiny)
library(bslib)
library(DT2)
library(htmlwidgets)

ui <- page_sidebar(
  theme = bs_theme(version = 5, bootswatch = "spacelab"),
  title = "DT2 — Exemplo Completo",
  sidebar = sidebar(
    h5("Ajustes de layout"),
    p("Demonstra: ColumnControl, botões agrupados com spacer, ",
      "renderers JS customizados, bandeiras e paginação traduzida.")
  ),
  # CSS das bandeiras
  tags$head(
    tags$link(
      rel = "stylesheet", type = "text/css",
      href = "https://cdn.jsdelivr.net/gh/lafeber/world-flags-sprite/stylesheets/flags32-both.css"
    ),
    tags$style(HTML("
      .f32 .flag { display:inline-block; width:32px; height:32px;
                    vertical-align:middle; margin-right:6px; }
      table.dataTable tbody td { vertical-align: middle; }
    "))
  ),
  card(
    card_header("DT2"),
    card_body(dt2_output("example", height = "auto"))
  ),
  card(
    card_header("Estado recebido do cliente (debug)"),
    card_body(verbatimTextOutput("state"))
  )
)

server <- function(input, output, session) {

  output$example <- render_dt2({

    # ── Renderers JS (assinatura: data, type, row, meta) ──────────────────────

    position_js <- JS("
      function(data, type, row, meta) {
        if (type === 'display') {
          var link = 'https://datatables.net';
          if (data && data[0] < 'H')      link = 'https://cloudtables.com';
          else if (data && data[0] < 'S')  link = 'https://editor.datatables.net';
          return '<a href=\"' + link + '\" target=\"_blank\" rel=\"noopener\">' + data + '</a>';
        }
        return data;
      }
    ")

    office_js <- JS("
      function(data, type, row, meta) {
        if (type === 'display') {
          var country = '';
          switch (data) {
            case 'Argentina':     country = 'ar'; break;
            case 'Edinburgh':     country = '_Scotland'; break;
            case 'London':        country = '_England'; break;
            case 'New York':
            case 'San Francisco': country = 'us'; break;
            case 'Sydney':        country = 'au'; break;
            case 'Tokyo':         country = 'jp'; break;
          }
          return '<span class=\"flag ' + country + '\"></span> ' + data;
        }
        return data;
      }
    ")

    extn_js <- JS("
      function(data, type, row, meta) {
        return (type === 'display')
          ? '<progress value=\"' + data + '\" max=\"9999\"></progress>'
          : data;
      }
    ")

    salary_js <- JS("
      (function() {
        var nfmt = DataTable.render.number('.', ',', 2, 'R$ ');
        return function(data, type, row, meta) {
          var number = nfmt.display(data);
          if (type === 'display') {
            var color = (data < 250000) ? 'red' : (data < 500000 ? 'orange' : 'green');
            return '<span style=\"color:' + color + '\">' + number + '</span>';
          }
          return number;
        };
      })()
    ")

    # ── Options ───────────────────────────────────────────────────────────────

    opts <- list(
      pageLength   = 10,
      lengthChange = TRUE,
      lengthMenu   = c(10, 25, 50, -1),

      # Layout v2: posiciona elementos livremente
      layout = list(
        topStart = "pageLength",
        topEnd   = list(
          buttons = list(
            list(extend = "copyHtml5", text = "Copiar"),
            list(extend = "csvHtml5"),
            list(extend = "excelHtml5", title = "Dados exportados"),
            list(extend = "pdfHtml5",   title = "Dados exportados"),
            list(extend = "print", text = "Imprimir",
                 title = "Dados exportados", messageTop = ""),
            # Spacer visual separando export de colunas
            list(extend = "spacer", style = "bar"),
            list(extend = "colvis", text = "Colunas")
          ),
          search = list(placeholder = "")
        ),
        bottomEnd = list(paging = list(
          firstLast    = TRUE,
          previousNext = TRUE,
          numbers      = FALSE
        ))
      ),

      # ColumnControl: menus dropdown no header
      columnControl = list(
        target  = 0,
        content = list(
          "order",
          "searchDropdown",
          list(
            list(extend = "orderAsc",        text = "Ordem crescente"),
            list(extend = "orderDesc",       text = "Ordem decrescente"),
            "spacer",
            list(extend = "colVisDropdown",  text = "Selecionar colunas")
          )
        )
      ),

      # Recomendado quando ColumnControl gerencia ordering
      ordering = list(indicators = FALSE, handler = FALSE),

      # Mapeamento colunas (DT2 aceita vetor de nomes)
      columns = names(df),

      # Renderers por coluna (targets = índice 0-based)
      columnDefs = list(
        list(targets = which(names(df) == "position") - 1L,
             render  = position_js),
        list(targets = which(names(df) == "office") - 1L,
             className = "f32",
             render    = office_js),
        list(targets = which(names(df) == "start_date") - 1L,
             className = "dt-body-center dt-left"),
        list(targets = which(names(df) == "extn") - 1L,
             className = "dt-left",
             render    = extn_js),
        list(targets = which(names(df) == "salary") - 1L,
             className = "dt-body-right dt-left",
             render    = salary_js)
      ),

      # Tradução pt-BR
      language = list(
        lengthMenu   = "Mostrar&nbsp; _MENU_",
        lengthLabels = list(
          `10` = "10", `25` = "25", `50` = "50", `-1` = "Todas"
        ),
        search       = "Buscar",
        paginate     = list(
          first = "&laquo;", previous = "\u2039",
          `next` = "\u203A", last = "&raquo;"
        ),
        info          = "Mostrando _START_ a _END_ de _TOTAL_ registros",
        infoEmpty     = "Mostrando 0 a 0 de 0 registros",
        infoFiltered  = "(filtrado de _MAX_ registros no total)",
        zeroRecords   = "Nenhum registro encontrado",
        emptyTable    = "Nenhum dado dispon\u00edvel",
        loadingRecords = "Carregando dados...",
        decimal       = ",",
        thousands     = ".",
        infoThousands = ".",
        buttons = list(
          copyTitle   = "Copiado para a \u00e1rea de transfer\u00eancia",
          copyKeys    = paste0(
            "Pressione <i>Ctrl</i> ou <i>\u2318</i> + <i>C</i> para copiar ",
            "os dados da tabela.<br><br>Para cancelar, clique nesta mensagem ",
            "ou pressione Esc."
          ),
          copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
        ),
        columnControl = list(
          buttons       = list(searchClear = "Limpar pesquisa"),
          colVis        = "Visibilidade da coluna",
          colVisDropdown = "Visibilidade da coluna",
          dropdown      = "Mostrar mais...",
          list          = list(
            all = "Todos", empty = "Vazio", none = "Nenhum",
            search = "Pesquisar..."
          ),
          orderAddAsc   = "Adicionar \u00e0 ordem crescente",
          orderAddDesc  = "Adicionar \u00e0 ordem decrescente",
          orderAsc      = "Ordem crescente",
          orderClear    = "Remover ordena\u00e7\u00e3o",
          orderDesc     = "Ordem decrescente",
          orderRemove   = "Remover ordena\u00e7\u00e3o",
          reorder       = "Reordenar",
          reorderLeft   = "Mover para a esquerda",
          reorderRight  = "Mover para a direita",
          search = list(
            datetime = list(
              empty = "Vazio", equal = "Igual a", greater = "Posterior a",
              less = "Anterior a", notEmpty = "N\u00e3o est\u00e1 vazio",
              notEqual = "Diferente de"
            ),
            number = list(
              empty = "Vazio", equal = "Igual a", greater = "Maior que",
              greaterOrEqual = "Maior ou igual a", less = "Menor que",
              lessOrEqual = "Menor ou igual a",
              notEmpty = "N\u00e3o est\u00e1 vazio", notEqual = "Diferente de"
            ),
            text = list(
              contains = "Cont\u00e9m", empty = "Vazio", ends = "Termina em",
              equal = "Igual a", notContains = "N\u00e3o cont\u00e9m",
              notEmpty = "N\u00e3o est\u00e1 vazio", notEqual = "Diferente de",
              starts = "Come\u00e7a por"
            )
          ),
          searchClear     = "Limpar pesquisa",
          searchDropdown  = "Pesquisar"
        )
      )
    )

    # ── Cria o widget ─────────────────────────────────────────────────────────
    # Styling via parâmetros diretos do dt2() (API v2)
    dt2(
      data       = df,
      compact    = TRUE,
      striped    = TRUE,
      hover      = TRUE,
      font_scale = 0.85,
      responsive = FALSE,   # responsive pode conflitar com renderers customizados
      options    = opts
    )
  })

  # ── Debug: mostra estado da tabela ──────────────────────────────────────────
  output$state <- renderPrint({
    s <- input$example_state
    if (is.null(s)) return("(interaja com a tabela)")
    cat("Reason:", s$reason, "\n")
    if (!is.null(s$page))
      cat("Page:",  s$page$page + 1, "of", s$page$pages, "\n")
    if (!is.null(s$search) && nzchar(s$search))
      cat("Search:", s$search, "\n")
    if (!is.null(s$order))
      cat("Order:", paste(s$order, collapse = ", "), "\n")
  })
}

shinyApp(ui, server)

#' Format numeric columns (DataTables renderer: number)
#'
#' Add a number renderer to one or more columns using
#' DataTables' built-in \code{DataTable.render.number}.
#'
#' @param options List of options (returned, with \code{columnDefs} updated).
#' @param col_specs Column names or 1-based indices to format.
#' @param thousands Thousands separator (character or \code{NULL} for auto).
#' @param decimal Decimal separator (character or \code{NULL} for auto).
#' @param digits Number of decimal places.
#' @param prefix,prefix_right String to prepend/append (e.g., currency symbol).
#'
#' @return Modified \code{options}.
#' @examples
#' opts <- list(columns = names(iris))
#' opts <- dt2_format_number(opts, "Sepal.Length", thousands = ".", decimal = ",",
#'                           digits = 2, prefix = "", prefix_right = "")
#' @export
dt2_format_number <- function(options = list(), col_specs,
                              thousands = NULL, decimal = NULL,
                              digits = 0, prefix = "", prefix_right = "") {
  col_specs <- .dt2_name_to_idx(col_specs, options)
  # DataTables v2 sugere DataTable.render.number(...)
  # https://datatables.net/manual/data/renderers
  js <- htmlwidgets::JS(
    sprintf("DataTable.render.number(%s,%s,%d,%s,%s)",
            .dt2_js_str(thousands), .dt2_js_str(decimal),
            as.integer(digits),
            .dt2_js_str(prefix), .dt2_js_str(prefix_right))
  )
  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Format date/time columns (DataTables renderer: datetime)
#'
#' Use DataTables' built-in \code{DataTable.render.datetime} to transform
#' date/time strings for display (and preserve sortability).
#'
#' @param options List of options (returned with \code{columnDefs} updated).
#' @param col_specs Column names or 1-based indices.
#' @param from Input format (e.g., \code{"YYYY-MM-DD"} or ISO8601 by default).
#' @param to Output format (e.g., \code{"DD/MM/YYYY"}).
#' @param locale Optional locale (e.g., \code{"pt-BR"}).
#' @param def Optional default output if input is invalid.
#'
#' @return Modified \code{options}.
#' @examples
#' opts <- list(columns = c("when", "val"))
#' opts <- dt2_format_datetime(opts, "when", from = "YYYY-MM-DD",
#'                             to = "DD/MM/YYYY", locale = "pt-BR")
#' @export
dt2_format_datetime <- function(options = list(), col_specs,
                                from = NULL, to = "DD/MM/YYYY",
                                locale = NULL, def = NULL) {
  col_specs <- .dt2_name_to_idx(col_specs, options)

  args <- c(
    .dt2_js_str(from,   "undefined"),
    .dt2_js_str(to,     "undefined"),
    .dt2_js_str(locale, "undefined"),
    .dt2_js_str(def,    "undefined")
  )
  # ver docs: https://datatables.net/plug-ins/dataRender/datetime  + manual de renderers
  js <- htmlwidgets::JS(sprintf("DataTable.render.datetime(%s)", paste(args, collapse = ", ")))
  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Attach a raw JS render function to columns
#'
#' Provide a custom JS renderer for one or more columns. Use this when
#' you need fine control over \code{columns.render}, including returning
#' different outputs based on \code{type} (display/sort/filter/type).
#'
#' @param options List returned, with \code{columnDefs} appended.
#' @param col_specs Column names or indices.
#' @param js_render A \code{htmlwidgets::JS()} function of signature
#'   \code{function(data, type, row, meta) { ... }}.
#'
#' @return Modified \code{options}.
#' @seealso \url{https://datatables.net/reference/option/columns.render}
#' @export
dt2_cols_render_js <- function(options = list(), col_specs, js_render) {
  col_specs <- .dt2_name_to_idx(col_specs, options)
  stopifnot(inherits(js_render, "JS_EVAL"))
  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js_render))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Orthogonal render (display/sort/filter/type) per column
#'
#' Supply different renderers for each orthogonal data request. Pass an
#' object with keys \code{display}, \code{sort}, \code{filter}, \code{type}
#' (all optional). Each value must be a JS function.
#'
#' @param options Options list to modify.
#' @param col_specs Column names or indices.
#' @param display Optional JS renderer for UI display.
#' @param sort Optional JS renderer used for ordering.
#' @param filter Optional JS renderer used for searching.
#' @param type Optional JS renderer used for type detection.
#'
#' @return Modified \code{options}.
#' @examples
#' opts <- list(columns = names(iris))
#' opts <- dt2_cols_render_orthogonal(
#'   opts, "Sepal.Length",
#'   display = htmlwidgets::JS("function(d,t,row,meta){ return d + ' cm'; }"),
#'   sort    = htmlwidgets::JS("function(d,t,row,meta){ return parseFloat(d); }")
#' )
#' @export
dt2_cols_render_orthogonal <- function(options = list(), col_specs,
                                       display = NULL, sort = NULL,
                                       filter = NULL, type = NULL) {
  col_specs <- .dt2_name_to_idx(col_specs, options)


  # Build an {display, sort, filter, type} object with the supplied parts
  parts <- c(
    if (!is.null(display)) sprintf("display:%s", as.character(display)) else NULL,
    if (!is.null(sort))    sprintf("sort:%s",    as.character(sort))    else NULL,
    if (!is.null(filter))  sprintf("filter:%s",  as.character(filter))  else NULL,
    if (!is.null(type))    sprintf("type:%s",    as.character(type))    else NULL
  )
  stopifnot(length(parts) > 0)
  js <- htmlwidgets::JS(sprintf("{%s}", paste(parts, collapse = ",")))

  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}


#' Abbreviate large numbers with fixed decimals (k / M / B)
#'
#' Adds a `columns.render` function that displays numbers as 1.2k, 3.4M, etc.
#' This renderer **lets you control** the number of decimal places via `digits`.
#' Use this when you want a fixed, compact style independent of locale rules.
#'
#' @param options A DataTables options list to be modified.
#' @param col_specs Column names or 1-based indices to format.
#' @param digits Integer, decimal places for the abbreviated display (default 1).
#' @param locale Optional BCP-47 locale string (e.g. "pt-BR"). If provided,
#'   the non-abbreviated part uses `toLocaleString(locale)` for grouping.
#'
#' @return The modified `options` list.
#' @export
#' @examples
#' opts <- list(columns = names(mtcars))
#' opts <- dt2_format_number_abbrev(opts, c("hp","qsec"), digits = 1, locale = "pt-BR")
dt2_format_number_abbrev <- function(options = list(), col_specs, digits = 1, locale = NULL) {
  col_specs <- .dt2_name_to_idx(col_specs, options)

  # JS renderer: abrevia com k/M/B e aplica toLocaleString(locale) na parte inteira se locale fornecido
  js <- if (is.null(locale) || !nzchar(locale)) {
    htmlwidgets::JS(sprintf(
      "function(d, t, row, meta){
         if (t !== 'display' && t !== 'filter') return d;
         var n = Number(d);
         if (!isFinite(n)) return d;
         var abs = Math.abs(n), sign = n < 0 ? '-' : '';
         function fmt(x){ return x.toFixed(%d); }
         if (abs >= 1e9)  return sign + fmt(abs/1e9) + 'B';
         if (abs >= 1e6)  return sign + fmt(abs/1e6) + 'M';
         if (abs >= 1e3)  return sign + fmt(abs/1e3) + 'k';
         return n.toFixed(%d);
       }", digits, digits))
  } else {
    htmlwidgets::JS(sprintf(
      "function(d, t, row, meta){
         if (t !== 'display' && t !== 'filter') return d;
         var n = Number(d);
         if (!isFinite(n)) return d;
         var abs = Math.abs(n), sign = n < 0 ? '-' : '';
         function fmt(x){ return Number(x.toFixed(%d)).toLocaleString('%s'); }
         if (abs >= 1e9)  return sign + fmt(abs/1e9) + 'B';
         if (abs >= 1e6)  return sign + fmt(abs/1e6) + 'M';
         if (abs >= 1e3)  return sign + fmt(abs/1e3) + 'k';
         return n.toLocaleString('%s', { minimumFractionDigits: %d, maximumFractionDigits: %d });
       }", digits, locale, locale, digits, digits))
  }

  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Format a date/time using DataTables' datetime renderer, with locale
#' @param options Options list (returned modified).
#' @param col_specs Column names or indices to format.
#' @param from Input format (e.g. `'YYYY-MM-DDTHH:mm:ssZ'` or `NULL` for ISO).
#' @param to   Output format (e.g. `'L LTS'`). See moment.js docs.
#' @param locale Locale string, e.g. `'pt-br'`.
#' @return Modified `options`.
#' @export
dt2_format_time_format <- function(options = list(), col_specs,
                                   from = NULL, to = "L", locale = "pt-br") {
  col_specs <- .dt2_name_to_idx(col_specs, options)

  # ativa locale no cliente (fallback para renders que usem moment direto)
  options$`_momentLocale` <- locale

  # DataTables v2: DataTable.render.datetime(from, to, locale)
  renderer_call <- if (is.null(from)) {
    sprintf("DataTable.render.datetime(%s,%s)", .dt2_js_str(to), .dt2_js_str(locale))
  } else {
    sprintf("DataTable.render.datetime(%s,%s,%s)",
            .dt2_js_str(from), .dt2_js_str(to), .dt2_js_str(locale))
  }
  render <- htmlwidgets::JS(renderer_call)

  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = render))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Relative time using moment.fromNow(), with locale
#' @param options list of options (returned updated)
#' @param col_specs names or indices to format
#' @param locale e.g. "pt-br" (requires moment-with-locales)
#' @return The modified `options` list with an updated `columnDefs` entry.
#' @export
dt2_format_time_relative <- function(options = list(), col_specs, locale = "pt-br") {
  col_specs <- .dt2_name_to_idx(col_specs, options)

  # ativa locale no cliente (usado por dt2.js)
  options$`_momentLocale` <- locale

  render_js <- htmlwidgets::JS(
    sprintf("
function(d, t, row, meta){
  if (d == null || d === '') return d;
  try {
    if (window.moment){
      var m = moment(d);
      if (m.isValid()) return m.fromNow(); // uses active locale
    }
  } catch(e){}
  return d;
}"
    )
  )

  cds <- lapply(col_specs, function(i) list(targets = i - 1L, render = render_js))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

# Tests for safe JS string interpolation in the format helpers.
# Previously strings were injected with sprintf("'%s'", x), which produced
# invalid JS when the value itself contained a quote.

test_that(".dt2_js_str produces valid, escaped JS string literals", {
  expect_equal(.dt2_js_str("R$"),        '"R$"')
  expect_equal(.dt2_js_str("a\"b"),      '"a\\"b"')
  expect_equal(.dt2_js_str(NULL),        "null")
  expect_equal(.dt2_js_str(NULL, "undefined"), "undefined")
})

test_that("dt2_format_number safely quotes a prefix containing a quote", {
  o <- dt2_format_number(list(columns = "x"), 1, prefix = "O'Brien $")
  r <- as.character(o$columnDefs[[1]]$render)
  # valid double-quoted JS literal ...
  expect_true(grepl('"O\'Brien $"', r, fixed = TRUE))
  # ... not the broken single-quoted form
  expect_false(grepl("'O'Brien $'", r, fixed = TRUE))
})

test_that("dt2_format_number renders NULL separators as null", {
  o <- dt2_format_number(list(columns = "x"), 1,
                         thousands = NULL, decimal = NULL, digits = 2)
  r <- as.character(o$columnDefs[[1]]$render)
  expect_true(grepl("render.number(null,null,2", r, fixed = TRUE))
})

test_that("dt2_format_datetime renders NULL args as undefined", {
  o <- dt2_format_datetime(list(columns = "x"), 1, from = NULL, to = "DD/MM/YYYY")
  r <- as.character(o$columnDefs[[1]]$render)
  expect_true(grepl("undefined", r, fixed = TRUE))
  expect_true(grepl('"DD/MM/YYYY"', r, fixed = TRUE))
})

#' @title DT2 Extension Registry
#' @description
#' Declarative registry of all DataTables extensions supported by DT2.
#' Each extension specifies its version, JS/CSS files for core mode,
#' and optional bridge files for Bootstrap 5 mode.
#'
#' Users select extensions via `dt2(extensions = c("Buttons", "Responsive"))`.
#' Only the selected extensions (plus required dependencies) are loaded.
#' @keywords internal
#' @name dt2-extensions

# Extension registry — one entry per extension
# Fields:
#   name      : canonical name (used in `extensions` arg)
#   version   : current version string
#   dir       : directory under inst/htmlwidgets/lib/
#   js_core   : JS files for core mode
#   js_bs5    : additional JS files for bootstrap5 mode (appended after js_core)
#   css_core  : CSS files for core mode
#   css_bs5   : CSS files for bootstrap5 mode (replaces css_core)
#   requires  : character vector of other extension names that must be loaded first
#   optional  : is this extension only loaded when explicitly requested? (default TRUE)

.dt2_extension_registry <- function() {
  list(
    Buttons = list(
      name = "Buttons", version = "3.2.5", dir = "buttons",
      js_core = c("js/dataTables.buttons.min.js",
                   "js/buttons.colVis.min.js",
                   "js/buttons.html5.min.js",
                   "js/buttons.print.min.js"),
      js_bs5  = c("js/dataTables.buttons.min.js",
                   "js/buttons.bootstrap5.min.js",
                   "js/buttons.colVis.min.js",
                   "js/buttons.html5.min.js",
                   "js/buttons.print.min.js"),
      css_core = "css/buttons.dataTables.min.css",
      css_bs5  = "css/buttons.bootstrap5.min.css",
      requires = character()
    ),

    ColReorder = list(
      name = "ColReorder", version = "2.1.1", dir = "colreorder",
      js_core  = "js/dataTables.colReorder.min.js",
      js_bs5   = "js/dataTables.colReorder.min.js",
      css_core = "css/colReorder.dataTables.min.css",
      css_bs5  = "css/colReorder.bootstrap5.min.css",
      requires = character()
    ),

    ColumnControl = list(
      name = "ColumnControl", version = "1.1.0", dir = "columncontrol",
      js_core  = "js/dataTables.columnControl.min.js",
      js_bs5   = c("js/dataTables.columnControl.min.js",
                    "js/columnControl.bootstrap5.min.js"),
      css_core = "css/columnControl.dataTables.min.css",
      css_bs5  = "css/columnControl.bootstrap5.min.css",
      requires = character()
    ),

    DateTime = list(
      name = "DateTime", version = "1.6.0", dir = "datetime",
      js_core  = "js/dataTables.dateTime.min.js",
      js_bs5   = "js/dataTables.dateTime.min.js",
      css_core = "css/dataTables.dateTime.min.css",
      css_bs5  = "css/dataTables.dateTime.min.css",
      requires = character()
    ),

    FixedColumns = list(
      name = "FixedColumns", version = "5.0.5", dir = "fixedcolumns",
      js_core  = "js/dataTables.fixedColumns.min.js",
      js_bs5   = "js/dataTables.fixedColumns.min.js",
      css_core = "css/fixedColumns.dataTables.min.css",
      css_bs5  = "css/fixedColumns.bootstrap5.min.css",
      requires = character()
    ),

    FixedHeader = list(
      name = "FixedHeader", version = "4.0.3", dir = "fixedheader",
      js_core  = "js/dataTables.fixedHeader.min.js",
      js_bs5   = "js/dataTables.fixedHeader.min.js",
      css_core = "css/fixedHeader.dataTables.min.css",
      css_bs5  = "css/fixedHeader.bootstrap5.min.css",
      requires = character()
    ),

    KeyTable = list(
      name = "KeyTable", version = "2.12.1", dir = "keytable",
      js_core  = "js/dataTables.keyTable.min.js",
      js_bs5   = "js/dataTables.keyTable.min.js",
      css_core = "css/keyTable.dataTables.min.css",
      css_bs5  = "css/keyTable.bootstrap5.min.css",
      requires = character()
    ),

    Responsive = list(
      name = "Responsive", version = "3.0.6", dir = "responsive",
      js_core  = "js/dataTables.responsive.min.js",
      js_bs5   = c("js/dataTables.responsive.min.js",
                    "js/responsive.bootstrap5.js"),
      css_core = "css/responsive.dataTables.min.css",
      css_bs5  = "css/responsive.bootstrap5.min.css",
      requires = character()
    ),

    RowGroup = list(
      name = "RowGroup", version = "1.6.0", dir = "rowgroup",
      js_core  = "js/dataTables.rowGroup.min.js",
      js_bs5   = "js/dataTables.rowGroup.min.js",
      css_core = "css/rowGroup.dataTables.min.css",
      css_bs5  = "css/rowGroup.bootstrap5.min.css",
      requires = character()
    ),

    RowReorder = list(
      name = "RowReorder", version = "1.5.0", dir = "rowreorder",
      js_core  = "js/dataTables.rowReorder.min.js",
      js_bs5   = "js/dataTables.rowReorder.min.js",
      css_core = "css/rowReorder.dataTables.min.css",
      css_bs5  = "css/rowReorder.bootstrap5.min.css",
      requires = character()
    ),

    Scroller = list(
      name = "Scroller", version = "2.4.3", dir = "scroller",
      js_core  = "js/dataTables.scroller.min.js",
      js_bs5   = "js/dataTables.scroller.min.js",
      css_core = "css/scroller.dataTables.min.css",
      css_bs5  = "css/scroller.bootstrap5.min.css",
      requires = character()
    ),

    SearchBuilder = list(
      name = "SearchBuilder", version = "1.8.4", dir = "searchbuilder",
      js_core  = "js/dataTables.searchBuilder.min.js",
      js_bs5   = c("js/dataTables.searchBuilder.min.js",
                    "js/searchBuilder.bootstrap5.min.js"),
      css_core = "css/searchBuilder.dataTables.min.css",
      css_bs5  = "css/searchBuilder.bootstrap5.min.css",
      requires = c("DateTime")
    ),

    SearchPanes = list(
      name = "SearchPanes", version = "2.3.5", dir = "searchpanes",
      js_core  = "js/dataTables.searchPanes.min.js",
      js_bs5   = c("js/dataTables.searchPanes.min.js",
                    "js/searchPanes.bootstrap5.min.js"),
      css_core = "css/searchPanes.dataTables.min.css",
      css_bs5  = "css/searchPanes.bootstrap5.min.css",
      requires = character()
    ),

    Select = list(
      name = "Select", version = "3.1.0", dir = "select",
      js_core  = "js/dataTables.select.min.js",
      js_bs5   = "js/dataTables.select.min.js",
      css_core = "css/select.dataTables.min.css",
      css_bs5  = "css/select.bootstrap5.min.css",
      requires = character()
    ),

    StateRestore = list(
      name = "StateRestore", version = "1.4.2", dir = "staterestore",
      js_core  = "js/dataTables.stateRestore.min.js",
      js_bs5   = c("js/dataTables.stateRestore.min.js",
                    "js/stateRestore.bootstrap5.min.js"),
      css_core = "css/stateRestore.dataTables.min.css",
      css_bs5  = "css/stateRestore.bootstrap5.min.css",
      requires = character()
    )
  )
}

#' List available DataTables extensions
#'
#' @return A data.frame with columns `name`, `version`, `dir`.
#' @export
#' @examples
#' dt2_extensions()
dt2_extensions <- function() {
  reg <- .dt2_extension_registry()
  data.frame(
    name    = vapply(reg, `[[`, "", "name"),
    version = vapply(reg, `[[`, "", "version"),
    dir     = vapply(reg, `[[`, "", "dir"),
    stringsAsFactors = FALSE
  )
}

#' Resolve extension names with dependency ordering
#' @param extensions character vector of extension names
#' @return ordered character vector (dependencies first)
#' @keywords internal
.dt2_resolve_extensions <- function(extensions) {
  reg <- .dt2_extension_registry()
  valid <- names(reg)

  # Normalize names (case-insensitive matching)
  resolved <- vapply(extensions, function(ext) {
    idx <- match(tolower(ext), tolower(valid))
    if (is.na(idx)) {
      cli::cli_warn("DT2: unknown extension {.val {ext}}. Available: {.val {valid}}")
      return(NA_character_)
    }
    valid[idx]
  }, character(1), USE.NAMES = FALSE)
  resolved <- resolved[!is.na(resolved)]

  # Add required dependencies
  all_needed <- resolved
  repeat {
    new_deps <- character()
    for (ext in all_needed) {
      reqs <- reg[[ext]]$requires
      new_deps <- c(new_deps, setdiff(reqs, all_needed))
    }
    if (length(new_deps) == 0) break
    all_needed <- c(new_deps, all_needed)
  }

  unique(all_needed)
}

#' Auto-detect extensions from options
#'
#' Scans the options list for keys that imply certain extensions.
#' For example, `buttons` implies Buttons, `select` implies Select, etc.
#'
#' @param options DataTables options list
#' @return character vector of detected extension names
#' @keywords internal
.dt2_detect_extensions <- function(options) {
  detected <- character()

  # Buttons: options$buttons OR buttons anywhere inside layout OR dom contains B
  if (!is.null(options$buttons)) {
    detected <- c(detected, "Buttons")
  }
  # Deep-scan layout for button references (handles nested lists)
  if (!is.null(options$layout)) {
    layout_str <- paste(deparse(options$layout), collapse = " ")
    if (grepl("button", layout_str, ignore.case = TRUE)) {
      detected <- c(detected, "Buttons")
    }
    # Check layout for searchBuilder, searchPanes references
    if (grepl("searchBuilder", layout_str, ignore.case = FALSE)) {
      detected <- c(detected, "SearchBuilder")
    }
    if (grepl("searchPanes", layout_str, ignore.case = FALSE)) {
      detected <- c(detected, "SearchPanes")
    }
  }
  # Legacy dom string: B = Buttons
  if (!is.null(options$dom) && grepl("B", options$dom, fixed = TRUE)) {
    detected <- c(detected, "Buttons")
  }

  # Select
  if (!is.null(options$select) && !identical(options$select, FALSE)) {
    detected <- c(detected, "Select")
  }

  # Responsive
  if (isTRUE(options$responsive)) {
    detected <- c(detected, "Responsive")
  }

  # ColumnControl
  if (!is.null(options$columnControl)) {
    detected <- c(detected, "ColumnControl")
  }

  # SearchBuilder
  if (!is.null(options$searchBuilder)) {
    detected <- c(detected, "SearchBuilder")
  }

  # SearchPanes
  if (!is.null(options$searchPanes)) {
    detected <- c(detected, "SearchPanes")
  }

  # FixedHeader
  if (!is.null(options$fixedHeader) && !identical(options$fixedHeader, FALSE)) {
    detected <- c(detected, "FixedHeader")
  }

  # FixedColumns
  if (!is.null(options$fixedColumns)) {
    detected <- c(detected, "FixedColumns")
  }

  # Scroller
  if (isTRUE(options$scroller) || is.list(options$scroller)) {
    detected <- c(detected, "Scroller")
  }

  # RowGroup
  if (!is.null(options$rowGroup)) {
    detected <- c(detected, "RowGroup")
  }

  # RowReorder
  if (!is.null(options$rowReorder)) {
    detected <- c(detected, "RowReorder")
  }

  # ColReorder
  if (!is.null(options$colReorder)) {
    detected <- c(detected, "ColReorder")
  }

  # KeyTable
  if (!is.null(options$keys) && !identical(options$keys, FALSE)) {
    detected <- c(detected, "KeyTable")
  }

  unique(detected)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)

## -----------------------------------------------------------------------------
# Just works — Bootstrap 5, Jost font, striped, compact
dt2(iris)

## -----------------------------------------------------------------------------
# Responsive is ON by default — table fills the container
dt2(mtcars[1:10, ])

## -----------------------------------------------------------------------------
dt2(iris, responsive = FALSE)

## -----------------------------------------------------------------------------
dt2(iris, striped = FALSE, hover = FALSE, font_scale = 1.0)

## -----------------------------------------------------------------------------
dt2(iris, theme = "minimal", options = list(pageLength = 5))

## -----------------------------------------------------------------------------
my_theme <- dt2_theme("clean", compact = TRUE, font_scale = 0.80)
my_theme

## -----------------------------------------------------------------------------
dt2(iris[1:10, ], theme = my_theme)

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart    = "search",
    topEnd      = "pageLength",
    bottomStart = "paging",
    bottomEnd   = "info"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart  = NULL,         # remove page length selector
    bottomEnd = "paging"      # keep only pagination
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart = list("pageLength", "info"),
    topEnd   = "search",
    bottomEnd = "paging"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topEnd = list(search = list(placeholder = "Type to filter..."))
  )
))

## -----------------------------------------------------------------------------
dt2(mtcars[1:15, ], options = list(
  pageLength = 8,
  buttons = list("copy", "csv", "excel"),
  layout  = list(topEnd = "buttons")
))

## -----------------------------------------------------------------------------
opts <- dt2_use_buttons(
  buttons  = c("copy", "csv", "excel", "pdf", "print"),
  position = "topEnd"
)
dt2(iris[1:20, ], options = opts)

## -----------------------------------------------------------------------------
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  buttons = list("copy", "csv", "excel"),
  layout = list(
    topStart = "buttons",
    topEnd   = list(search = list(placeholder = "Filter...")),
    bottomEnd = "paging"
  )
))

## -----------------------------------------------------------------------------
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topEnd = list(
      buttons = list(
        list(extend = "collection", text = "Export",
             buttons = list("copyHtml5", "csvHtml5", "excelHtml5", "pdfHtml5")),
        list(extend = "spacer", style = "bar"),
        "print",
        list(extend = "spacer", style = "bar"),
        list(extend = "colvis", text = "Columns")
      )
    )
  )
))

## -----------------------------------------------------------------------------
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topEnd = list(
      buttons = list(
        "copy", "csv", "excel",
        list(extend = "spacer", style = "bar"),
        list(extend = "colvis", text = "Columns")
      )
    )
  )
))

## -----------------------------------------------------------------------------
dt2(mtcars[1:10, ], options = list(
  buttons = list("copy", "csv"),
  layout  = list(
    topEnd      = "search",
    bottomStart = "buttons",
    bottomEnd   = "paging"
  )
))

## -----------------------------------------------------------------------------
# Primary blue buttons
dt2(iris[1:10, ],
    button_class = "btn btn-sm btn-primary",
    options = list(
      buttons = list("copy", "csv", "excel"),
      layout = list(topEnd = "buttons")
    ))

## -----------------------------------------------------------------------------
opts <- dt2_use_buttons(
  buttons = c("copy", "csv", "excel"),
  button_class = "btn btn-sm btn-outline-dark"
)
dt2(iris[1:10, ], options = opts)

## -----------------------------------------------------------------------------
# Default pagination — compact, themed by Bootstrap
dt2(iris)

## -----------------------------------------------------------------------------
dt2(iris[1:10, ], options = list(
  paging = FALSE
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      type = "simple"
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      numbers   = FALSE,
      firstLast = TRUE
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      buttons = 3
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      boundaryNumbers = FALSE
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      firstLast    = TRUE,
      previousNext = TRUE,
      numbers      = TRUE
    ))
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  layout = list(
    bottomStart = "paging",
    bottomEnd   = "info"
  )
))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  scroller  = TRUE,
  scrollY   = 300,         # viewport height in pixels
  paging    = TRUE         # required for Scroller
))

## -----------------------------------------------------------------------------
big <- data.frame(
  id    = 1:5000,
  value = round(rnorm(5000), 3),
  group = sample(LETTERS[1:5], 5000, replace = TRUE)
)
dt2(big, options = list(
  scroller    = TRUE,
  scrollY     = 400,
  deferRender = TRUE     # improves performance: renders rows on demand
))

## -----------------------------------------------------------------------------
dt2(big, options = list(
  scroller    = TRUE,
  scrollY     = 350,
  deferRender = TRUE,
  buttons     = list("copy", "csv", "excel"),
  layout      = list(
    topEnd    = "buttons",
    topStart  = list(search = list(placeholder = "Search..."))
  )
))

## -----------------------------------------------------------------------------
opts <- list(columns = names(mtcars))
opts <- dt2_format_number(opts, "hp", thousands = ",", digits = 0)
opts <- dt2_format_number(opts, "wt", digits = 2, prefix_right = " tons")
dt2(mtcars[1:10, ], options = opts)

## -----------------------------------------------------------------------------
df <- data.frame(
  city   = c("São Paulo", "Recife", "NYC", "Tokyo"),
  pop    = c(12.33e6, 1.65e6, 8.34e6, 13.96e6),
  budget = c(6.5e10, 4.2e9, 1.07e11, 7.36e10)
)
opts <- list(columns = names(df))
opts <- dt2_format_number_abbrev(opts, c("pop", "budget"),
                                  digits = 1, locale = "pt-BR")
dt2(df, options = opts)

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 5,
  searching  = TRUE,
  ordering   = TRUE,
  language   = list(search = "Filter:", info = "_TOTAL_ rows")
))

## -----------------------------------------------------------------------------
library(jsonlite)
library(dplyr)
library(tibble)
library(lubridate)
library(DT2)
library(htmlwidgets)

# ── Flag sprite CSS (carregado via dependency para garantir que entra no HTML) ──
flag_dep <- htmltools::htmlDependency(
  name = "world-flags-sprite",
  version = "0.0.1",
  head = '<link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/lafeber/world-flags-sprite/stylesheets/flags32-both.css">',
  src = c(href = ".")
)

# ── Dados ─────────────────────────────────────────────────────────────────────
json_txt <- '{
  "data": [
    {"name":"Tiger Nixon","position":"System Architect","salary":"320800","start_date":"2011-04-25","office":"Edinburgh","extn":"5421"},
    {"name":"Garrett Winters","position":"Accountant","salary":"170750","start_date":"2011-07-25","office":"Tokyo","extn":"8422"},
    {"name":"Ashton Cox","position":"Junior Technical Author","salary":"86000","start_date":"2009-01-12","office":"San Francisco","extn":"1562"},
    {"name":"Cedric Kelly","position":"Senior JavaScript Developer","salary":"433060","start_date":"2012-03-29","office":"Edinburgh","extn":"6224"},
    {"name":"Airi Satou","position":"Accountant","salary":"162700","start_date":"2008-11-28","office":"Tokyo","extn":"5407"},
    {"name":"Brielle Williamson","position":"Integration Specialist","salary":"372000","start_date":"2012-12-02","office":"New York","extn":"4804"},
    {"name":"Herrod Chandler","position":"Sales Assistant","salary":"137500","start_date":"2012-08-06","office":"San Francisco","extn":"9608"},
    {"name":"Rhona Davidson","position":"Integration Specialist","salary":"327900","start_date":"2010-10-14","office":"Tokyo","extn":"6200"},
    {"name":"Colleen Hurst","position":"JavaScript Developer","salary":"205500","start_date":"2009-09-15","office":"San Francisco","extn":"2360"},
    {"name":"Sonya Frost","position":"Software Engineer","salary":"103600","start_date":"2008-12-13","office":"Edinburgh","extn":"1667"}
  ]
}'

df <- fromJSON(json_txt, flatten = TRUE)$data %>%
  as_tibble() %>%
  mutate(
    salary     = as.numeric(salary),
    extn       = as.integer(extn),
    start_date = ymd(start_date)
  )

# ── JS Renderers ──────────────────────────────────────────────────────────────
office_js <- JS("
  function(data, type) {
    if (type !== 'display') return data;
    var cc = {Argentina:'ar', Edinburgh:'_Scotland', London:'_England',
              'New York':'us', 'San Francisco':'us', Sydney:'au', Tokyo:'jp'};
    return '<span class=\"flag ' + (cc[data]||'') + '\"></span> ' + data;
  }
")

salary_js <- JS("
  (function() {
    var nfmt = DataTable.render.number('.', ',', 2, 'R$ ');
    return function(data, type) {
      var txt = nfmt.display(data);
      if (type !== 'display') return txt;
      var c = data < 250000 ? 'red' : data < 500000 ? 'orange' : 'green';
      return '<span style=\"color:' + c + '\">' + txt + '</span>';
    };
  })()
")

extn_js <- JS("
  function(data, type) {
    return type === 'display'
      ? '<progress value=\"' + data + '\" max=\"9999\"></progress>'
      : data;
  }
")

# ── Tabela ────────────────────────────────────────────────────────────────────
w <- dt2(df,
  compact    = TRUE,
  striped    = TRUE,
  hover      = TRUE,
  font_scale = 0.85,
  responsive = FALSE,
  options    = list(
    pageLength = 10,
    lengthMenu = c(5, 10, 25, -1),
    columns    = names(df),
    scrollX = TRUE,  
    layout = list(
      topStart = "pageLength",
      topEnd   = list(
        buttons = list(
          list(extend = "copyHtml5", text = "Copiar"),
          list(extend = "csvHtml5"),
          list(extend = "excelHtml5"),
          list(extend = "spacer", style = "bar"),
          list(extend = "colvis", text = "Colunas")
        ),
        search = list(placeholder = "")
      ),
      bottomEnd = list(paging = list(numbers = FALSE))
    ),

    columnControl = list(
      target  = 0,
      content = list("order", "searchDropdown", list(
        list(extend = "orderAsc",       text = "Ordem crescente"),
        list(extend = "orderDesc",      text = "Ordem decrescente"),
        "spacer",
        list(extend = "colVisDropdown", text = "Selecionar colunas")
      ))
    ),
    ordering = list(indicators = FALSE, handler = FALSE),

    columnDefs = list(
      list(targets = which(names(df) == "office") - 1L,
           className = "f32", render = office_js),
      list(targets = which(names(df) == "salary") - 1L,
           className = "dt-body-right", render = salary_js),
      list(targets = which(names(df) == "extn") - 1L,
           render = extn_js)
    ),

    language = list(
      lengthMenu   = "Mostrar _MENU_",
      search       = "Buscar",
      info         = "Mostrando _START_ a _END_ de _TOTAL_ registros",
      infoEmpty    = "Nenhum registro",
      zeroRecords  = "Nenhum registro encontrado",
      emptyTable   = "Nenhum dado disponível",
      decimal      = ",", thousands = ".", infoThousands = ".",
      lengthLabels = list(`10` = "10", `25` = "25", `-1` = "Todas"),
      paginate     = list(first = "«", previous = "‹", `next` = "›", last = "»"),
      buttons = list(
        copyTitle   = "Copiado!",
        copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
      ),
      columnControl = list(
        orderAsc = "Crescente", orderDesc = "Decrescente",
        searchDropdown = "Pesquisar", colVisDropdown = "Colunas",
        searchClear = "Limpar",
        search = list(
          text   = list(contains = "Contém", starts = "Começa por",
                        ends = "Termina em", equal = "Igual a"),
          number = list(greater = "Maior que", less = "Menor que",
                        equal = "Igual a")
        )
      )
    )
  )
)

# Anexa a dependency do flag sprite ao widget
w$dependencies <- c(w$dependencies, list(flag_dep))
w

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)

## -----------------------------------------------------------------------------
dt2_extensions()

## -----------------------------------------------------------------------------
dt2(mtcars[1:15, ], extensions = "Buttons", options = list(
  pageLength = 8,
  layout = list(
    topEnd = list(
      buttons = list("copy", "csv", "excel", "pdf", "print")
    )
  )
))

## -----------------------------------------------------------------------------
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topStart = "pageLength",
    topEnd = list(
      buttons = list(
        list(extend = "collection", text = "Export",
             buttons = list(
               list(extend = "copyHtml5", text = "Copy"),
               list(extend = "csvHtml5"),
               list(extend = "excelHtml5", title = "My Data"),
               list(extend = "pdfHtml5",   title = "My Data")
             )),
        list(extend = "colvis", text = "Columns")
      ),
      search = list(placeholder = "Filter...")
    ),
    bottomEnd = "paging"
  )
))

## -----------------------------------------------------------------------------
dt2(iris[1:20, ], options = list(
  select = list(style = "os", items = "row"),
  pageLength = 10
))

## -----------------------------------------------------------------------------
dt2(iris, responsive = FALSE, options = list(pageLength = 5))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 8,
  columnControl = list("order", "searchDropdown",
    list(
      list(extend = "orderAsc",  text = "Sort Ascending"),
      list(extend = "orderDesc", text = "Sort Descending"),
      "spacer",
      list(extend = "colVisDropdown", text = "Toggle Columns")
    )
  ),
  ordering = list(indicators = FALSE, handler = FALSE)
))

## ----eval=FALSE---------------------------------------------------------------
# dt2(iris, options = list(fixedHeader = TRUE, pageLength = 50))

## -----------------------------------------------------------------------------
dt2(iris, options = list(
  pageLength = 8,
  select = list(style = "multi", items = "row"),
  layout = list(
    topEnd = list(
      buttons = list(
        list(extend = "selected",  text = "Selected Only"),
        list(extend = "selectAll", text = "Select All"),
        list(extend = "selectNone", text = "Deselect"),
        "spacer",
        list(extend = "csvHtml5", exportOptions = list(modifier = list(selected = TRUE)))
      )
    )
  ),
  columnControl = list("order", "searchDropdown"),
  ordering = list(indicators = FALSE, handler = FALSE)
))

#' Shiny output for DT2
#'
#' Place a DT2 table in a Shiny UI.
#'
#' @param outputId Output ID (must match the `render_dt2()` call in server).
#' @param width,height CSS dimensions.
#' @return An `htmlwidgets` Shiny output (HTML container) suitable for
#'   inclusion in a Shiny UI definition.
#' @export
dt2_output <- function(outputId, width = "100%", height = "auto") {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("Package 'shiny' is required for dt2_output().", call. = FALSE)
  }
  htmlwidgets::shinyWidgetOutput(outputId, "dt2", width, height, package = "DT2")
}

#' Shiny render function for DT2
#'
#' Render a DT2 table in a Shiny server function.
#'
#' @param expr Expression returning a [dt2()] widget.
#' @param env,quoted Standard `shinyRenderWidget` arguments.
#' @return A Shiny render function (closure produced by
#'   [htmlwidgets::shinyRenderWidget()]) that emits a DT2 widget.
#' @export
render_dt2 <- function(expr, env = parent.frame(), quoted = FALSE) {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("Package 'shiny' is required for render_dt2().", call. = FALSE)
  }
  if (!quoted) expr <- substitute(expr)
  htmlwidgets::shinyRenderWidget(expr, dt2_output, env, quoted = TRUE)
}

#' Observe DataTables events published by dt2.js
#'
#' Listen for table events (init, draw, order, search, page, select, deselect).
#'
#' @param input Shiny input object.
#' @param id Widget ID.
#' @param handler Function with signature `(event, type, indexes, rowData)`.
#' @return No return value, called for side effects. Sets up a Shiny
#'   observer that calls `handler` whenever the table emits an event.
#' @export
observe_dt2_events <- function(input, id, handler) {
  if (!requireNamespace("shiny", quietly = TRUE)) {
    stop("Package 'shiny' is required.", call. = FALSE)
  }
  shiny::observeEvent(input[[paste0(id, "_event")]], {
    evt <- input[[paste0(id, "_event")]]
    handler(evt$event, evt$type, evt$indexes, evt$rowData)
  }, ignoreInit = TRUE)
}

#' Access the current state snapshot of a DT2 table
#'
#' Returns a list with `reason`, `order`, `search`, `page`, `selected`, `state`
#' reflecting the current client-side table state.
#'
#' @param input Shiny input object.
#' @param id Widget ID.
#' @return A list with the current table state.
#' @export
dt2_state <- function(input, id) {
  input[[paste0(id, "_state")]]
}

# ---- Public API ---------------------------------------------------------------

#' Check for DataTables library updates
#'
#' Queries the npm registry to compare installed library versions against
#' the latest available versions. Version constraints are enforced to prevent
#' incompatible major version upgrades (e.g. jQuery 3.x will not jump to 4.x).
#'
#' @param quiet Logical. If \code{TRUE}, returns the result invisibly without
#'   printing. Default \code{FALSE}.
#' @return A data.frame (invisibly) with columns:
#'   \code{library}, \code{installed}, \code{latest}, \code{latest_ok},
#'   \code{constraint}, \code{status}.
#'
#'   Status values:
#'   \describe{
#'     \item{\code{"ok"}}{Library is up to date.}
#'     \item{\code{"UPDATE"}}{A compatible update is available.}
#'     \item{\code{"PINNED"}}{A new major version exists, but is blocked by
#'       the version constraint. The library is up to date within its
#'       allowed range.}
#'     \item{\code{"error"}}{Lookup failed (check your internet connection).}
#'   }
#' @export
#' @examples
#' \donttest{
#' dt2_check_updates()
#'
#' # programmatic use
#' updates <- dt2_check_updates(quiet = TRUE)
#' updates[updates$status == "UPDATE", ]
#' }
dt2_check_updates <- function(quiet = FALSE) {

  libs        <- .dt2_lib_versions()
  npm_map     <- .dt2_npm_map()
  constraints <- .dt2_version_constraints()

  n         <- length(libs)
  latest    <- character(n)  # actual npm latest (may exceed constraint)
  latest_ok <- character(n)  # latest within constraint
  status    <- character(n)

  if (!quiet) message("Checking ", n, " libraries against npm registry...")

  for (i in seq_along(libs)) {
    nm  <- names(libs)[i]
    pkg <- npm_map[nm]
    pin <- constraints[nm]  # e.g. "3." or "0.2." or NA (no constraint)

    if (is.na(pkg)) {
      latest[i] <- latest_ok[i] <- NA_character_
      status[i] <- "error"
      next
    }

    # Fetch latest from npm
    ver <- .npm_latest_version(pkg)
    latest[i] <- ver

    if (is.na(ver)) {
      latest_ok[i] <- NA_character_
      status[i] <- "error"
      next
    }

    # No constraint or latest is within constraint
    if (is.na(pin) || startsWith(ver, pin)) {
      latest_ok[i] <- ver
      if (ver == libs[i]) {
        status[i] <- "ok"
      } else if (.version_gt(ver, libs[i])) {
        status[i] <- "UPDATE"
      } else {
        status[i] <- "ok"
      }
    } else {
      # Latest exceeds constraint — find best within range
      constrained <- .npm_latest_in_range(pkg, pin)
      latest_ok[i] <- constrained

      if (is.na(constrained) || constrained == libs[i]) {
        status[i] <- "PINNED"
      } else if (.version_gt(constrained, libs[i])) {
        status[i] <- "UPDATE"
      } else {
        status[i] <- "PINNED"
      }
    }
  }

  result <- data.frame(
    library    = names(libs),
    installed  = unname(libs),
    latest     = unname(latest),
    latest_ok  = unname(latest_ok),
    constraint = unname(ifelse(is.na(constraints[names(libs)]),
                               "", constraints[names(libs)])),
    status     = status,
    stringsAsFactors = FALSE
  )

  if (!quiet) .print_update_table(result)

  invisible(result)
}


#' Update DataTables JS/CSS libraries (developer tool)
#'
#' Checks for updates (respecting version constraints), patches version
#' numbers in the source files, and optionally runs
#' \code{tools/get-dt2-libs.sh} to download the new files.
#'
#' This function only works from the \strong{package source tree} (i.e.
#' during development). It will refuse to run from an installed package.
#'
#' @param pkg_dir Path to the DT2 source root (the directory containing
#'   \code{DESCRIPTION}). Defaults to the current working directory.
#' @param download Logical. If \code{TRUE}, runs the shell script after
#'   patching version numbers. Default \code{TRUE}.
#' @param dry_run Logical. If \code{TRUE}, shows what would change without
#'   modifying any files. Default \code{FALSE}.
#'
#' @details
#' The workflow is:
#' \enumerate{
#'   \item Query npm for the latest compatible version of every library.
#'   \item Patch \code{tools/get-dt2-libs.sh} (version variables).
#'   \item Patch \code{R/dt2_extensions.R} (extension registry).
#'   \item Patch \code{R/dt2_check_updates.R} (core lib versions).
#'   \item Patch \code{R/dt2_deps.R} (DataTables core version).
#'   \item Run \code{bash tools/get-dt2-libs.sh} to download the files.
#' }
#'
#' Version constraints prevent incompatible upgrades:
#' \itemize{
#'   \item jQuery is pinned to 3.x (DataTables 2 requires jQuery 3).
#'   \item pdfmake is pinned to 0.2.x (0.3.x has breaking changes and
#'     is not available on cdnjs).
#'   \item Bootstrap is pinned to 5.x.
#' }
#'
#' Libraries marked as \code{"PINNED"} are skipped. Only \code{"UPDATE"}
#' items are applied.
#'
#' @return Invisibly, a data.frame with the update results.
#' @export
#' @examples
#' \dontrun{
#' # Developer-only tool: requires the DT2 package source tree
#' # (DESCRIPTION, tools/get-dt2-libs.sh, R/dt2_extensions.R, ...).
#' # It cannot run from an installed package, so it is not executable
#' # in CRAN check or from a regular user session.
#'
#' # from the DT2 source root:
#' dt2_update_libs()
#'
#' # preview changes without modifying anything:
#' dt2_update_libs(dry_run = TRUE)
#' }
dt2_update_libs <- function(pkg_dir = ".", download = TRUE, dry_run = FALSE) {

  pkg_dir <- normalizePath(pkg_dir, mustWork = FALSE)

  # ---- validate source tree ----
  script_path <- file.path(pkg_dir, "tools", "get-dt2-libs.sh")
  ext_path    <- file.path(pkg_dir, "R", "dt2_extensions.R")
  upd_path    <- file.path(pkg_dir, "R", "dt2_check_updates.R")
  deps_path   <- file.path(pkg_dir, "R", "dt2_deps.R")

  required <- c(
    DESCRIPTION             = file.path(pkg_dir, "DESCRIPTION"),
    `tools/get-dt2-libs.sh` = script_path,
    `R/dt2_extensions.R`    = ext_path
  )
  missing <- !file.exists(required)
  if (any(missing)) {
    stop("Not a DT2 source tree. Missing: ",
         paste(names(required)[missing], collapse = ", "),
         "\nRun this from the package root or set pkg_dir=.",
         call. = FALSE)
  }

  # ---- check for updates ----
  result  <- dt2_check_updates(quiet = FALSE)
  updates <- result[result$status == "UPDATE", , drop = FALSE]

  if (nrow(updates) == 0) {
    message("\nNothing to update!")
    return(invisible(result))
  }

  if (dry_run) {
    message("\n[DRY RUN] Would update ", nrow(updates), " library(ies):")
    for (i in seq_len(nrow(updates))) {
      message("  ", updates$library[i], ": ",
              updates$installed[i], " -> ", updates$latest_ok[i])
    }
    message("\n  Files that would be modified:")
    message("    - tools/get-dt2-libs.sh")
    message("    - R/dt2_extensions.R")
    message("    - R/dt2_check_updates.R")
    message("    - R/dt2_deps.R")
    return(invisible(result))
  }

  message("\nPatching ", nrow(updates), " version(s)...")

  # ---- shell script variable map ----
  shell_var_map <- .dt2_shell_var_map()

  # extension names in the registry
  ext_names <- c("Buttons", "ColReorder", "ColumnControl", "DateTime",
                 "FixedColumns", "FixedHeader", "KeyTable", "Responsive",
                 "RowGroup", "RowReorder", "Scroller", "SearchBuilder",
                 "SearchPanes", "Select", "StateRestore")

  # ---- patch each update ----
  for (i in seq_len(nrow(updates))) {
    lib     <- updates$library[i]
    old_ver <- updates$installed[i]
    new_ver <- updates$latest_ok[i]

    message("  ", lib, ": ", old_ver, " -> ", new_ver)

    # 1) Shell script
    var <- shell_var_map[lib]
    if (!is.na(var)) {
      .patch_file(script_path,
        paste0(var, '="', old_ver, '"'),
        paste0(var, '="', new_ver, '"')
      )
    }

    # 2) Extension registry (version + dir combo is unique per extension)
    if (lib %in% ext_names) {
      reg <- .dt2_extension_registry()
      ext_dir <- reg[[lib]]$dir
      .patch_file(ext_path,
        paste0('version = "', old_ver, '", dir = "', ext_dir, '"'),
        paste0('version = "', new_ver, '", dir = "', ext_dir, '"')
      )
    }

    # 3) Core versions in dt2_check_updates.R and dt2_deps.R
    if (lib == "DataTables") {
      .patch_file(upd_path,
        paste0('DataTables = "', old_ver, '"'),
        paste0('DataTables = "', new_ver, '"')
      )
      .patch_file(deps_path,
        paste0('dt_ver <- "', old_ver, '"'),
        paste0('dt_ver <- "', new_ver, '"')
      )
    } else if (lib %in% c("jQuery", "Moment", "JSZip", "PDFMake", "Bootstrap")) {
      .patch_file(upd_path,
        paste0(lib, ' = "', old_ver, '"'),
        paste0(lib, ' = "', new_ver, '"')
      )
    }
  }

  message("Done patching files.")

  # ---- download ----
  if (download) {
    message("\nRunning tools/get-dt2-libs.sh ...")
    old_wd <- setwd(pkg_dir)
    on.exit(setwd(old_wd), add = TRUE)
    rc <- system2("bash", "tools/get-dt2-libs.sh")
    if (rc != 0) {
      warning("Shell script exited with code ", rc, call. = FALSE)
    } else {
      message("\nLibraries downloaded successfully.")
    }
  } else {
    message("\nSkipping download. Run manually:")
    message("  cd ", pkg_dir)
    message("  bash tools/get-dt2-libs.sh")
  }

  invisible(result)
}


# ---- Version constraints -----------------------------------------------------

#' Version constraints for safe updates
#'
#' Defines the allowed version prefix for each library. Updates are only
#' applied within this prefix. For example, jQuery is constrained to
#' \code{"3."} so it will never auto-upgrade to 4.x.
#'
#' Rationale for each constraint:
#' \describe{
#'   \item{jQuery \code{"3."}}{DataTables 2.x requires jQuery 3.}
#'   \item{PDFMake \code{"0.2."}}{0.3.x has breaking changes and is not
#'     available on cdnjs.}
#'   \item{Bootstrap \code{"5."}}{DataTables BS5 styling requires Bootstrap 5.}
#'   \item{Moment \code{"2."}}{Major version pin for stability.}
#'   \item{JSZip \code{"3."}}{Major version pin for stability.}
#' }
#'
#' DataTables extensions do NOT have explicit constraints because their
#' major version tracks DataTables core compatibility (managed by the
#' DataTables project itself).
#'
#' @return Named character vector. Name = library name, value = version prefix.
#'   Only libraries with constraints are included (NA = no constraint).
#' @keywords internal
.dt2_version_constraints <- function() {
  c(
    jQuery    = "3.",
    PDFMake   = "0.2.",
    Bootstrap = "5.",
    Moment    = "2.",
    JSZip     = "3."
  )
}


# ---- npm helpers -------------------------------------------------------------

#' Mapping of DT2 library names to npm package names
#' @keywords internal
.dt2_npm_map <- function() {
  c(
    DataTables    = "datatables.net",
    jQuery        = "jquery",
    Moment        = "moment",
    JSZip         = "jszip",
    PDFMake       = "pdfmake",
    Bootstrap     = "bootstrap",
    Buttons       = "datatables.net-buttons",
    ColReorder    = "datatables.net-colreorder",
    ColumnControl = "datatables.net-columncontrol",
    DateTime      = "datatables.net-datetime",
    FixedColumns  = "datatables.net-fixedcolumns",
    FixedHeader   = "datatables.net-fixedheader",
    KeyTable      = "datatables.net-keytable",
    Responsive    = "datatables.net-responsive",
    RowGroup      = "datatables.net-rowgroup",
    RowReorder    = "datatables.net-rowreorder",
    Scroller      = "datatables.net-scroller",
    SearchBuilder = "datatables.net-searchbuilder",
    SearchPanes   = "datatables.net-searchpanes",
    Select        = "datatables.net-select",
    StateRestore  = "datatables.net-staterestore"
  )
}


#' Shell script variable names for each library
#' @keywords internal
.dt2_shell_var_map <- function() {
  c(
    DataTables    = "DT_VER",
    jQuery        = "JQUERY_VER",
    Moment        = "MOMENT_VER",
    JSZip         = "JSZIP_VER",
    PDFMake       = "PDFMAKE_VER",
    Bootstrap     = "BS_VER",
    Buttons       = "BTN_VER",
    ColReorder    = "CR_VER",
    ColumnControl = "CC_VER",
    DateTime      = "DTM_VER",
    FixedColumns  = "FC_VER",
    FixedHeader   = "FH_VER",
    KeyTable      = "KT_VER",
    Responsive    = "RS_VER",
    RowGroup      = "RG_VER",
    RowReorder    = "RR_VER",
    Scroller      = "SCR_VER",
    SearchBuilder = "SB_VER",
    SearchPanes   = "SP_VER",
    Select        = "SEL_VER",
    StateRestore  = "SR_VER"
  )
}


#' Get all current library versions in DT2
#'
#' Core library versions here must be kept in sync with
#' \code{R/dt2_deps.R} and \code{tools/get-dt2-libs.sh}.
#' Extension versions are read from the registry automatically.
#'
#' @return Named character vector
#' @keywords internal
.dt2_lib_versions <- function() {
  core <- c(
    DataTables = "2.3.4",
    jQuery     = "3.7.0",
    Moment     = "2.29.4",
    JSZip      = "3.10.1",
    PDFMake    = "0.2.7",
    Bootstrap  = "5.3.8"
  )

  reg <- .dt2_extension_registry()
  ext_vers <- vapply(reg, `[[`, "", "version")
  names(ext_vers) <- vapply(reg, `[[`, "", "name")

  c(core, ext_vers)
}


#' Fetch latest version of an npm package
#' @param pkg npm package name
#' @return version string or NA
#' @keywords internal
.npm_latest_version <- function(pkg) {
  url <- paste0("https://registry.npmjs.org/", pkg, "/latest")
  tryCatch({
    con <- url(url, open = "rb")
    on.exit(close(con), add = TRUE)
    raw <- readLines(con, warn = FALSE)
    json_text <- paste(raw, collapse = "")

    m <- regmatches(json_text, regexpr('"version"\\s*:\\s*"[^"]+"', json_text))
    if (length(m) == 0) return(NA_character_)
    ver <- sub('^"version"\\s*:\\s*"', "", m[1])
    sub('"$', "", ver)
  }, error = function(e) {
    NA_character_
  })
}


#' Fetch the latest version of an npm package within a version prefix
#'
#' Queries the full package metadata from npm and extracts all version
#' strings, then returns the highest version that starts with \code{prefix}.
#'
#' @param pkg npm package name
#' @param prefix version prefix, e.g. \code{"3."} or \code{"0.2."}
#' @return version string or NA
#' @keywords internal
.npm_latest_in_range <- function(pkg, prefix) {
  url <- paste0("https://registry.npmjs.org/", pkg)
  tryCatch({
    con <- url(url, open = "rb")
    on.exit(close(con), add = TRUE)
    raw <- readLines(con, warn = FALSE)
    json_text <- paste(raw, collapse = "")

    # Extract all version keys from "versions":{"x.y.z":{...},...}
    # We look for the "versions" object and extract its keys
    versions_block <- regmatches(json_text,
      regexpr('"versions"\\s*:\\s*\\{', json_text))
    if (length(versions_block) == 0) return(NA_character_)

    # Extract all quoted keys that look like versions (digits and dots)
    all_vers <- regmatches(json_text,
      gregexpr('"(\\d+\\.\\d+[^"]*)"\\s*:\\s*\\{', json_text))[[1]]
    all_vers <- sub('"\\s*:\\s*\\{$', "", all_vers)
    all_vers <- sub('^"', "", all_vers)

    # Filter: must start with prefix, no pre-release tags (alpha, beta, rc)
    matching <- all_vers[startsWith(all_vers, prefix)]
    matching <- matching[!grepl("[a-zA-Z]", matching)]  # exclude pre-release

    if (length(matching) == 0) return(NA_character_)

    # Find the highest version
    best <- matching[1]
    for (v in matching[-1]) {
      if (.version_gt(v, best)) best <- v
    }
    best
  }, error = function(e) {
    NA_character_
  })
}


# ---- version utilities -------------------------------------------------------

#' Compare two version strings (a > b?)
#' @keywords internal
.version_gt <- function(a, b) {
  va <- as.integer(strsplit(a, "[.-]")[[1]])
  vb <- as.integer(strsplit(b, "[.-]")[[1]])
  maxlen <- max(length(va), length(vb))
  va <- c(va, rep(0L, maxlen - length(va)))
  vb <- c(vb, rep(0L, maxlen - length(vb)))
  for (i in seq_len(maxlen)) {
    if (is.na(va[i]) || is.na(vb[i])) return(FALSE)
    if (va[i] > vb[i]) return(TRUE)
    if (va[i] < vb[i]) return(FALSE)
  }
  FALSE
}


# ---- file patching -----------------------------------------------------------

#' Replace a string in a file (first occurrence only)
#' @param path file path
#' @param old string to find (fixed, not regex)
#' @param new replacement string
#' @keywords internal
.patch_file <- function(path, old, new) {
  if (!file.exists(path)) return(invisible(NULL))
  txt  <- readLines(path, warn = FALSE)
  full <- paste(txt, collapse = "\n")
  if (!grepl(old, full, fixed = TRUE)) return(invisible(NULL))
  full <- sub(old, new, full, fixed = TRUE)
  writeLines(strsplit(full, "\n", fixed = TRUE)[[1]], path)
}


# ---- pretty printing ---------------------------------------------------------

#' Pretty-print the update check table
#' @keywords internal
.print_update_table <- function(result) {
  n_update <- sum(result$status == "UPDATE", na.rm = TRUE)
  n_pinned <- sum(result$status == "PINNED", na.rm = TRUE)
  n_error  <- sum(result$status == "error",  na.rm = TRUE)

  message("")
  message(sprintf("%-18s %-11s %-11s %-11s %s",
                  "Library", "Installed", "Latest", "Compat.", "Status"))
  message(strrep("-", 65))

  for (r in seq_len(nrow(result))) {
    st <- result$status[r]
    flag <- switch(st,
      "UPDATE" = "\U26A0\UFE0F  UPDATE",
      "ok"     = "\U2705 ok",
      "PINNED" = "\U0001F512 PINNED",
      "error"  = "\U274C error",
      st
    )
    # Show constraint info for pinned libs
    note <- ""
    if (st == "PINNED" && result$constraint[r] != "") {
      note <- paste0(" [pin: ", result$constraint[r], "x]")
    }

    message(sprintf("%-18s %-11s %-11s %-11s %s%s",
      result$library[r],
      result$installed[r],
      ifelse(is.na(result$latest[r]), "???", result$latest[r]),
      ifelse(is.na(result$latest_ok[r]), "???", result$latest_ok[r]),
      flag,
      note
    ))
  }

  message(strrep("-", 65))
  if (n_update > 0) {
    message(n_update, " compatible update(s) available.",
            " Use dt2_update_libs() to apply.")
  } else if (n_error == 0 && n_pinned == 0) {
    message("All libraries are up to date!")
  } else if (n_error == 0) {
    message("All libraries are up to date within their constraints.")
  }
  if (n_pinned > 0) {
    message(n_pinned, " library(ies) pinned to avoid breaking changes.",
            " Edit .dt2_version_constraints() to change.")
  }
  if (n_error > 0) {
    message(n_error, " lookup(s) failed (check your internet connection).")
  }
  message("")
}

#' Internal: parse DataTables server-side request (query string to list)
#' @keywords internal
.dt2_parse_ssp_request <- function(req, n_cols) {
  qs <- req$QUERY_STRING %||% ""
  if (is.null(qs) || identical(qs, "")) return(list(draw = 1L, start = 0L, length = 10L))

  kv <- strsplit(qs, "&", fixed = TRUE)[[1]]
  kv <- kv[nzchar(kv)]
  parts <- strsplit(kv, "=", fixed = TRUE)
  parts <- lapply(parts, function(x) utils::URLdecode(if (length(x) == 2) x[2] else ""))

  # Keys must be URL-decoded too: dt2.js encodes them with encodeURIComponent,
  # so e.g. "search[value]" arrives as "search%5Bvalue%5D" and bracketed
  # order keys as "order%5B0%5D%5Bcolumn%5D". Decoding here lets the lookups
  # below (q[["search[value]"]], "order[i][column]") match.
  keys <- vapply(strsplit(kv, "=", fixed = TRUE),
                 function(x) utils::URLdecode(x[[1]]), character(1))
  q <- stats::setNames(parts, keys)

  num <- function(x, default = NA_integer_) {
    y <- suppressWarnings(as.integer(x))
    ifelse(is.na(y), default, y)
  }

  draw   <- num(q[["draw"]],   1L)
  start  <- num(q[["start"]],  0L)
  length <- num(q[["length"]], 10L)

  # global search
  search_value <- q[["search[value]"]]
  search_regex <- isTRUE(q[["search[regex]"]] %in% c("true", "TRUE", "1"))

  # ordering (podem existir múltiplas entradas order[i][...])
  # coletamos pares (column, dir) por i = 0..n
  ord <- list()
  for (i in 0:max(0, n_cols - 1)) {
    col_key <- sprintf("order[%d][column]", i)
    dir_key <- sprintf("order[%d][dir]", i)
    if (!is.null(q[[col_key]]) && !is.null(q[[dir_key]])) {
      ord[[length(ord) + 1]] <- list(
        column = num(q[[col_key]], 0L) + 1L,  # 1-based em R
        dir    = if (tolower(q[[dir_key]]) %in% c("desc", "descending")) "desc" else "asc"
      )
    }
  }

  list(
    draw = draw, start = start, length = length,
    search = list(value = search_value, regex = search_regex),
    order = ord
  )
}

#' Internal: build DataTables JSON payload
#' @keywords internal
.dt2_payload <- function(draw, total, filtered, data_rows) {
  list(
    draw = draw,
    recordsTotal = as.integer(total),
    recordsFiltered = as.integer(filtered),
    data = data_rows
  )
}

#' Default server-side handler (filter/order/page)
#'
#' @param names character() column names in display order.
#' @return function(data, req) -> list(draw, recordsTotal, recordsFiltered, data)
#' @export
dt2_ssp_handler <- function(names) {
  force(names)
  function(data, req) {
    stopifnot(is.data.frame(data))
    n_cols <- length(names)
    pars <- .dt2_parse_ssp_request(req, n_cols)

    draw   <- pars$draw
    start  <- max(0L, pars$start)
    length <- max(0L, pars$length)
    idx_cols <- names

    # base
    df <- data

    # search global (case-insensitive, não regex por padrão)
    if (!is.null(pars$search$value) && nzchar(pars$search$value)) {
      pat <- tolower(pars$search$value)
      keep <- Reduce(`|`, lapply(df[idx_cols], function(col) {
        grepl(pat, tolower(as.character(col)), fixed = TRUE)
      }))
      df <- df[keep, , drop = FALSE]
    }

    # ordering (aplica em cascata)
    if (length(pars$order)) {
      for (ord in rev(pars$order)) { # último primeiro para estabilidade
        j <- max(1L, min(n_cols, ord$column))
        nm <- idx_cols[j]
        if (ord$dir == "desc") {
          df <- df[order(df[[nm]], decreasing = TRUE, na.last = TRUE), , drop = FALSE]
        } else {
          df <- df[order(df[[nm]], decreasing = FALSE, na.last = TRUE), , drop = FALSE]
        }
      }
    }

    total <- nrow(data)
    filt  <- nrow(df)

    # paginação
    if (length >= 0) {
      i1 <- start + 1L
      i2 <- min(filt, start + length)
      if (i1 <= i2 && filt > 0) df <- df[i1:i2, , drop = FALSE] else df <- df[0, , drop = FALSE]
    }

    # retorna como array de objetos (chaves = nomes)
    rows <- lapply(seq_len(nrow(df)), function(i) {
      as.list(stats::setNames(df[i, idx_cols, drop = TRUE], idx_cols))
    })

    .dt2_payload(draw, total, filt, rows)
  }
}

#' Bind a DataTables v2 server-side endpoint to a widget id
#'
#' @param id Output id of the widget (e.g., "tbl").
#' @param data A data.frame with the source data.
#' @param session Shiny session (default: current).
#' @param handler Optional custom handler function(data, req) -> list(...).
#' @return No return value, called for side effects. Registers a Shiny
#'   observer on `session` that responds to client-side server-processing
#'   requests for the given widget `id`.
#' @export
dt2_bind_server <- function(id, data, session = shiny::getDefaultReactiveDomain(), handler = NULL) {
  stopifnot(!is.null(session), is.character(id), length(id) == 1)
  stopifnot(is.data.frame(data))
  # nomes em exibição; se o JS recebeu options$columns, use-os
  col_names <- names(data)
  handler <- handler %||% dt2_ssp_handler(col_names)

  req_name  <- paste0(id, "_server_req")
  resp_name <- paste0(id, "_server_resp")

  shiny::observeEvent(session$input[[req_name]], {
    req <- session$input[[req_name]]
    # Shim "req" com QUERY_STRING (htmlwidgets v1.6+ envia objeto; nós montamos uma string)
    qs <- req$queryString %||% ""  # nosso dt2.js envia request + queryString
    fake_req <- list(QUERY_STRING = qs)

    payload <- handler(data, fake_req)
    session$sendCustomMessage(resp_name, payload)
  }, ignoreInit = TRUE, priority = 10)
}

# Regression tests for the server-side processing (SSP) request parser.
# Bug: dt2.js encodes query-string KEYS with encodeURIComponent
# (e.g. "search[value]" -> "search%5Bvalue%5D"), but the parser only
# URL-decoded the values, never the keys. As a result global search and
# ordering were silently never applied -- only pagination worked.

# Mirror how dt2.js encodes a key (encodeURIComponent on the key text).
enc <- function(s) utils::URLencode(s, reserved = TRUE)

ssp_qs <- function(...) paste(..., sep = "&")

test_that(".dt2_parse_ssp_request decodes encoded search/order keys", {
  qs <- ssp_qs(
    "draw=2", "start=0", "length=10",
    paste0(enc("search[value]"),     "=", enc("foo bar")),
    paste0(enc("search[regex]"),     "=false"),
    paste0(enc("order[0][column]"),  "=1"),
    paste0(enc("order[0][dir]"),     "=desc")
  )
  pars <- .dt2_parse_ssp_request(list(QUERY_STRING = qs), n_cols = 3)

  expect_equal(pars$draw, 2L)
  expect_equal(pars$search$value, "foo bar")  # was NULL before the fix
  expect_false(pars$search$regex)
  expect_length(pars$order, 1)
  expect_equal(pars$order[[1]]$column, 2L)     # 0-based 1 -> 1-based 2
  expect_equal(pars$order[[1]]$dir, "desc")
})

test_that("dt2_ssp_handler applies global search and ordering end-to-end", {
  df <- data.frame(
    id   = 1:5,
    name = c("alpha", "beta", "gamma", "Alpha", "BETA"),
    stringsAsFactors = FALSE
  )
  h <- dt2_ssp_handler(names(df))

  qs <- ssp_qs(
    "draw=1", "start=0", "length=10",
    paste0(enc("search[value]"),    "=", enc("alpha")),
    paste0(enc("order[0][column]"), "=0"),
    paste0(enc("order[0][dir]"),    "=desc")
  )
  out <- h(df, list(QUERY_STRING = qs))

  expect_equal(out$recordsTotal, 5L)
  # case-insensitive global search matches "alpha" and "Alpha"
  expect_equal(out$recordsFiltered, 2L)
  # ordered by id descending -> first row is id 4 ("Alpha"), not id 1
  expect_equal(out$data[[1]]$id, 4L)
})

test_that("dt2_ssp_handler paginates", {
  df <- data.frame(id = 1:100)
  h  <- dt2_ssp_handler(names(df))
  qs <- ssp_qs("draw=1", "start=20", "length=5")
  out <- h(df, list(QUERY_STRING = qs))

  expect_equal(out$recordsTotal, 100L)
  expect_equal(out$recordsFiltered, 100L)
  expect_equal(length(out$data), 5L)
  expect_equal(out$data[[1]]$id, 21L)
})

# ---- Theme presets -----------------------------------------------------------

#' @keywords internal
.dt2_presets <- function() {
  list(
    default = list(
      striped = TRUE, hover = TRUE, compact = TRUE,
      font_scale = 0.8, style = "bootstrap5",
      button_class = "btn btn-sm btn-outline-secondary"
    ),
    clean = list(
      striped = TRUE, hover = TRUE, compact = FALSE,
      font_scale = 0.85, style = "bootstrap5",
      button_class = "btn btn-sm btn-outline-secondary"
    ),
    minimal = list(
      striped = FALSE, hover = FALSE, compact = FALSE,
      font_scale = 0.9, style = "bootstrap5",
      button_class = "btn btn-sm btn-outline-secondary"
    ),
    compact = list(
      striped = TRUE, hover = TRUE, compact = TRUE,
      font_scale = 0.75, style = "bootstrap5",
      button_class = "btn btn-sm btn-outline-secondary"
    )
  )
}


# ---- dt2_theme(): reusable theme objects ------------------------------------

#' Create a reusable DT2 theme
#'
#' @description
#' Creates a theme object that can be passed to [dt2()] via the `theme`
#' parameter. Useful when you want the same look across many tables.
#'
#' For quick one-off styling, you can also pass arguments directly to
#' [dt2()] (e.g., `dt2(iris, striped = FALSE)`).
#'
#' @param preset A named preset to start from: `"default"`, `"clean"`,
#'   `"minimal"`, or `"compact"`. Remaining arguments override the preset.
#' @param striped Logical; alternate row colours.
#' @param hover Logical; highlight rows on hover.
#' @param compact Logical; reduce cell padding.
#' @param font_scale Numeric; font-size multiplier (e.g., 0.85 = 85%).
#' @param style Styling framework: `"bootstrap5"` (default) or `"core"`
#'   (plain DataTables).
#' @param button_class CSS class string for Buttons extension buttons.
#'   Default: `"btn btn-sm btn-outline-secondary"`. See Bootstrap 5
#'   button classes for options (e.g., `"btn btn-sm btn-primary"`).
#'
#' @return A `dt2_theme` object (a named list).
#' @export
#'
#' @examples
#' # Create and reuse
#' my_theme <- dt2_theme("clean", compact = TRUE)
#' \donttest{
#' dt2(iris, theme = my_theme)
#' dt2(mtcars, theme = my_theme)
#' }
#'
#' # Custom button style
#' dt2_theme("default", button_class = "btn btn-sm btn-primary")
#'
#' # Presets
#' dt2_theme("minimal")
#' dt2_theme("compact")
dt2_theme <- function(preset = "default",
                      striped = NULL, hover = NULL, compact = NULL,
                      font_scale = NULL, style = NULL,
                      button_class = NULL) {

  # ---- Backward compat: old API was dt2_theme(options_list, striped=T, ...) ----
  if (is.list(preset)) {
    options <- preset
    th <- .dt2_presets()[["default"]]
    if (!is.null(striped))    th$striped    <- isTRUE(striped)
    if (!is.null(hover))      th$hover      <- isTRUE(hover)
    if (!is.null(compact))    th$compact    <- isTRUE(compact)
    if (!is.null(font_scale)) th$font_scale <- font_scale
    if (!is.null(style))      th$style      <- match.arg(style, c("bootstrap5", "core"))
    options$dt2_theme <- th
    return(options)
  }

  presets <- .dt2_presets()
  base <- presets[[preset %||% "default"]]
  if (is.null(base)) {
    stop("Unknown theme preset: '", preset,
         "'. Available: ", paste(names(presets), collapse = ", "),
         call. = FALSE)
  }

  # Override with explicit args
  if (!is.null(striped))      base$striped      <- isTRUE(striped)
  if (!is.null(hover))        base$hover        <- isTRUE(hover)
  if (!is.null(compact))      base$compact      <- isTRUE(compact)
  if (!is.null(font_scale))   base$font_scale   <- font_scale
  if (!is.null(style))        base$style        <- match.arg(style, c("bootstrap5", "core"))
  if (!is.null(button_class)) base$button_class <- button_class

  structure(base, class = "dt2_theme")
}

#' @export
print.dt2_theme <- function(x, ...) {
  cat("DT2 theme:\n")
  cat("  striped      =", x$striped, "\n")
  cat("  hover        =", x$hover, "\n")
  cat("  compact      =", x$compact, "\n")
  cat("  font_scale   =", x$font_scale, "\n")
  cat("  style        =", x$style, "\n")
  cat("  button_class =", x$button_class, "\n")
  cat("  class        =", x$class %||% "<none>", "\n")
  invisible(x)
}


# ---- dt2(): main function ---------------------------------------------------

#' Create a DT2 DataTable widget
#'
#' @description
#' The main function for creating interactive DataTables.
#' Works standalone (R Markdown, Quarto, Viewer) and inside Shiny.
#'
#' **Styling** is controlled directly via `theme`, `striped`, `hover`,
#' `compact`, `font_scale` -- or a CSS `class` string for full control.
#'
#' **DataTables configuration** goes in `options` (1:1 mapping to the
#' JavaScript API). The two concerns are cleanly separated.
#'
#' @param data A `data.frame`, `tibble`, or `matrix`.
#' @param theme A theme preset name (`"default"`, `"clean"`, `"minimal"`,
#'   `"compact"`) or a [dt2_theme()] object. Default: `"default"`.
#' @param striped,hover,compact Logical; override the theme.
#'   `NULL` (default) = use theme value.
#' @param font_scale Numeric; override the theme font-scale.
#'   `NULL` (default) = use theme value.
#' @param style Styling framework: `"bootstrap5"` (default) or `"core"`.
#' @param class Optional CSS class string (e.g., `"table table-dark"`).
#'   If provided, overrides all theme-generated classes.
#' @param button_class CSS class for Buttons extension buttons.
#'   Default: `"btn btn-sm btn-outline-secondary"`.
#'   Examples: `"btn btn-sm btn-primary"`, `"btn btn-sm btn-outline-dark"`.
#' @param responsive Logical; enable the Responsive extension so the table
#'   fills 100\% width and adapts to narrow screens by collapsing columns.
#'   Default: `TRUE`. Set `FALSE` to disable.
#' @param options List of DataTables options. See
#'   \url{https://datatables.net/reference/option/}.
#' @param extensions Character vector of extensions to load
#'   (e.g., `c("Buttons", "Select")`). Auto-detected from `options`
#'   when `NULL`.
#' @param width,height CSS dimensions.
#' @param elementId Optional HTML element ID.
#'
#' @return An `htmlwidget` object.
#' @export
#'
#' @examples
#' # Just works — beautiful defaults
#' dt2(iris)
#'
#' \donttest{
#' # Override style inline
#' dt2(iris, striped = FALSE)
#' dt2(iris, font_scale = 0.85, compact = FALSE)
#'
#' # Theme presets
#' dt2(iris, theme = "minimal")
#' dt2(iris, theme = "compact")
#'
#' # Reusable theme
#' my_theme <- dt2_theme("clean", compact = TRUE)
#' dt2(iris, theme = my_theme)
#'
#' # Override a preset
#' dt2(iris, theme = "minimal", striped = TRUE)
#'
#' # CSS class override (power users)
#' dt2(iris, class = "table table-bordered table-dark")
#'
#' # DataTables options (separate from styling)
#' dt2(iris, options = list(pageLength = 5, searching = FALSE))
#'
#' # Disable responsive (fixed-width columns)
#' dt2(iris, responsive = FALSE)
#'
#' # Everything composes
#' dt2(mtcars,
#'     theme = "clean",
#'     compact = TRUE,
#'     options = list(pageLength = 25))
#'
#' # Buttons
#' dt2(mtcars, options = list(
#'   buttons = list("copy", "csv", "excel"),
#'   layout = list(topEnd = "buttons")
#' ))
#'
#' # Custom button style
#' dt2(mtcars,
#'     button_class = "btn btn-sm btn-primary",
#'     options = list(
#'       buttons = list("copy", "csv", "excel"),
#'       layout = list(topEnd = "buttons")
#'     ))
#' }
dt2 <- function(data,
                # ---- styling ----
                theme        = "default",
                striped      = NULL,
                hover        = NULL,
                compact      = NULL,
                font_scale   = NULL,
                style        = NULL,
                class        = NULL,
                button_class = NULL,
                # ---- behavior ----
                responsive = TRUE,
                # ---- DataTables config ----
                options    = list(),
                extensions = NULL,
                # ---- widget ----
                width = "100%", height = NULL, elementId = NULL) {

  # ---- Resolve theme ---------------------------------------------------------
  # Priority: direct args > theme object/preset > defaults

  if (inherits(theme, "dt2_theme")) {
    th <- as.list(theme)
  } else if (is.character(theme) && length(theme) == 1) {
    presets <- .dt2_presets()
    th <- presets[[theme]]
    if (is.null(th)) {
      stop("Unknown theme: '", theme,
           "'. Available: ", paste(names(presets), collapse = ", "),
           call. = FALSE)
    }
  } else {
    th <- .dt2_presets()[["default"]]
  }

  # Override with direct args (NULL = use theme value)
  if (!is.null(striped))      th$striped      <- isTRUE(striped)
  if (!is.null(hover))        th$hover        <- isTRUE(hover)
  if (!is.null(compact))      th$compact      <- isTRUE(compact)
  if (!is.null(font_scale))   th$font_scale   <- font_scale
  if (!is.null(style))        th$style        <- match.arg(style, c("bootstrap5", "core"))
  if (!is.null(button_class)) th$button_class <- button_class

  # CSS class override
  th$class <- class  # NULL or string

  # Framework settings
  bs <- th$style %||% "bootstrap5"
  include_bs <- (bs == "bootstrap5")

  # ---- Backward compat: clean up old dt2_theme in options --------------------
  if (!is.null(options$dt2_theme)) {
    old <- options$dt2_theme
    if (is.null(striped) && !is.null(old$striped))       th$striped    <- old$striped
    if (is.null(hover) && !is.null(old$hover))           th$hover      <- old$hover
    if (is.null(compact) && !is.null(old$compact))       th$compact    <- old$compact
    if (is.null(font_scale) && !is.null(old$font_scale)) th$font_scale <- old$font_scale
    options$dt2_theme <- NULL
  }

  # ---- Inject theme for JS side ---------------------------------------------
  options$dt2_theme <- list(
    bs           = bs,
    striped      = th$striped,
    hover        = th$hover,
    compact      = th$compact,
    font_scale   = th$font_scale,
    class        = th$class,
    button_class = th$button_class
  )

  # ---- Responsive default -----------------------------------------------------
  # Enable by default so tables fill 100% width and adapt to screen size.
  # User can opt out: dt2(iris, responsive = FALSE)
  if (isTRUE(responsive) && is.null(options$responsive)) {
    options$responsive <- TRUE
  } else if (identical(responsive, FALSE)) {
    options$responsive <- NULL  # ensure extension is not loaded
  }

  # ---- Column names -----------------------------------------------------------
  # Expose the data's column names as options$columns when the user didn't set
  # them. This is the canonical list that name-based helpers resolve against and
  # is equivalent to the column list dt2.js derives client-side, so it does not
  # change rendering -- it just makes the names available downstream.
  if (is.null(options$columns)) {
    cn <- colnames(data)
    if (!is.null(cn)) options$columns <- cn
  }

  # ---- Extensions auto-detect ------------------------------------------------
  if (is.null(extensions)) {
    extensions <- .dt2_detect_extensions(options)
  }

  # ---- Build payload ---------------------------------------------------------
  x <- list(
    data    = data,
    options = options
  )

  deps <- dt2_deps(
    bs         = bs,
    include_bs = include_bs,
    extensions = extensions
  )

  htmlwidgets::createWidget(
    name = "dt2",
    x, width, height,
    package = "DT2",
    elementId = elementId,
    sizingPolicy = htmlwidgets::sizingPolicy(
      defaultWidth  = "100%",
      defaultHeight = "auto",
      knitr.defaultWidth  = "100%",
      knitr.defaultHeight = "auto",
      browser.fill  = FALSE,
      viewer.fill   = FALSE,
      knitr.figure  = FALSE,
      padding       = 0
    ),
    dependencies = deps
  )
}

# print.dt2_theme() previously omitted the `class` field.

test_that("print.dt2_theme() shows the class field", {
  expect_output(print(dt2_theme("default")), "class")
})

test_that("dt2_theme() applies preset overrides", {
  th <- dt2_theme("minimal", striped = TRUE)
  expect_s3_class(th, "dt2_theme")
  expect_true(th$striped)
  expect_false(th$hover)  # from the 'minimal' preset
})

#' Allow raw HTML rendering via `columns.render`
#'
#' @description Mark columns to render raw HTML using a JS render function.
#' @param options Options list.
#' @param cols Names or 1-based indices.
#' @param js_render JS function (via [htmlwidgets::JS]) with signature `(data, type, row, meta)`
#'   returning a string of HTML when `type == "display"`.
#' @return Updated `options`.
#' @export
dt2_cols_html <- function(options = list(), cols, js_render) {
  cols <- .dt2_name_to_idx(cols, options)
  cds <- lapply(cols, function(i) list(
    targets = i - 1L,
    render  = js_render
  ))
  options$columnDefs <- c(options$columnDefs %||% list(), cds)
  options
}

#' Simple HTML template per column (replace `{{VAL}}`)
#'
#' @param options Options list.
#' @param col Name or index of target column.
#' @param template HTML string with `{{VAL}}` placeholder.
#' @return Updated `options`.
#' @export
dt2_col_template <- function(options = list(), col, template) {
  col <- .dt2_name_to_idx(col, options)
  js <- htmlwidgets::JS(
    sprintf(
      "function(d,t,row,meta){ if(t!=='display') return d; var html=%s; return html.replace(/\\{\\{VAL\\}\\}/g, d); }",
      jsonlite::toJSON(template, auto_unbox = TRUE)
    )
  )
  dt2_cols_html(options, col, js)
}

#' Configure DataTables Buttons and (optionally) move them to a custom container
#'
#' @param options A DT2 `options` list you are building.
#' @param buttons Character vector with button names (e.g. `"copyHtml5"`, `"csvHtml5"`, `"excelHtml5"`, `"pdfHtml5"`, `"print"`).
#'   You can also pass a list with full button objects.
#' @param target Optional CSS selector (e.g. `"#btn-slot"` or `".my-toolbar"`) to receive the
#'   buttons container. If provided, DT2 will move the rendered buttons to that container after init.
#' @return The modified `options` list.
#' @details Requires the **Buttons** extension. For CSV/Excel/PDF you also need **JSZip** and **pdfMake** (incl. `vfs_fonts`).
#'
#'   Prefer [dt2_use_buttons()] for the common case: it takes simple button ids,
#'   styles them with a CSS class, and places them in the layout. Use
#'   `dt2_buttons()` when you need full button objects or want to move the
#'   rendered buttons into a custom container via `target`.
#' @seealso [dt2_use_buttons()]
#' @export
dt2_buttons <- function(options = list(),
                        buttons = c("copyHtml5", "csvHtml5", "excelHtml5", "pdfHtml5", "print"),
                        target  = NULL) {
  options$buttons <- as.list(buttons)
  if (!is.null(target)) options$dt2_buttons_target <- target
  options
}

#' Checkbox input per row
#'
#' @param options Options list.
#' @param col Target column (name or 1-based index).
#' @param input_id_prefix Prefix for element ids (e.g., "row_chk_").
#' @param value_col Optional boolean column to define initial state.
#' @return Updated `options`.
#' @export
dt2_col_checkbox <- function(options = list(), col, input_id_prefix = "row_chk_", value_col = NULL) {
  col <- .dt2_name_to_idx(col, options)

  # HÍBRIDO: lê por índice (array) OU por nome (objeto)
  if (is.null(value_col)) {
    value_js <- "false"
    name_js  <- "null"
  } else {
    if (is.character(value_col)) {
      idx  <- match(value_col, options$columns)
      name <- value_col
    } else {
      idx  <- as.integer(value_col)
      name <- options$columns[[idx]]
    }
    value_js <- sprintf("(Array.isArray(row) ? row[%d] : row[%s])",
                        idx - 1L,
                        jsonlite::toJSON(name, auto_unbox = TRUE))
  }

  js <- htmlwidgets::JS(sprintf(
    "function(d,t,row,meta){ if(t!=='display') return d;
       var rid = '%s' + (meta.row+1);
       var checked = %s ? ' checked' : '';
       return '<input type=\"checkbox\" class=\"dt2-row-checkbox form-check-input\" id=\"'+rid+'\"'+checked+'/>'; }",
    input_id_prefix, value_js))

  dt2_cols_html(options, col, js)
}

#' Action button per row
#'
#' @param options Options list.
#' @param col Target column (name or 1-based index).
#' @param label Button label.
#' @param input_id_prefix Prefix for element ids (e.g., "row_btn_").
#' @return Updated `options`.
#' @export
dt2_col_button <- function(options = list(), col, label = "Action", input_id_prefix = "row_btn_") {
  col <- .dt2_name_to_idx(col, options)
  js <- htmlwidgets::JS(sprintf(
    "function(d,t,row,meta){ if(t!=='display') return d;
       var rid = '%s' + (meta.row+1);
       return '<button type=\"button\" class=\"dt2-row-button btn btn-sm btn-primary\" id=\"'+rid+'\">%s</button>';
     }", input_id_prefix, htmltools::htmlEscape(label)))
  dt2_cols_html(options, col, js)
}

#' Build htmlwidgets dependencies for DT2
#'
#' Constructs the dependency list dynamically based on the requested extensions.
#' Only loads CSS/JS for selected extensions, keeping the page lightweight.
#'
#' All version numbers are read from [.dt2_lib_versions()] (defined in
#' `dt2_check_updates.R`) so there is a single source of truth for versions.
#'
#' @param bs `"bootstrap5"` (default) or `"core"` styling mode.
#' @param include_bs Logical; if TRUE and `bs="bootstrap5"`, include Bootstrap assets.
#'   Default TRUE. Set FALSE only if your host page already provides Bootstrap.
#' @param extensions Character vector of extension names (e.g., `c("Buttons", "Select")`).
#'   Use `dt2_extensions()` to see all available extensions.
#' @return List of `htmlDependency()` objects in correct load order.
#' @keywords internal
#' @importFrom htmltools htmlDependency
dt2_deps <- function(bs = c("bootstrap5", "core"),
                     include_bs = TRUE,
                     extensions = character()) {
  bs <- match.arg(bs)
  bs <- .dt2_lock_mode(bs)

  # Single source of truth for versions
  vers <- .dt2_lib_versions()

  pkg_path <- function(...) {
    p <- system.file(..., package = "DT2")
    if (!nzchar(p)) stop("DT2: resource not found: ", file.path(...), call. = FALSE)
    p
  }

  dep <- function(name, version, src_dir, script = NULL, stylesheet = NULL) {
    htmltools::htmlDependency(
      name = name, version = version, src = c(file = src_dir),
      script = script, stylesheet = stylesheet
    )
  }

  deps <- list()

  # ------------------------------------------------------------------
  # 0) Bootstrap (if bs5 mode and include_bs)
  # ------------------------------------------------------------------
  if (bs == "bootstrap5" && isTRUE(include_bs)) {
    bs_ver <- vers[["Bootstrap"]]
    deps <- append(deps, list(
      dep("bootstrap", bs_ver,
          pkg_path("htmlwidgets", "lib", "bootstrap", bs_ver),
          script = "js/bootstrap.bundle.min.js",
          stylesheet = "css/bootstrap.min.css")
    ))
  }

  # ------------------------------------------------------------------
  # 1) DataTables core CSS
  # ------------------------------------------------------------------
  dt_ver <- vers[["DataTables"]]
  if (bs == "core") {
    deps <- append(deps, list(
      dep("datatables-core-css", dt_ver,
          pkg_path("htmlwidgets", "lib", "datatables", dt_ver),
          stylesheet = "css/dataTables.dataTables.min.css")
    ))
  } else {
    deps <- append(deps, list(
      dep("datatables-bs5-css", dt_ver,
          pkg_path("htmlwidgets", "lib", "datatables", dt_ver),
          stylesheet = "css/dataTables.bootstrap5.min.css")
    ))
  }

  # ------------------------------------------------------------------
  # 2) Extension CSS (only for requested extensions)
  # ------------------------------------------------------------------
  reg <- .dt2_extension_registry()
  ext_names <- .dt2_resolve_extensions(extensions)

  for (ext_name in ext_names) {
    ext <- reg[[ext_name]]
    css_files <- if (bs == "core") ext$css_core else ext$css_bs5
    if (length(css_files) > 0) {
      css_files <- if (is.character(css_files)) css_files else unlist(css_files)
      deps <- append(deps, list(
        dep(paste0("dt-", tolower(ext_name), "-css"), ext$version,
            pkg_path("htmlwidgets", "lib", ext$dir, ext$version),
            stylesheet = css_files)
      ))
    }
  }

  # ------------------------------------------------------------------
  # 3) Common JS libraries (jQuery, moment)
  # ------------------------------------------------------------------
  jq_ver <- vers[["jQuery"]]
  mo_ver <- vers[["Moment"]]

  deps <- append(deps, list(
    dep("jquery", jq_ver,
        pkg_path("htmlwidgets", "lib", "jquery", jq_ver),
        script = "jquery.min.js"),
    dep("moment", mo_ver,
        pkg_path("htmlwidgets", "lib", "moment", mo_ver),
        script = "moment.min.js")
  ))

  # jszip and pdfmake only if Buttons is loaded
  if ("Buttons" %in% ext_names) {
    jz_ver <- vers[["JSZip"]]
    pm_ver <- vers[["PDFMake"]]
    deps <- append(deps, list(
      dep("jszip", jz_ver,
          pkg_path("htmlwidgets", "lib", "jszip", jz_ver),
          script = "jszip.min.js"),
      dep("pdfmake", pm_ver,
          pkg_path("htmlwidgets", "lib", "pdfmake", pm_ver),
          script = c("pdfmake.min.js", "vfs_fonts.js"))
    ))
  }

  # ------------------------------------------------------------------
  # 4) DataTables core JS
  # ------------------------------------------------------------------
  if (bs == "core") {
    deps <- append(deps, list(
      dep("datatables-core-js", dt_ver,
          pkg_path("htmlwidgets", "lib", "datatables", dt_ver),
          script = "js/dataTables.min.js")
    ))
  } else {
    deps <- append(deps, list(
      dep("datatables-core-js", dt_ver,
          pkg_path("htmlwidgets", "lib", "datatables", dt_ver),
          script = c("js/dataTables.min.js", "js/dataTables.bootstrap5.min.js"))
    ))
  }

  # ------------------------------------------------------------------
  # 5) Extension JS (in dependency order)
  # ------------------------------------------------------------------
  for (ext_name in ext_names) {
    ext <- reg[[ext_name]]
    js_files <- if (bs == "core") ext$js_core else ext$js_bs5
    if (length(js_files) > 0) {
      js_files <- if (is.character(js_files)) js_files else unlist(js_files)
      deps <- append(deps, list(
        dep(paste0("dt-", tolower(ext_name), "-js"), ext$version,
            pkg_path("htmlwidgets", "lib", ext$dir, ext$version),
            script = js_files)
      ))
    }
  }

  # ------------------------------------------------------------------
  # 6) DT2 CSS fixes (always last)
  # ------------------------------------------------------------------
  deps <- append(deps, list(
    dep("dt2-fixes", "1.0.0",
        pkg_path("htmlwidgets"),
        stylesheet = "dt2-fixes.css")
  ))

  deps
}

#!/usr/bin/env bash
set -euo pipefail

# =============================================================================
# DT2 Library Downloader
# =============================================================================
# Update the version variables below, then run:
#   bash tools/get-dt2-libs.sh
#
# All files are saved under inst/htmlwidgets/lib/
# =============================================================================

# ---- VERSION CONFIGURATION (edit these to update) ----
JQUERY_VER="3.7.0"
MOMENT_VER="2.29.4"
JSZIP_VER="3.10.1"
PDFMAKE_VER="0.2.7"
BS_VER="5.3.8"

DT_VER="2.3.4"         # DataTables core
BTN_VER="3.2.5"        # Buttons
CR_VER="2.1.1"         # ColReorder
CC_VER="1.1.0"         # ColumnControl
DTM_VER="1.6.0"        # DateTime
FC_VER="5.0.5"         # FixedColumns
FH_VER="4.0.3"         # FixedHeader
KT_VER="2.12.1"        # KeyTable
RS_VER="3.0.6"         # Responsive
RG_VER="1.6.0"         # RowGroup
RR_VER="1.5.0"         # RowReorder
SCR_VER="2.4.3"        # Scroller
SB_VER="1.8.4"         # SearchBuilder
SP_VER="2.3.5"         # SearchPanes
SEL_VER="3.1.0"        # Select
SR_VER="1.4.2"         # StateRestore
# ---- END VERSION CONFIGURATION ----

ROOT="inst/htmlwidgets/lib"
mkdir -p "$ROOT"

fetch() {
  local out="$1"
  local url="$2"
  mkdir -p "$(dirname "$out")"
  echo "  -> $out"
  curl -fsSL "$url" -o "$out"
}

echo "=== DT2 Library Download ==="
echo "Target: $ROOT"
echo ""

# --- jQuery ---
echo "[jQuery $JQUERY_VER]"
fetch "$ROOT/jquery/$JQUERY_VER/jquery.min.js" \
  "https://code.jquery.com/jquery-${JQUERY_VER}.min.js"

# --- Moment.js ---
echo "[Moment $MOMENT_VER]"
fetch "$ROOT/moment/$MOMENT_VER/moment.min.js" \
  "https://cdnjs.cloudflare.com/ajax/libs/moment.js/$MOMENT_VER/moment.min.js"

# --- JSZip ---
echo "[JSZip $JSZIP_VER]"
fetch "$ROOT/jszip/$JSZIP_VER/jszip.min.js" \
  "https://cdnjs.cloudflare.com/ajax/libs/jszip/$JSZIP_VER/jszip.min.js"

# --- pdfmake ---
# NOTE: cdnjs only hosts up to 0.2.12. jsdelivr mirrors npm and has all versions.
echo "[pdfmake $PDFMAKE_VER]"
fetch "$ROOT/pdfmake/$PDFMAKE_VER/pdfmake.min.js" \
  "https://cdn.jsdelivr.net/npm/pdfmake@$PDFMAKE_VER/build/pdfmake.min.js"
fetch "$ROOT/pdfmake/$PDFMAKE_VER/vfs_fonts.js" \
  "https://cdn.jsdelivr.net/npm/pdfmake@$PDFMAKE_VER/build/vfs_fonts.js"

# --- Bootstrap 5 ---
echo "[Bootstrap $BS_VER]"
fetch "$ROOT/bootstrap/$BS_VER/js/bootstrap.bundle.min.js" \
  "https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/$BS_VER/js/bootstrap.bundle.min.js"
fetch "$ROOT/bootstrap/$BS_VER/css/bootstrap.min.css" \
  "https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/$BS_VER/css/bootstrap.min.css"

# --- DataTables core ---
echo "[DataTables $DT_VER]"
fetch "$ROOT/datatables/$DT_VER/js/dataTables.min.js" \
  "https://cdn.datatables.net/$DT_VER/js/dataTables.min.js"
fetch "$ROOT/datatables/$DT_VER/css/dataTables.dataTables.min.css" \
  "https://cdn.datatables.net/$DT_VER/css/dataTables.dataTables.min.css"
fetch "$ROOT/datatables/$DT_VER/js/dataTables.bootstrap5.min.js" \
  "https://cdn.datatables.net/$DT_VER/js/dataTables.bootstrap5.min.js"
fetch "$ROOT/datatables/$DT_VER/css/dataTables.bootstrap5.min.css" \
  "https://cdn.datatables.net/$DT_VER/css/dataTables.bootstrap5.min.css"

# ---- Helper for extensions ----
# Usage: dl_ext "name" "VER" "dir" "js_files" "css_files"
# js_files and css_files are space-separated
dl_ext() {
  local name="$1" ver="$2" dir="$3"
  shift 3
  local js_files="$1" css_files="$2"
  echo "[$name $ver]"
  for f in $js_files; do
    fetch "$ROOT/$dir/$ver/$f" "https://cdn.datatables.net/$dir/$ver/$f"
  done
  for f in $css_files; do
    fetch "$ROOT/$dir/$ver/$f" "https://cdn.datatables.net/$dir/$ver/$f"
  done
}

# --- Buttons ---
echo "[Buttons $BTN_VER]"
for f in js/dataTables.buttons.min.js js/buttons.colVis.min.js js/buttons.html5.min.js js/buttons.print.min.js js/buttons.bootstrap5.min.js; do
  fetch "$ROOT/buttons/$BTN_VER/$f" "https://cdn.datatables.net/buttons/$BTN_VER/$f"
done
for f in css/buttons.dataTables.min.css css/buttons.bootstrap5.min.css; do
  fetch "$ROOT/buttons/$BTN_VER/$f" "https://cdn.datatables.net/buttons/$BTN_VER/$f"
done

# --- ColReorder ---
dl_ext "ColReorder" "$CR_VER" "colreorder" \
  "js/dataTables.colReorder.min.js" \
  "css/colReorder.dataTables.min.css css/colReorder.bootstrap5.min.css"

# --- ColumnControl ---
echo "[ColumnControl $CC_VER]"
for f in js/dataTables.columnControl.min.js js/columnControl.bootstrap5.min.js; do
  fetch "$ROOT/columncontrol/$CC_VER/$f" "https://cdn.datatables.net/columncontrol/$CC_VER/$f"
done
for f in css/columnControl.dataTables.min.css css/columnControl.bootstrap5.min.css; do
  fetch "$ROOT/columncontrol/$CC_VER/$f" "https://cdn.datatables.net/columncontrol/$CC_VER/$f"
done

# --- DateTime ---
dl_ext "DateTime" "$DTM_VER" "datetime" \
  "js/dataTables.dateTime.min.js" \
  "css/dataTables.dateTime.min.css"

# --- FixedColumns ---
dl_ext "FixedColumns" "$FC_VER" "fixedcolumns" \
  "js/dataTables.fixedColumns.min.js" \
  "css/fixedColumns.dataTables.min.css css/fixedColumns.bootstrap5.min.css"

# --- FixedHeader ---
dl_ext "FixedHeader" "$FH_VER" "fixedheader" \
  "js/dataTables.fixedHeader.min.js" \
  "css/fixedHeader.dataTables.min.css css/fixedHeader.bootstrap5.min.css"

# --- KeyTable ---
dl_ext "KeyTable" "$KT_VER" "keytable" \
  "js/dataTables.keyTable.min.js" \
  "css/keyTable.dataTables.min.css css/keyTable.bootstrap5.min.css"

# --- Responsive ---
echo "[Responsive $RS_VER]"
for f in js/dataTables.responsive.min.js js/responsive.bootstrap5.js; do
  fetch "$ROOT/responsive/$RS_VER/$f" "https://cdn.datatables.net/responsive/$RS_VER/$f"
done
for f in css/responsive.dataTables.min.css css/responsive.bootstrap5.min.css; do
  fetch "$ROOT/responsive/$RS_VER/$f" "https://cdn.datatables.net/responsive/$RS_VER/$f"
done

# --- RowGroup ---
dl_ext "RowGroup" "$RG_VER" "rowgroup" \
  "js/dataTables.rowGroup.min.js" \
  "css/rowGroup.dataTables.min.css css/rowGroup.bootstrap5.min.css"

# --- RowReorder ---
dl_ext "RowReorder" "$RR_VER" "rowreorder" \
  "js/dataTables.rowReorder.min.js" \
  "css/rowReorder.dataTables.min.css css/rowReorder.bootstrap5.min.css"

# --- Scroller ---
dl_ext "Scroller" "$SCR_VER" "scroller" \
  "js/dataTables.scroller.min.js" \
  "css/scroller.dataTables.min.css css/scroller.bootstrap5.min.css"

# --- SearchBuilder ---
echo "[SearchBuilder $SB_VER]"
for f in js/dataTables.searchBuilder.min.js js/searchBuilder.bootstrap5.min.js; do
  fetch "$ROOT/searchbuilder/$SB_VER/$f" "https://cdn.datatables.net/searchbuilder/$SB_VER/$f"
done
for f in css/searchBuilder.dataTables.min.css css/searchBuilder.bootstrap5.min.css; do
  fetch "$ROOT/searchbuilder/$SB_VER/$f" "https://cdn.datatables.net/searchbuilder/$SB_VER/$f"
done

# --- SearchPanes ---
echo "[SearchPanes $SP_VER]"
for f in js/dataTables.searchPanes.min.js js/searchPanes.bootstrap5.min.js; do
  fetch "$ROOT/searchpanes/$SP_VER/$f" "https://cdn.datatables.net/searchpanes/$SP_VER/$f"
done
for f in css/searchPanes.dataTables.min.css css/searchPanes.bootstrap5.min.css; do
  fetch "$ROOT/searchpanes/$SP_VER/$f" "https://cdn.datatables.net/searchpanes/$SP_VER/$f"
done

# --- Select ---
dl_ext "Select" "$SEL_VER" "select" \
  "js/dataTables.select.min.js" \
  "css/select.dataTables.min.css css/select.bootstrap5.min.css"

# --- StateRestore ---
echo "[StateRestore $SR_VER]"
for f in js/dataTables.stateRestore.min.js js/stateRestore.bootstrap5.min.js; do
  fetch "$ROOT/staterestore/$SR_VER/$f" "https://cdn.datatables.net/staterestore/$SR_VER/$f"
done
for f in css/stateRestore.dataTables.min.css css/stateRestore.bootstrap5.min.css; do
  fetch "$ROOT/staterestore/$SR_VER/$f" "https://cdn.datatables.net/staterestore/$SR_VER/$f"
done

echo ""
echo "=== Done! ==="
echo ""
echo "Installed versions:"
echo "  DataTables   $DT_VER"
echo "  jQuery       $JQUERY_VER"
echo "  Bootstrap    $BS_VER"
echo "  Buttons      $BTN_VER"
echo "  ColReorder   $CR_VER"
echo "  ColumnControl $CC_VER"
echo "  DateTime     $DTM_VER"
echo "  FixedColumns $FC_VER"
echo "  FixedHeader  $FH_VER"
echo "  KeyTable     $KT_VER"
echo "  Responsive   $RS_VER"
echo "  RowGroup     $RG_VER"
echo "  RowReorder   $RR_VER"
echo "  Scroller     $SCR_VER"
echo "  SearchBuilder $SB_VER"
echo "  SearchPanes  $SP_VER"
echo "  Select       $SEL_VER"
echo "  StateRestore $SR_VER"
echo ""
echo "Remember to update version strings in R/dt2_extensions.R and R/dt2_deps.R"
echo "if you changed any version above."

---
title: "Shiny Integration"
author: "DT2 Team"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Shiny Integration}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

All examples below are **complete Shiny apps** using `bslib` with Bootstrap 5.
Copy, paste, and run.


## Basic Usage

The minimal DT2 Shiny app uses `dt2_output()` in the UI and
`render_dt2()` in the server. DT2 tables integrate naturally with
`bslib` cards and Bootstrap 5 themes — the table inherits the page
theme automatically:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("DT2 in Shiny"),
    card_body(dt2_output("my_table"))
  )
)

server <- function(input, output, session) {
  output$my_table <- render_dt2({
    dt2(iris, options = list(pageLength = 10))
  })
}

shinyApp(ui, server)
```


## Reading Table State

Every DT2 table pushes its state (`order`, `search`, `page`,
`selected` rows) to `input$<id>_state` automatically:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Table State"),
    card_body(dt2_output("tbl"))
  ),
  card(
    card_header("Current state"),
    card_body(verbatimTextOutput("state_info"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    dt2(iris, options = list(pageLength = 10))
  })

  output$state_info <- renderPrint({
    state <- input$tbl_state
    if (is.null(state)) return("(interact with the table)")
    cat("Reason:", state$reason, "\n")
    cat("Page:",   state$page$page + 1, "of", state$page$pages, "\n")
    cat("Search:", if (nzchar(state$search)) state$search else "(none)", "\n")
    cat("Order:",  paste(state$order, collapse = ", "), "\n")
  })
}

shinyApp(ui, server)
```


## Proxy: Server-Side Manipulation

Use `dt2_proxy()` to update a table without re-rendering it. This is
essential for responsive dashboards: you can replace the data, change
the page, adjust sorting, or update the search filter — all without the
flicker of a full re-render:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_sidebar(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  title = "Proxy Demo",
  sidebar = sidebar(
    actionButton("refresh",  "Refresh data", class = "btn-sm btn-outline-primary w-100 mb-2"),
    actionButton("go_last",  "Go to last page", class = "btn-sm btn-outline-primary w-100 mb-2"),
    actionButton("sort_sl",  "Sort Sepal.Length \u2193", class = "btn-sm btn-outline-primary w-100 mb-2"),
    hr(),
    textInput("search_text", "Search", placeholder = "e.g. setosa"),
    actionButton("search_btn", "Apply", class = "btn-sm btn-primary w-100")
  ),
  card(
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    dt2(iris, options = list(pageLength = 10))
  })

  proxy <- dt2_proxy("tbl")

  observeEvent(input$refresh, {
    new_data <- iris[sample(nrow(iris), 50), ]
    dt2_replace_data(proxy, new_data)
  })

  observeEvent(input$go_last, {
    dt2_proxy_page(proxy, "last")
  })

  observeEvent(input$sort_sl, {
    dt2_proxy_order(proxy, c("Sepal.Length", "desc"),
                    columns = names(iris))
  })

  observeEvent(input$search_btn, {
    dt2_proxy_search(proxy, input$search_text)
  })
}

shinyApp(ui, server)
```


## Buttons with Custom Style

Override the default button appearance with any Bootstrap 5 button class.
The `dt2_use_buttons()` helper accepts a `button_class` parameter that
applies to all buttons in the group. Here, solid primary blue buttons
replace the default outlined style:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Styled Buttons"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    opts <- dt2_use_buttons(
      buttons = c("copy", "csv", "excel", "print"),
      position = "topEnd",
      button_class = "btn btn-sm btn-primary"
    )
    dt2(mtcars, options = opts)
  })
}

shinyApp(ui, server)
```


## Infinite Scrolling (Scroller)

Use virtual scrolling instead of pagination for large datasets. The
Scroller extension renders only the visible rows inside a fixed-height
container; more rows load seamlessly as the user scrolls. Combined with
`deferRender = TRUE`, this keeps memory usage low even for tens of
thousands of rows:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Scroller \u2014 10 000 rows"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  big <- data.frame(
    id    = 1:10000,
    value = round(rnorm(10000), 3),
    group = sample(LETTERS[1:5], 10000, replace = TRUE)
  )

  output$tbl <- render_dt2({
    dt2(big, options = list(
      scroller    = TRUE,
      scrollY     = 400,
      deferRender = TRUE,
      buttons     = list("copy", "csv"),
      layout      = list(
        topStart = list(search = list(placeholder = "Filter...")),
        topEnd   = "buttons"
      )
    ))
  })
}

shinyApp(ui, server)
```


## Custom Layout

DataTables v2 uses `layout` to position toolbar elements around the
table. Each slot (`topStart`, `topEnd`, `bottomStart`, `bottomEnd`)
accepts a string, list, or `NULL`. This example rearranges search,
buttons, info, and pagination into non-default positions:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Custom Layout"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    dt2(iris, options = list(
      pageLength = 10,
      layout = list(
        topStart  = "pageLength",
        topEnd    = list(
          buttons = list("copy", "csv", "excel"),
          search  = list(placeholder = "Search...")
        ),
        bottomStart = "info",
        bottomEnd   = "paging"
      )
    ))
  })
}

shinyApp(ui, server)
```


## Pagination Types

DataTables v2 configures pagination through the `layout` option using
`paging` as a named list, replacing the deprecated `pagingType`
parameter. This example shows three common configurations side by side:
simple navigation (prev/next only), full navigation with all elements,
and no pagination at all.

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  layout_columns(
    col_widths = c(6, 6),
    card(
      card_header("Simple (prev / next only)"),
      card_body(dt2_output("tbl_simple"))
    ),
    card(
      card_header("Full (first / 1 2 3 / last)"),
      card_body(dt2_output("tbl_full"))
    )
  ),
  card(
    card_header("No pagination"),
    card_body(dt2_output("tbl_none"))
  )
)

server <- function(input, output, session) {
  output$tbl_simple <- render_dt2({
    dt2(iris, options = list(
      pageLength = 5,
      layout = list(bottomEnd = list(paging = list(
        type = "simple"
      )))
    ))
  })
  output$tbl_full <- render_dt2({
    dt2(iris, options = list(
      pageLength = 5,
      layout = list(bottomEnd = list(paging = list(
        firstLast = TRUE, previousNext = TRUE, numbers = TRUE
      )))
    ))
  })
  output$tbl_none <- render_dt2({
    dt2(iris[1:15, ], options = list(paging = FALSE))
  })
}

shinyApp(ui, server)
```


## Inline Inputs (Checkboxes and Buttons)

DT2 can embed Shiny inputs (checkboxes, action buttons) directly inside
table cells. When the user clicks a checkbox or button, DT2 sends the
event to Shiny via `input$<id>_state` with `reason = "input"`. This
pattern is useful for row-level actions like selection, approval
workflows, or delete buttons:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Inline Inputs"),
    card_body(dt2_output("tbl"))
  ),
  card(
    card_header("Events"),
    card_body(verbatimTextOutput("events"))
  )
)

server <- function(input, output, session) {
  df <- data.frame(
    name   = c("Alice", "Bob", "Carol", "Dave"),
    score  = c(95, 82, 91, 78),
    select = NA,
    action = NA
  )

  output$tbl <- render_dt2({
    opts <- list(columns = names(df), pageLength = 10)
    opts <- dt2_col_checkbox(opts, "select")
    opts <- dt2_col_button(opts, "action", label = "View")
    dt2(df, options = opts)
  })

  event_log <- reactiveVal("")

  observeEvent(input$tbl_row_check, {
    info <- input$tbl_row_check
    msg <- paste0("Checkbox row ", info$row, ": ", info$value, "\n")
    event_log(paste0(event_log(), msg))
  })

  observeEvent(input$tbl_row_button, {
    info <- input$tbl_row_button
    msg <- paste0("Button clicked row ", info$row, "\n")
    event_log(paste0(event_log(), msg))
  })

  output$events <- renderText(event_log())
}

shinyApp(ui, server)
```


## Server-Side Processing

For very large datasets (100k+ rows), use server-side processing. Only
the visible page is sent to the browser — sorting, filtering, and
pagination happen on the R server. This avoids sending the entire dataset
to the client, dramatically improving initial load time:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  card(
    card_header("Server-Side Processing \u2014 100k rows"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  large_data <- data.frame(
    id    = seq_len(100000),
    value = round(rnorm(100000), 4),
    group = sample(LETTERS[1:5], 100000, replace = TRUE)
  )

  dt2_bind_server("tbl", large_data)

  output$tbl <- render_dt2({
    dt2(large_data, options = list(
      columns      = names(large_data),
      pageLength   = 25,
      server_side  = TRUE,
      language     = list(info = "Showing _START_ to _END_ of _TOTAL_ rows")
    ))
  })
}

shinyApp(ui, server)
```


## Dataset Switcher with bslib

A common dashboard pattern: let the user choose a dataset and a theme
preset from the sidebar. The table re-renders reactively whenever either
input changes. This demonstrates how DT2 integrates naturally with
`bslib`'s `page_sidebar()` layout:

```{r}
library(shiny)
library(bslib)
library(DT2)

ui <- page_sidebar(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  title = "DT2 + bslib",
  sidebar = sidebar(
    selectInput("dataset", "Dataset",
                c("iris", "mtcars", "airquality")),
    selectInput("theme_preset", "Theme preset",
                c("default", "clean", "minimal", "compact"))
  ),
  card(
    card_header("Table"),
    card_body(dt2_output("tbl"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({
    data <- switch(input$dataset,
      iris       = iris,
      mtcars     = mtcars,
      airquality = airquality
    )
    dt2(data,
        theme = input$theme_preset,
        options = list(pageLength = 10))
  })
}

shinyApp(ui, server)
```


## Shared Theme Across Tables

Define a theme once with `dt2_theme()` and reuse it across all tables in
your app. This ensures consistent font scaling, button styling, and
compactness without repeating configuration. Changes propagate instantly
to every table that uses the theme:

```{r}
library(shiny)
library(bslib)
library(DT2)

my_theme <- dt2_theme("clean",
  compact      = TRUE,
  font_scale   = 0.85,
  button_class = "btn btn-sm btn-outline-dark"
)

ui <- page_fillable(
  theme = bs_theme(version = 5, bootswatch = "flatly"),
  padding = "1rem",
  layout_columns(
    col_widths = c(6, 6),
    card(
      card_header("iris"),
      card_body(dt2_output("tbl1"))
    ),
    card(
      card_header("mtcars"),
      card_body(dt2_output("tbl2"))
    )
  )
)

server <- function(input, output, session) {
  output$tbl1 <- render_dt2({
    dt2(iris, theme = my_theme, options = dt2_use_buttons(
      buttons = c("copy", "csv")
    ))
  })

  output$tbl2 <- render_dt2({
    dt2(mtcars, theme = my_theme, options = list(pageLength = 15))
  })
}

shinyApp(ui, server)
```

## Complete Example: Custom Renderers, Flags, and ColumnControl

This Shiny app demonstrates most DT2 features together: ColumnControl
dropdowns, export buttons with spacer, custom JS renderers (links,
flags, progress bars, colored currency), and full pt-BR translation.

Copy and run it directly — it is also available at
`system.file("examples/app_complete.R", package = "DT2")`.

```{r}
library(jsonlite)
library(dplyr)
library(tibble)
library(lubridate)

# ── Sample data (57 employees) ────────────────────────────────────────────────
json_url <- "https://raw.githubusercontent.com/StrategicProjects/DT2/main/inst/examples/employees.json"
# For offline use, the same JSON is bundled in inst/examples/employees.json

json_txt <- '{
  "data": [
    {"name":"Tiger Nixon","position":"System Architect","salary":"320800","start_date":"2011-04-25","office":"Edinburgh","extn":"5421"},
    {"name":"Garrett Winters","position":"Accountant","salary":"170750","start_date":"2011-07-25","office":"Tokyo","extn":"8422"},
    {"name":"Ashton Cox","position":"Junior Technical Author","salary":"86000","start_date":"2009-01-12","office":"San Francisco","extn":"1562"},
    {"name":"Cedric Kelly","position":"Senior JavaScript Developer","salary":"433060","start_date":"2012-03-29","office":"Edinburgh","extn":"6224"},
    {"name":"Airi Satou","position":"Accountant","salary":"162700","start_date":"2008-11-28","office":"Tokyo","extn":"5407"},
    {"name":"Brielle Williamson","position":"Integration Specialist","salary":"372000","start_date":"2012-12-02","office":"New York","extn":"4804"},
    {"name":"Herrod Chandler","position":"Sales Assistant","salary":"137500","start_date":"2012-08-06","office":"San Francisco","extn":"9608"},
    {"name":"Rhona Davidson","position":"Integration Specialist","salary":"327900","start_date":"2010-10-14","office":"Tokyo","extn":"6200"},
    {"name":"Colleen Hurst","position":"JavaScript Developer","salary":"205500","start_date":"2009-09-15","office":"San Francisco","extn":"2360"},
    {"name":"Sonya Frost","position":"Software Engineer","salary":"103600","start_date":"2008-12-13","office":"Edinburgh","extn":"1667"}
  ]
}'

df <- fromJSON(json_txt, flatten = TRUE)$data %>%
  as_tibble() %>%
  mutate(
    salary     = as.numeric(salary),
    extn       = as.integer(extn),
    start_date = ymd(start_date)
  )

# ── Shiny App ─────────────────────────────────────────────────────────────────
library(shiny)
library(bslib)
library(DT2)
library(htmlwidgets)

ui <- page_sidebar(
  theme = bs_theme(version = 5, bootswatch = "spacelab"),
  title = "DT2 — Complete Example",
  sidebar = sidebar(
    h5("Features"),
    tags$ul(
      tags$li("ColumnControl (order + search dropdowns)"),
      tags$li("Export buttons with spacer"),
      tags$li("Custom JS renderers"),
      tags$li("pt-BR translation")
    )
  ),
  # Flag sprites
  tags$head(
    tags$link(
      rel = "stylesheet", type = "text/css",
      href = "https://cdn.jsdelivr.net/gh/lafeber/world-flags-sprite/stylesheets/flags32-both.css"
    ),
    tags$style(HTML("
      .f32 .flag { display:inline-block; width:32px; height:32px;
                    vertical-align:middle; margin-right:6px; }
      table.dataTable tbody td { vertical-align: middle; }
    "))
  ),
  card(
    card_header("Employee Table"),
    card_body(dt2_output("tbl", height = "auto"))
  )
)

server <- function(input, output, session) {
  output$tbl <- render_dt2({

    # ── JS Renderers ──────────────────────────────────────────
    office_js <- JS("
      function(data, type) {
        if (type !== 'display') return data;
        var cc = {Argentina:'ar', Edinburgh:'_Scotland', London:'_England',
                  'New York':'us', 'San Francisco':'us', Sydney:'au', Tokyo:'jp'};
        var flag = cc[data] || '';
        return '<span class=\"flag ' + flag + '\"></span> ' + data;
      }
    ")

    salary_js <- JS("
      (function() {
        var nfmt = DataTable.render.number('.', ',', 2, 'R$ ');
        return function(data, type) {
          var txt = nfmt.display(data);
          if (type !== 'display') return txt;
          var c = data < 250000 ? 'red' : data < 500000 ? 'orange' : 'green';
          return '<span style=\"color:' + c + '\">' + txt + '</span>';
        };
      })()
    ")

    extn_js <- JS("
      function(data, type) {
        return type === 'display'
          ? '<progress value=\"' + data + '\" max=\"9999\"></progress>'
          : data;
      }
    ")

    # ── Options ───────────────────────────────────────────────
    opts <- list(
      pageLength = 10,
      lengthMenu = c(10, 25, -1),
      columns    = names(df),

      layout = list(
        topStart = "pageLength",
        topEnd   = list(
          buttons = list(
            list(extend = "copyHtml5", text = "Copiar"),
            list(extend = "csvHtml5"),
            list(extend = "excelHtml5"),
            list(extend = "spacer", style = "bar"),
            list(extend = "colvis", text = "Colunas")
          ),
          search = list(placeholder = "")
        ),
        bottomEnd = "paging"
      ),

      columnControl = list(
        target  = 0,
        content = list("order", "searchDropdown", list(
          list(extend = "orderAsc", text = "Ordem crescente"),
          list(extend = "orderDesc", text = "Ordem decrescente"),
          "spacer",
          list(extend = "colVisDropdown", text = "Selecionar colunas")
        ))
      ),
      ordering = list(indicators = FALSE, handler = FALSE),

      columnDefs = list(
        list(targets = which(names(df) == "office") - 1L,
             className = "f32", render = office_js),
        list(targets = which(names(df) == "salary") - 1L,
             className = "dt-body-right", render = salary_js),
        list(targets = which(names(df) == "extn") - 1L,
             render = extn_js)
      ),

      language = list(
        lengthMenu   = "Mostrar _MENU_",
        search       = "Buscar",
        info         = "Mostrando _START_ a _END_ de _TOTAL_ registros",
        infoEmpty    = "Nenhum registro",
        zeroRecords  = "Nenhum registro encontrado",
        emptyTable   = "Nenhum dado disponível",
        decimal      = ",", thousands = ".", infoThousands = ".",
        lengthLabels = list(`10` = "10", `25` = "25", `-1` = "Todas"),
        paginate     = list(first="«", previous="‹", `next`="›", last="»"),
        buttons = list(
          copyTitle   = "Copiado!",
          copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
        ),
        columnControl = list(
          orderAsc = "Crescente", orderDesc = "Decrescente",
          searchDropdown = "Pesquisar", colVisDropdown = "Colunas",
          searchClear = "Limpar",
          search = list(
            text   = list(contains="Contém", starts="Começa por",
                          ends="Termina em", equal="Igual a"),
            number = list(greater="Maior que", less="Menor que",
                          equal="Igual a")
          )
        )
      )
    )

    dt2(df,
        compact = TRUE, striped = TRUE, hover = TRUE,
        font_scale = 0.85, responsive = FALSE,
        options = opts)
  })
}

shinyApp(ui, server)
```

---
title: "Getting Started with DT2"
author: "DT2 Team"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Getting Started with DT2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)
```

## Overview

**DT2** is a modern R binding for [DataTables v2](https://datatables.net/)
built on `htmlwidgets`. It works **with or without Shiny** — render
interactive tables in R Markdown, Quarto, and the RStudio Viewer.

With zero configuration, DT2 produces a Bootstrap 5 styled table with
search, sorting, pagination, and the Jost font — all from a single
function call:

```{r}
# Just works — Bootstrap 5, Jost font, striped, compact
dt2(iris)
```


## Responsive Tables

DT2 enables the Responsive extension by default. Tables fill 100% of the
available width and gracefully adapt to narrow screens by collapsing
columns that don't fit. This is essential for dashboards and reports
viewed on mobile devices or embedded in narrow layout containers:

```{r}
# Responsive is ON by default — table fills the container
dt2(mtcars[1:10, ])
```

On wide screens all columns are visible. On narrow screens, columns that
overflow are hidden and accessible via a toggle (`+`) on each row — the
user can expand any row to reveal the hidden columns in a child row.

To disable responsive behaviour (for example, when using custom renderers
that inject complex HTML, or when you prefer horizontal scrolling with
`scrollX = TRUE`):

```{r}
dt2(iris, responsive = FALSE)
```


## Styling

DT2 provides multiple levels of control over table appearance, from
quick inline parameters to reusable theme objects. All styling options
cascade: direct arguments override theme values, which override the
built-in defaults.

### Inline overrides

Style parameters are first-class arguments of `dt2()`. Override any
individual setting without creating a theme object — useful for quick
one-off adjustments:

```{r}
dt2(iris, striped = FALSE, hover = FALSE, font_scale = 1.0)
```

### Theme presets

DT2 ships with four built-in presets that cover common use cases. Each
preset adjusts striping, compactness, and font scaling to match a
particular aesthetic:

| Preset | Striped | Compact | Font scale |
|---------|---------|---------|------------|
| `"default"` | ✓ | ✓ | 0.80 |
| `"clean"` | ✓ | — | 0.85 |
| `"minimal"` | — | — | 0.90 |
| `"compact"` | ✓ | ✓ | 0.75 |

```{r}
dt2(iris, theme = "minimal", options = list(pageLength = 5))
```

### Reusable themes

Create a theme once, apply it to many tables:

```{r}
my_theme <- dt2_theme("clean", compact = TRUE, font_scale = 0.80)
my_theme
```

```{r}
dt2(iris[1:10, ], theme = my_theme)
```

### CSS class override

For full control, pass a raw CSS class string:

```r
dt2(iris, class = "table table-bordered table-dark table-sm")
```


## Table Layout

DataTables 2 uses `layout` to control where elements (search box, page
length selector, buttons, pagination, info text) appear around the table.
This replaces the old `dom` string from DataTables v1 with a clean,
named-position system that is much easier to read and maintain.

Each position is a slot in a grid above and below the table. You can
place any element in any slot, combine multiple elements in the same
slot, or set a slot to `NULL` to remove it entirely.

### Layout positions

```
+------------------+------------------+
| topStart         | topEnd           |
+------------------+------------------+
| top2Start        | top2End          |
+------------------+------------------+
|               TABLE                 |
+------------------+------------------+
| bottomStart      | bottomEnd        |
+------------------+------------------+
| bottom2Start     | bottom2End       |
+------------------+------------------+
```

### Default layout

By default, DataTables places:

- **topStart**: page length selector
- **topEnd**: search box
- **bottomStart**: info ("Showing 1 to 10 of 150 entries")
- **bottomEnd**: pagination

### Rearranging elements

Move the search box to the left and info to the right:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart    = "search",
    topEnd      = "pageLength",
    bottomStart = "paging",
    bottomEnd   = "info"
  )
))
```

### Removing elements

Set a position to `NULL` to remove it:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart  = NULL,         # remove page length selector
    bottomEnd = "paging"      # keep only pagination
  )
))
```

### Multiple elements in one position

Wrap multiple elements in a list:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart = list("pageLength", "info"),
    topEnd   = "search",
    bottomEnd = "paging"
  )
))
```

### Search box placeholder

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topEnd = list(search = list(placeholder = "Type to filter..."))
  )
))
```


## Buttons (Export / Copy / Print)

The Buttons extension adds one-click export functionality to your tables.
Users can copy data to the clipboard, download as CSV/Excel/PDF, or send
the table directly to the printer. DT2 auto-detects when buttons are
configured and loads the required JavaScript and CSS automatically — no
need to specify `extensions = "Buttons"` manually.

### Quick setup

The simplest way to add export buttons — just list the button types you
want and place them in the layout:

```{r}
dt2(mtcars[1:15, ], options = list(
  pageLength = 8,
  buttons = list("copy", "csv", "excel"),
  layout  = list(topEnd = "buttons")
))
```

DT2 auto-detects that buttons are needed and loads the Buttons extension.

### Using the helper

`dt2_use_buttons()` builds the options for you:

```{r}
opts <- dt2_use_buttons(
  buttons  = c("copy", "csv", "excel", "pdf", "print"),
  position = "topEnd"
)
dt2(iris[1:20, ], options = opts)
```

### Buttons + search together

Place buttons and search in the same row:

```{r}
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  buttons = list("copy", "csv", "excel"),
  layout = list(
    topStart = "buttons",
    topEnd   = list(search = list(placeholder = "Filter...")),
    bottomEnd = "paging"
  )
))
```

### Grouped buttons with dropdown

For tables with many export options, group related buttons inside a
dropdown menu using `extend = "collection"`. This keeps the toolbar
compact while still offering all export formats. The `colvis` button
lets users toggle column visibility on the fly:

```{r}
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topEnd = list(
      buttons = list(
        list(extend = "collection", text = "Export",
             buttons = list("copyHtml5", "csvHtml5", "excelHtml5", "pdfHtml5")),
        list(extend = "spacer", style = "bar"),
        "print",
        list(extend = "spacer", style = "bar"),
        list(extend = "colvis", text = "Columns")
      )
    )
  )
))
```

### Separated button groups with spacer

Use `extend = "spacer"` to visually separate groups of buttons.
The `style = "bar"` variant adds a vertical divider:

```{r}
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topEnd = list(
      buttons = list(
        "copy", "csv", "excel",
        list(extend = "spacer", style = "bar"),
        list(extend = "colvis", text = "Columns")
      )
    )
  )
))
```

### Bottom buttons

```{r}
dt2(mtcars[1:10, ], options = list(
  buttons = list("copy", "csv"),
  layout  = list(
    topEnd      = "search",
    bottomStart = "buttons",
    bottomEnd   = "paging"
  )
))
```

### Button styling

DT2 defaults to compact, outlined buttons (`btn btn-sm btn-outline-secondary`).
Override globally via `button_class`:

```{r}
# Primary blue buttons
dt2(iris[1:10, ],
    button_class = "btn btn-sm btn-primary",
    options = list(
      buttons = list("copy", "csv", "excel"),
      layout = list(topEnd = "buttons")
    ))
```

Or per-table with the helper:

```{r}
opts <- dt2_use_buttons(
  buttons = c("copy", "csv", "excel"),
  button_class = "btn btn-sm btn-outline-dark"
)
dt2(iris[1:10, ], options = opts)
```

You can also set it once in a reusable theme:

```r
my_theme <- dt2_theme("default", button_class = "btn btn-sm btn-primary")
dt2(iris, theme = my_theme, options = dt2_use_buttons())
dt2(mtcars, theme = my_theme, options = dt2_use_buttons())
```

Common Bootstrap 5 button classes:

| Class | Look |
|-------|------|
| `btn btn-sm btn-outline-secondary` | Light outline (default) |
| `btn btn-sm btn-outline-dark` | Dark outline |
| `btn btn-sm btn-primary` | Solid blue |
| `btn btn-sm btn-secondary` | Solid grey |
| `btn btn-sm btn-light` | Light background |


### Pagination styling

DataTables v2 configures pagination through the `layout` option, using
`paging` as a named list. This replaces the deprecated `pagingType`
parameter and gives you fine-grained control over which navigation
elements appear: page number buttons, previous/next arrows, first/last
buttons, and how many page-number buttons to show at once.

The default pagination includes previous/next arrows and page numbers.
DT2 uses Bootstrap 5 `.page-link` classes, so pagination automatically
inherits your Bootswatch theme colours.

```{r}
# Default pagination — compact, themed by Bootstrap
dt2(iris)
```

To remove pagination entirely (show all rows at once), set `paging = FALSE`
at the root level of `options`. This is useful for small datasets where the
user should see everything without navigating:

```{r}
dt2(iris[1:10, ], options = list(
  paging = FALSE
))
```

**Simple — previous / next only (no page numbers).**
Useful for mobile-friendly interfaces or when exact page numbers are not
meaningful:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      type = "simple"
    ))
  )
))
```

**Hide page numbers but keep first / last buttons.**
Lets users jump to the beginning or end of large datasets without
cluttering the interface with numbered buttons:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      numbers   = FALSE,
      firstLast = TRUE
    ))
  )
))
```

**Limit how many page number buttons are shown.**
For datasets with many pages this prevents the pagination bar from
becoming too wide — DataTables inserts ellipsis (`…`) to indicate
hidden pages:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      buttons = 3
    ))
  )
))
```

**Hide boundary numbers (first/last page) but keep arrows.**
The first and last page numbers are removed, leaving only the
surrounding page numbers and navigation arrows:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      boundaryNumbers = FALSE
    ))
  )
))
```

**Full — first / prev / numbers / next / last.**
Shows every navigation element for maximum control. This is equivalent
to the deprecated `pagingType = "full_numbers"`:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomEnd = list(paging = list(
      firstLast    = TRUE,
      previousNext = TRUE,
      numbers      = TRUE
    ))
  )
))
```

#### Quick reference

| Behaviour | `layout` paging option |
|---|---|
| Simple (← →) | `list(type = "simple")` |
| No numbers (« ‹ › ») | `list(numbers = FALSE, firstLast = TRUE)` |
| Limit buttons | `list(buttons = 3)` |
| No boundary numbers | `list(boundaryNumbers = FALSE)` |
| No first/last | `list(firstLast = FALSE)` |
| Full with all elements | `list(firstLast = TRUE, previousNext = TRUE, numbers = TRUE)` |
| No pagination | Set `paging = FALSE` at root level |

> **Note:** `pagingType` (e.g. `"simple"`, `"full_numbers"`) still works
> but is deprecated in DataTables v2. Prefer the `layout` approach above.

**Move pagination to the left.**
Sometimes it makes sense to swap info and pagination — for example, when
the pagination controls should align with left-side content:

```{r}
dt2(iris, options = list(
  layout = list(
    bottomStart = "paging",
    bottomEnd   = "info"
  )
))
```


## Infinite Scrolling (Scroller)

Instead of pagination, you can use virtual scrolling for large datasets.
The Scroller extension renders only the visible rows and loads more as the
user scrolls — ideal for datasets with thousands of rows where pagination
would create an overwhelming number of pages.

The table container gets a fixed height (set via `scrollY` in pixels) and
rows are rendered on demand as the user scrolls, keeping memory usage low
even for very large datasets:

```{r}
dt2(iris, options = list(
  scroller  = TRUE,
  scrollY   = 300,         # viewport height in pixels
  paging    = TRUE         # required for Scroller
))
```

DT2 auto-detects the Scroller extension when `scroller = TRUE` is set.

With a larger dataset:

```{r}
big <- data.frame(
  id    = 1:5000,
  value = round(rnorm(5000), 3),
  group = sample(LETTERS[1:5], 5000, replace = TRUE)
)
dt2(big, options = list(
  scroller    = TRUE,
  scrollY     = 400,
  deferRender = TRUE     # improves performance: renders rows on demand
))
```

Combine with buttons:

```{r}
dt2(big, options = list(
  scroller    = TRUE,
  scrollY     = 350,
  deferRender = TRUE,
  buttons     = list("copy", "csv", "excel"),
  layout      = list(
    topEnd    = "buttons",
    topStart  = list(search = list(placeholder = "Search..."))
  )
))
```

For datasets over ~50k rows, consider server-side processing instead.
See `vignette("shiny-integration")` for details.


## Column Formatting

DT2 provides helper functions that build DataTables `columnDefs` for
common formatting tasks — number separators, decimal places, prefixes,
and suffixes. These modify your options list in-place and generate the
correct JavaScript `render` functions behind the scenes, so you don't
need to write any JS:

Note the `opts <- list(columns = names(mtcars))` line: name-based helpers
resolve column names against `options$columns`, so set it once before calling
them (or pass 1-based indices). DT2 warns if it is missing.

```{r}
opts <- list(columns = names(mtcars))
opts <- dt2_format_number(opts, "hp", thousands = ",", digits = 0)
opts <- dt2_format_number(opts, "wt", digits = 2, prefix_right = " tons")
dt2(mtcars[1:10, ], options = opts)
```

### Abbreviated numbers (k/M/B)

For large numbers (population, budgets, revenues), use abbreviated
formatting that automatically converts values to human-readable suffixes.
The `locale` parameter controls the decimal separator and number grouping:

```{r}
df <- data.frame(
  city   = c("São Paulo", "Recife", "NYC", "Tokyo"),
  pop    = c(12.33e6, 1.65e6, 8.34e6, 13.96e6),
  budget = c(6.5e10, 4.2e9, 1.07e11, 7.36e10)
)
opts <- list(columns = names(df))
opts <- dt2_format_number_abbrev(opts, c("pop", "budget"),
                                  digits = 1, locale = "pt-BR")
dt2(df, options = opts)
```


## DataTables Options

Every option from the [DataTables JS API](https://datatables.net/reference/option/)
maps 1:1 to an R named list. If you find an example on the DataTables
website, you can translate it directly to R without any wrapper functions.
This means the full power of DataTables is available — including options
not covered by DT2's helper functions:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  searching  = TRUE,
  ordering   = TRUE,
  language   = list(search = "Filter:", info = "_TOTAL_ rows")
))
```

See `vignette("js-config")` for the complete translation guide.


## Saving as HTML

Any DT2 table can be saved as a standalone HTML file using
`htmlwidgets::saveWidget()`. The resulting file includes all CSS and
JavaScript — it works offline and can be emailed, hosted, or embedded
in other pages:

```r
w <- dt2(iris)
htmlwidgets::saveWidget(w, "my_table.html", selfcontained = TRUE)
```

## Keeping Libraries Up to Date

DT2 bundles DataTables core, extensions, jQuery, and other JS/CSS
dependencies inside the package. You can check whether newer versions
are available on npm and CDN without leaving R:

```r
dt2_check_updates()
```

If you are developing DT2 itself, apply updates automatically:

```r
dt2_update_libs()            # patch + download
dt2_update_libs(dry_run = TRUE)  # preview only
```

Version constraints prevent breaking upgrades (jQuery stays on 3.x,
pdfmake on 0.2.x). See `?.dt2_version_constraints` for details.


## Complete Example: Custom Renderers, Flags, and ColumnControl

This example brings together most DT2 features in a single, realistic
table: ColumnControl dropdown menus in the header for sorting and
filtering, export buttons separated by a visual spacer, custom JavaScript
renderers (country flags via CSS sprites, coloured salary values, progress
bars), and a complete Portuguese (Brazil) translation.

Copy and run it directly — the full 57-row version is also available at
`system.file("examples/app_complete.R", package = "DT2")`.


```{css, echo=FALSE}
.f32 .flag {
  display: inline-block;
  width: 32px;
  height: 32px;
  vertical-align: middle;
  margin-right: 6px;
}
table.dataTable tbody td {
  vertical-align: middle;
}
```

```{r}
library(jsonlite)
library(dplyr)
library(tibble)
library(lubridate)
library(DT2)
library(htmlwidgets)

# ── Flag sprite CSS (carregado via dependency para garantir que entra no HTML) ──
flag_dep <- htmltools::htmlDependency(
  name = "world-flags-sprite",
  version = "0.0.1",
  head = '<link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/lafeber/world-flags-sprite/stylesheets/flags32-both.css">',
  src = c(href = ".")
)

# ── Dados ─────────────────────────────────────────────────────────────────────
json_txt <- '{
  "data": [
    {"name":"Tiger Nixon","position":"System Architect","salary":"320800","start_date":"2011-04-25","office":"Edinburgh","extn":"5421"},
    {"name":"Garrett Winters","position":"Accountant","salary":"170750","start_date":"2011-07-25","office":"Tokyo","extn":"8422"},
    {"name":"Ashton Cox","position":"Junior Technical Author","salary":"86000","start_date":"2009-01-12","office":"San Francisco","extn":"1562"},
    {"name":"Cedric Kelly","position":"Senior JavaScript Developer","salary":"433060","start_date":"2012-03-29","office":"Edinburgh","extn":"6224"},
    {"name":"Airi Satou","position":"Accountant","salary":"162700","start_date":"2008-11-28","office":"Tokyo","extn":"5407"},
    {"name":"Brielle Williamson","position":"Integration Specialist","salary":"372000","start_date":"2012-12-02","office":"New York","extn":"4804"},
    {"name":"Herrod Chandler","position":"Sales Assistant","salary":"137500","start_date":"2012-08-06","office":"San Francisco","extn":"9608"},
    {"name":"Rhona Davidson","position":"Integration Specialist","salary":"327900","start_date":"2010-10-14","office":"Tokyo","extn":"6200"},
    {"name":"Colleen Hurst","position":"JavaScript Developer","salary":"205500","start_date":"2009-09-15","office":"San Francisco","extn":"2360"},
    {"name":"Sonya Frost","position":"Software Engineer","salary":"103600","start_date":"2008-12-13","office":"Edinburgh","extn":"1667"}
  ]
}'

df <- fromJSON(json_txt, flatten = TRUE)$data %>%
  as_tibble() %>%
  mutate(
    salary     = as.numeric(salary),
    extn       = as.integer(extn),
    start_date = ymd(start_date)
  )

# ── JS Renderers ──────────────────────────────────────────────────────────────
office_js <- JS("
  function(data, type) {
    if (type !== 'display') return data;
    var cc = {Argentina:'ar', Edinburgh:'_Scotland', London:'_England',
              'New York':'us', 'San Francisco':'us', Sydney:'au', Tokyo:'jp'};
    return '<span class=\"flag ' + (cc[data]||'') + '\"></span> ' + data;
  }
")

salary_js <- JS("
  (function() {
    var nfmt = DataTable.render.number('.', ',', 2, 'R$ ');
    return function(data, type) {
      var txt = nfmt.display(data);
      if (type !== 'display') return txt;
      var c = data < 250000 ? 'red' : data < 500000 ? 'orange' : 'green';
      return '<span style=\"color:' + c + '\">' + txt + '</span>';
    };
  })()
")

extn_js <- JS("
  function(data, type) {
    return type === 'display'
      ? '<progress value=\"' + data + '\" max=\"9999\"></progress>'
      : data;
  }
")

# ── Tabela ────────────────────────────────────────────────────────────────────
w <- dt2(df,
  compact    = TRUE,
  striped    = TRUE,
  hover      = TRUE,
  font_scale = 0.85,
  responsive = FALSE,
  options    = list(
    pageLength = 10,
    lengthMenu = c(5, 10, 25, -1),
    columns    = names(df),
    scrollX = TRUE,  
    layout = list(
      topStart = "pageLength",
      topEnd   = list(
        buttons = list(
          list(extend = "copyHtml5", text = "Copiar"),
          list(extend = "csvHtml5"),
          list(extend = "excelHtml5"),
          list(extend = "spacer", style = "bar"),
          list(extend = "colvis", text = "Colunas")
        ),
        search = list(placeholder = "")
      ),
      bottomEnd = list(paging = list(numbers = FALSE))
    ),

    columnControl = list(
      target  = 0,
      content = list("order", "searchDropdown", list(
        list(extend = "orderAsc",       text = "Ordem crescente"),
        list(extend = "orderDesc",      text = "Ordem decrescente"),
        "spacer",
        list(extend = "colVisDropdown", text = "Selecionar colunas")
      ))
    ),
    ordering = list(indicators = FALSE, handler = FALSE),

    columnDefs = list(
      list(targets = which(names(df) == "office") - 1L,
           className = "f32", render = office_js),
      list(targets = which(names(df) == "salary") - 1L,
           className = "dt-body-right", render = salary_js),
      list(targets = which(names(df) == "extn") - 1L,
           render = extn_js)
    ),

    language = list(
      lengthMenu   = "Mostrar _MENU_",
      search       = "Buscar",
      info         = "Mostrando _START_ a _END_ de _TOTAL_ registros",
      infoEmpty    = "Nenhum registro",
      zeroRecords  = "Nenhum registro encontrado",
      emptyTable   = "Nenhum dado disponível",
      decimal      = ",", thousands = ".", infoThousands = ".",
      lengthLabels = list(`10` = "10", `25` = "25", `-1` = "Todas"),
      paginate     = list(first = "«", previous = "‹", `next` = "›", last = "»"),
      buttons = list(
        copyTitle   = "Copiado!",
        copySuccess = list(`_` = "%d linhas copiadas", `1` = "1 linha copiada")
      ),
      columnControl = list(
        orderAsc = "Crescente", orderDesc = "Decrescente",
        searchDropdown = "Pesquisar", colVisDropdown = "Colunas",
        searchClear = "Limpar",
        search = list(
          text   = list(contains = "Contém", starts = "Começa por",
                        ends = "Termina em", equal = "Igual a"),
          number = list(greater = "Maior que", less = "Menor que",
                        equal = "Igual a")
        )
      )
    )
  )
)

# Anexa a dependency do flag sprite ao widget
w$dependencies <- c(w$dependencies, list(flag_dep))
w
```

## Next Steps

- **Extensions**: `vignette("extensions-guide")` — Select, Responsive,
  ColumnControl, SearchBuilder, and more.
- **Shiny**: `vignette("shiny-integration")` — proxy, events, SSP.
- **JS Config**: `vignette("js-config")` — translating datatables.net
  examples to R, advanced layout, callbacks.
- **Formatting**: `vignette("formatting")` — all column formatting helpers.

---
title: "Column Formatting and Rendering"
author: "DT2 Team"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Column Formatting and Rendering}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)
```

## Overview

DT2 provides a set of helper functions that build `columnDefs` entries for
common formatting needs. All helpers follow the same pattern:

```r
opts <- list(columns = names(df))
opts <- dt2_format_*(opts, col_specs, ...)
dt2(df, options = opts)
```

The `col_specs` argument accepts column names (matching `options$columns`) or
1-based integer indices.

> **Why `columns = names(df)`?** When you refer to columns *by name*, DT2 needs
> to know the column order to translate names into positions. That list lives in
> `options$columns`, so set it once (`opts <- list(columns = names(df))`) before
> calling any name-based helper. If you skip it, DT2 emits a warning and the
> target cannot be resolved — pass 1-based indices instead if you prefer not to
> set it. (`dt2()` itself fills `options$columns` from the data for rendering,
> but the helpers run *before* `dt2()`, so they need it set explicitly.)

## Number Formatting

`dt2_format_number()` uses DataTables' built-in `DataTable.render.number()`:

```{r}
df <- data.frame(
  item   = c("Widget A", "Widget B", "Widget C"),
  price  = c(1234.5, 99999.99, 450.0),
  volume = c(1500000, 230000, 85000)
)

opts <- list(columns = names(df))
opts <- dt2_format_number(opts, "price",  thousands = ".", decimal = ",",
                           digits = 2, prefix = "R$ ")
opts <- dt2_format_number(opts, "volume", thousands = ",", digits = 0)
dt2(df, options = opts)
```

## Abbreviated Numbers

`dt2_format_number_abbrev()` displays numbers as 1.2k, 3.4M, 5.6B:

```{r}
df <- data.frame(
  metric = c("Users", "Revenue", "Transactions", "API Calls"),
  value  = c(2.1e6, 850e3, 12.5e6, 3.7e9)
)
opts <- list(columns = names(df))
opts <- dt2_format_number_abbrev(opts, "value", digits = 1)
dt2(df, options = opts)
```

## Date/Time Formatting

`dt2_format_datetime()` uses `DataTable.render.datetime()` for locale-aware
date display:

```{r}
df <- data.frame(
  event = c("Launch", "Update", "Sunset"),
  date  = c("2025-01-15", "2025-06-20", "2025-12-31")
)
opts <- list(columns = names(df))
opts <- dt2_format_datetime(opts, "date", to = "DD/MM/YYYY")
dt2(df, options = opts)
```

## Relative Time

`dt2_format_time_relative()` uses Moment.js to show "2 hours ago", etc.:

```{r}
df <- data.frame(
  action  = c("Login", "Upload", "Export", "Logout"),
  when    = format(Sys.time() - c(60, 3600*3, 86400*2, 15), "%Y-%m-%d %H:%M:%S")
)
opts <- list(columns = names(df))
opts <- dt2_format_time_relative(opts, "when", locale = "en")
dt2(df, options = opts)
```

## Custom JS Renderers

For anything beyond the built-in helpers, use `dt2_cols_render_js()`:

```{r}
# Traffic light renderer
traffic_light <- htmlwidgets::JS("
  function(data, type, row, meta) {
    if (type !== 'display') return data;
    var v = parseFloat(data);
    var color = v >= 80 ? '#198754' : (v >= 50 ? '#ffc107' : '#dc3545');
    return '<span style=\"display:inline-block;width:12px;height:12px;' +
           'border-radius:50%;background:' + color + ';margin-right:6px\">' +
           '</span>' + data + '%';
  }
")

df <- data.frame(
  server = c("web-01", "web-02", "db-01", "cache-01"),
  uptime = c(99.9, 87.3, 45.2, 100.0)
)
opts <- list(columns = names(df))
opts <- dt2_cols_render_js(opts, "uptime", traffic_light)
dt2(df, options = opts)
```

## HTML Templates

`dt2_col_template()` provides a simple way to wrap cell values in HTML:

```{r}
df <- data.frame(
  name   = c("Alice", "Bob", "Carol"),
  email  = c("alice@example.com", "bob@example.com", "carol@example.com"),
  score  = c(95, 82, 91)
)
opts <- list(columns = names(df))
opts <- dt2_col_template(opts, "email",
  '<a href="mailto:{{VAL}}">{{VAL}}</a>')
dt2(df, options = opts)
```

## Orthogonal Rendering

`dt2_cols_render_orthogonal()` lets you return different values for display vs.
sorting vs. filtering:

```{r eval=FALSE}
opts <- dt2_cols_render_orthogonal(opts, "price",
  display = htmlwidgets::JS("function(d) { return '$' + d.toFixed(2); }"),
  sort    = htmlwidgets::JS("function(d) { return parseFloat(d); }")
)
```

## Named Renderers (Reuse Across Tables)

Register a renderer once, use it in multiple tables:

```{r}
dt2_register_renderer("pct_bar", htmlwidgets::JS("
  function(data, type) {
    if (type !== 'display') return data;
    var pct = Math.min(100, Math.max(0, parseFloat(data)));
    return '<div style=\"background:#e9ecef;border-radius:3px;margin:6px\">' +
           '<div style=\"width:' + pct + '%;background:#0d6efd;height:10px;' +
           'border-radius:3px\"></div></div>';
  }
"))

df <- data.frame(task = c("A", "B", "C"), done = c(75, 40, 95))
opts <- list(columns = names(df))
opts <- dt2_use_renderer(opts, "done", "pct_bar")
dt2(df, options = opts)
```

---
title: "Translating DataTables.net Config to R"
author: "DT2 Team"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Translating DataTables.net Config to R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)
```

## The 1:1 Mapping Principle

DT2's `options` argument maps directly to the DataTables JavaScript
configuration object. An R named list becomes a JS object; R vectors become
JS arrays. This means you can translate any example from
[datatables.net](https://datatables.net/examples/) directly to R.

### Translation Rules

| JavaScript | R |
|---|---|
| `{ key: value }` | `list(key = value)` |
| `[1, 2, 3]` | `c(1, 2, 3)` or `list(1, 2, 3)` |
| `true` / `false` | `TRUE` / `FALSE` |
| `null` | `NULL` |
| `"string"` | `"string"` |
| `function(d) { ... }` | `htmlwidgets::JS("function(d) { ... }")` |

### Example: JS to R

**JavaScript** (from datatables.net):
```js
$('#myTable').DataTable({
  pageLength: 25,
  ordering: true,
  language: {
    search: "Filter:",
    lengthMenu: "Show _MENU_ entries"
  },
  columnDefs: [
    { targets: 0, visible: false },
    { targets: [1, 2], className: "text-center" }
  ]
});
```

**R** (with DT2):
```{r}
dt2(iris, options = list(
  pageLength = 25,
  ordering   = TRUE,
  language   = list(
    search     = "Filter:",
    lengthMenu = "Show _MENU_ entries"
  ),
  columnDefs = list(
    list(targets = 0, visible = FALSE),
    list(targets = c(1, 2), className = "text-center")
  )
))
```


## Layout — The Complete Guide

DataTables 2 replaced the old `dom` string with `layout`, a structured way
to position elements around the table. This is the most important change
from DataTables 1.x.

### Position grid

```
+------------------+------------------+
| topStart         | topEnd           |
+------------------+------------------+
| top (full width)                    |
+------------------+------------------+
| top2Start        | top2End          |
+------------------+------------------+
|               TABLE                 |
+------------------+------------------+
| bottomStart      | bottomEnd        |
+------------------+------------------+
| bottom (full width)                 |
+------------------+------------------+
| bottom2Start     | bottom2End       |
+------------------+------------------+
```

### Available elements

| Element | Description |
|---|---|
| `"search"` | Search/filter input |
| `"paging"` | Page navigation |
| `"info"` | "Showing X to Y of Z entries" |
| `"pageLength"` | Entries per page selector |
| `"buttons"` | Buttons toolbar (requires Buttons extension) |
| `"searchBuilder"` | SearchBuilder (requires extension) |
| `"searchPanes"` | SearchPanes (requires extension) |
| `NULL` | Remove whatever would normally be in that position |

### Default layout

When you don't specify `layout`, DataTables uses:

```r
layout = list(
  topStart    = "pageLength",
  topEnd      = "search",
  bottomStart = "info",
  bottomEnd   = "paging"
)
```

### Rearranging

Move search to the left, page length to the right:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart = list(search = list(placeholder = "Filter...")),
    topEnd   = "pageLength"
  )
))
```

### Removing elements

Set any position to `NULL`:

```{r}
dt2(iris, options = list(
  pageLength = 10,
  searching = FALSE,
  layout = list(
    topStart  = NULL,      # no page length selector
    topEnd    = NULL,      # no search box
    bottomStart = NULL,    # no info
    bottomEnd = "paging"   # only pagination
  )
))
```

### Multiple elements in one position

Wrap them in a list:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topStart  = list("pageLength", "info"),
    topEnd    = "search",
    bottomEnd = "paging"
  )
))
```

### Full-width rows

Use `top`, `top2`, `bottom`, `bottom2` for full-width rows:

```r
dt2(iris, options = list(
  layout = list(
    top       = "search",       # full-width search bar
    topStart  = "pageLength",
    topEnd    = "buttons",
    bottomEnd = "paging"
  )
))
```

### Buttons in layout

Place the Buttons toolbar in any position:

```{r}
# Buttons on the left
dt2(iris[1:15, ], options = list(
  buttons = list("copy", "csv", "excel"),
  layout = list(
    topStart  = "buttons",
    topEnd    = "search",
    bottomEnd = "paging"
  )
))
```

```{r}
# Buttons on the bottom
dt2(iris[1:15, ], options = list(
  buttons = list("copy", "csv"),
  layout = list(
    topEnd      = "search",
    bottomStart = "buttons",
    bottomEnd   = "paging"
  )
))
```

### Customized search box

```{r}
dt2(iris, options = list(
  pageLength = 5,
  layout = list(
    topEnd = list(search = list(
      placeholder = "Type to filter...",
      text = "Search:"
    ))
  )
))
```

### Complete custom layout

```{r}
dt2(mtcars[1:20, ], options = list(
  pageLength = 10,
  buttons = list(
    list(extend = "collection", text = "Export \u25BC",
         buttons = list("copyHtml5", "csvHtml5", "excelHtml5")),
    list(extend = "colvis", text = "Columns")
  ),
  layout = list(
    topStart    = "buttons",
    topEnd      = list(search = list(placeholder = "Filter...")),
    bottomStart = "info",
    bottomEnd   = "paging"
  )
))
```

### Migrating from `dom`

If you're coming from DataTables 1.x or the DT package:

| Old `dom` | New `layout` |
|---|---|
| `"frtip"` | (default — no need to specify) |
| `"tp"` | `list(topStart = NULL, topEnd = NULL, bottomStart = NULL, bottomEnd = "paging")` |
| `"Bfrtip"` | `list(topStart = "buttons")` |
| `"lfBrtip"` | `list(topStart = list("pageLength", "buttons"))` |

DT2 will automatically convert `dom` strings containing `"B"` to the
`layout` equivalent, but using `layout` directly is recommended.


## Using `htmlwidgets::JS()` for Callbacks

Whenever DataTables expects a JavaScript function, wrap it in
`htmlwidgets::JS()`:

```{r}
dt2(iris, options = list(
  pageLength = 5,
  createdRow = htmlwidgets::JS("
    function(row, data, dataIndex) {
      if (data['Sepal.Length'] > 5) {
        row.style.backgroundColor = '#fff3cd';
      }
    }
  ")
))
```

### Common Callbacks

| DataTables Option | Purpose |
|---|---|
| `createdRow` | Modify each row after creation |
| `initComplete` | Run code after table initializes |
| `drawCallback` | Run code after each draw |
| `headerCallback` | Modify the header after draw |
| `footerCallback` | Modify the footer after draw |
| `columns.render` | Custom cell rendering |

## Column Renderers

### Built-in DataTables Renderers

DataTables v2 provides built-in renderers accessible via `DataTable.render.*`:

```{r}
dt2(mtcars[1:10, c("mpg", "hp", "wt")], options = list(
  columnDefs = list(
    list(
      targets = 0,
      render = htmlwidgets::JS("DataTable.render.number('.', ',', 1, '', ' mpg')")
    ),
    list(
      targets = 1,
      render = htmlwidgets::JS("DataTable.render.number(',', '.', 0, '', ' hp')")
    )
  )
))
```

### Custom Render Function

```{r}
progress_render <- htmlwidgets::JS("
  function(data, type, row, meta) {
    if (type !== 'display') return data;
    var pct = Math.min(100, Math.max(0, parseFloat(data)));
    var color = pct > 70 ? '#198754' : (pct > 40 ? '#ffc107' : '#dc3545');
    return '<div style=\"background:#eee;border-radius:4px;overflow:hidden\">' +
           '<div style=\"width:' + pct + '%;background:' + color +
           ';height:14px;border-radius:4px\"></div></div>';
  }
")

df <- data.frame(
  task     = c("Design", "Backend", "Testing", "Deploy"),
  progress = c(85, 60, 30, 95)
)

dt2(df, options = list(
  pageLength = 10,
  columnDefs = list(
    list(targets = 1, render = progress_render)
  )
))
```


## Internationalization (i18n)

### Inline language object

```{r}
dt2(iris[1:10, ], options = list(
  pageLength = 5,
  language = list(
    search       = "Buscar:",
    lengthMenu   = "Mostrar _MENU_ registros",
    info         = "Mostrando _START_ a _END_ de _TOTAL_",
    paginate     = list(
      first    = "<<",
      previous = "<",
      `next`   = ">",
      last     = ">>"
    ),
    zeroRecords  = "Nenhum registro encontrado",
    emptyTable   = "Tabela vazia"
  )
))
```

### CDN language URL

DataTables provides ready-made translation files for 70+ languages:

```r
dt2(iris, options = list(
  language = list(
    url = "https://cdn.datatables.net/plug-ins/2.3.4/i18n/pt-BR.json"
  )
))
```


## Debugging

Open the browser console and look for `[DT2]` log messages. Access the
DataTables API object directly via JavaScript:

```js
// In browser console:
var el = document.getElementById('my_table');
var api = el._dt2;
api.page.info();      // pagination state
api.order();          // current ordering
api.search();         // current search term
api.rows().data();    // all row data
```

---
title: "Extensions Guide"
author: "DT2 Team"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Extensions Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE
)
library(DT2)
```

## How Extensions Work

DT2 loads extensions **on demand**. Instead of shipping all 15+ extensions to
every page, only the CSS/JS for extensions you actually use are included.

There are two ways to specify extensions:

1. **Explicit** — pass `extensions = c("Buttons", "Select")` to `dt2()`
2. **Auto-detection** — DT2 scans your `options` list and infers which
   extensions are needed (this is the default when `extensions = NULL`)

### Listing Available Extensions

```{r}
dt2_extensions()
```

## Buttons (Export / Copy / Print)

The Buttons extension adds export and utility buttons.

```{r}
dt2(mtcars[1:15, ], extensions = "Buttons", options = list(
  pageLength = 8,
  layout = list(
    topEnd = list(
      buttons = list("copy", "csv", "excel", "pdf", "print")
    )
  )
))
```

### Grouped Buttons with Labels

```{r}
dt2(iris[1:20, ], options = list(
  pageLength = 10,
  layout = list(
    topStart = "pageLength",
    topEnd = list(
      buttons = list(
        list(extend = "collection", text = "Export",
             buttons = list(
               list(extend = "copyHtml5", text = "Copy"),
               list(extend = "csvHtml5"),
               list(extend = "excelHtml5", title = "My Data"),
               list(extend = "pdfHtml5",   title = "My Data")
             )),
        list(extend = "colvis", text = "Columns")
      ),
      search = list(placeholder = "Filter...")
    ),
    bottomEnd = "paging"
  )
))
```

Note: DT2 auto-detected that `layout` contains `buttons`, so it loaded
the Buttons extension automatically (no need to set `extensions` explicitly).

## Select (Row Selection)

Enable row selection with the Select extension:

```{r}
dt2(iris[1:20, ], options = list(
  select = list(style = "os", items = "row"),
  pageLength = 10
))
```

In Shiny, selected rows are available via `input$<id>_state$selected`.

## Responsive

DT2 enables the Responsive extension by default. Tables fill 100% width
and columns that don't fit are collapsed into expandable child rows.

To disable for a specific table:

```{r}
dt2(iris, responsive = FALSE, options = list(pageLength = 5))
```

To customise Responsive behaviour (e.g., disable child rows):

```r
dt2(iris, options = list(
  responsive = list(details = FALSE)  # hide overflow silently
))
```

## ColumnControl

ColumnControl adds per-column controls (order indicators, search dropdowns,
column visibility) directly in the table header:

```{r}
dt2(iris, options = list(
  pageLength = 8,
  columnControl = list("order", "searchDropdown",
    list(
      list(extend = "orderAsc",  text = "Sort Ascending"),
      list(extend = "orderDesc", text = "Sort Descending"),
      "spacer",
      list(extend = "colVisDropdown", text = "Toggle Columns")
    )
  ),
  ordering = list(indicators = FALSE, handler = FALSE)
))
```

## FixedHeader

Keep the header visible while scrolling:

```{r eval=FALSE}
dt2(iris, options = list(fixedHeader = TRUE, pageLength = 50))
```

## Combining Extensions

Extensions compose naturally. Here's Buttons + Select + ColumnControl:

```{r}
dt2(iris, options = list(
  pageLength = 8,
  select = list(style = "multi", items = "row"),
  layout = list(
    topEnd = list(
      buttons = list(
        list(extend = "selected",  text = "Selected Only"),
        list(extend = "selectAll", text = "Select All"),
        list(extend = "selectNone", text = "Deselect"),
        "spacer",
        list(extend = "csvHtml5", exportOptions = list(modifier = list(selected = TRUE)))
      )
    )
  ),
  columnControl = list("order", "searchDropdown"),
  ordering = list(indicators = FALSE, handler = FALSE)
))
```

## Adding New Plugins

To add a DataTables plugin not yet in the registry:

1. Add version variable and download commands in `tools/get-dt2-libs.sh`
2. Register it in `R/dt2_extensions.R` (`.dt2_extension_registry()`)
3. Add auto-detection rules in `.dt2_detect_extensions()`
4. Add npm mapping in `R/dt2_check_updates.R` (`.dt2_npm_map()`)
5. Run `bash tools/get-dt2-libs.sh` to download the files
6. Test with `dt2(data, extensions = "YourPlugin")`

The declarative registry in `dt2_extensions.R` is the single source of truth
for extension metadata (version, JS files, CSS files, dependencies).

## Updating Library Versions

Check for newer versions of all bundled JS/CSS libraries:

```r
dt2_check_updates()
```

```
Library            Installed   Latest      Compat.     Status
-----------------------------------------------------------------
DataTables         2.3.4       2.3.7       2.3.7       ⚠️  UPDATE
jQuery             3.7.0       4.0.0       3.7.1       🔒 PINNED [pin: 3.x]
Buttons            3.2.5       3.2.6       3.2.6       ⚠️  UPDATE
...
```

If you are developing DT2 (working from the source tree), apply compatible
updates automatically:

```r
dt2_update_libs()                    # patch files + download new JS/CSS
dt2_update_libs(dry_run = TRUE)      # preview without changing anything
dt2_update_libs(download = FALSE)    # patch R files only (manual download)
```

Version constraints prevent incompatible upgrades:

- **jQuery** pinned to 3.x (DataTables 2 requires jQuery 3)
- **pdfmake** pinned to 0.2.x (0.3.x has breaking changes)
- **Bootstrap** pinned to 5.x

Libraries marked as `PINNED` are up to date within their allowed range.
Edit `.dt2_version_constraints()` to change the allowed ranges.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_server_processing.R
\name{.dt2_parse_ssp_request}
\alias{.dt2_parse_ssp_request}
\title{Internal: parse DataTables server-side request (query string to list)}
\usage{
.dt2_parse_ssp_request(req, n_cols)
}
\description{
Internal: parse DataTables server-side request (query string to list)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_inputs.R
\name{dt2_col_checkbox}
\alias{dt2_col_checkbox}
\title{Checkbox input per row}
\usage{
dt2_col_checkbox(
  options = list(),
  col,
  input_id_prefix = "row_chk_",
  value_col = NULL
)
}
\arguments{
\item{options}{Options list.}

\item{col}{Target column (name or 1-based index).}

\item{input_id_prefix}{Prefix for element ids (e.g., "row_chk_").}

\item{value_col}{Optional boolean column to define initial state.}
}
\value{
Updated \code{options}.
}
\description{
Checkbox input per row
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.patch_file}
\alias{.patch_file}
\title{Replace a string in a file (first occurrence only)}
\usage{
.patch_file(path, old, new)
}
\arguments{
\item{path}{file path}

\item{old}{string to find (fixed, not regex)}

\item{new}{replacement string}
}
\description{
Replace a string in a file (first occurrence only)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.npm_latest_in_range}
\alias{.npm_latest_in_range}
\title{Fetch the latest version of an npm package within a version prefix}
\usage{
.npm_latest_in_range(pkg, prefix)
}
\arguments{
\item{pkg}{npm package name}

\item{prefix}{version prefix, e.g. \code{"3."} or \code{"0.2."}}
}
\value{
version string or NA
}
\description{
Queries the full package metadata from npm and extracts all version
strings, then returns the highest version that starts with \code{prefix}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_shiny.R
\name{dt2_output}
\alias{dt2_output}
\title{Shiny output for DT2}
\usage{
dt2_output(outputId, width = "100\%", height = "auto")
}
\arguments{
\item{outputId}{Output ID (must match the \code{render_dt2()} call in server).}

\item{width, height}{CSS dimensions.}
}
\value{
An \code{htmlwidgets} Shiny output (HTML container) suitable for
inclusion in a Shiny UI definition.
}
\description{
Place a DT2 table in a Shiny UI.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_proxy_page}
\alias{dt2_proxy_page}
\title{Page navigation (proxy)}
\usage{
dt2_proxy_page(
  proxy,
  page = c("first", "previous", "next", "last", "number"),
  number = NULL
)
}
\arguments{
\item{proxy}{A \code{\link[=dt2_proxy]{dt2_proxy()}} object.}

\item{page}{Navigation action: \code{"first"}, \code{"previous"}, \code{"next"}, \code{"last"},
or \code{"number"} (go to a specific page).}

\item{number}{Page number (1-based). Only used when \code{page = "number"}.}
}
\value{
The proxy, invisibly.
}
\description{
Page navigation (proxy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_html.R
\name{dt2_col_template}
\alias{dt2_col_template}
\title{Simple HTML template per column (replace \code{{{VAL}}})}
\usage{
dt2_col_template(options = list(), col, template)
}
\arguments{
\item{options}{Options list.}

\item{col}{Name or index of target column.}

\item{template}{HTML string with \code{{{VAL}}} placeholder.}
}
\value{
Updated \code{options}.
}
\description{
Simple HTML template per column (replace \code{{{VAL}}})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_cols_align}
\alias{dt2_cols_align}
\title{Column align (Bootstrap 5 classes)}
\usage{
dt2_cols_align(options = list(), cols, align = c("left", "center", "right"))
}
\arguments{
\item{options}{Options list.}

\item{cols}{Names or 1-based indices.}

\item{align}{"left","center","right".}
}
\value{
Updated \code{options}.
}
\description{
Column align (Bootstrap 5 classes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_extensions.R
\name{.dt2_resolve_extensions}
\alias{.dt2_resolve_extensions}
\title{Resolve extension names with dependency ordering}
\usage{
.dt2_resolve_extensions(extensions)
}
\arguments{
\item{extensions}{character vector of extension names}
}
\value{
ordered character vector (dependencies first)
}
\description{
Resolve extension names with dependency ordering
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.version_gt}
\alias{.version_gt}
\title{Compare two version strings (a > b?)}
\usage{
.version_gt(a, b)
}
\description{
Compare two version strings (a > b?)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_draw}
\alias{dt2_draw}
\title{Redraw the table (proxy)}
\usage{
dt2_draw(proxy)
}
\arguments{
\item{proxy}{\code{\link[=dt2_proxy]{dt2_proxy()}}.}
}
\value{
The \code{proxy} object, returned invisibly.
}
\description{
Redraw the table (proxy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_cols_hide}
\alias{dt2_cols_hide}
\title{Hide columns}
\usage{
dt2_cols_hide(options = list(), cols)
}
\arguments{
\item{options}{Options list.}

\item{cols}{Names or 1-based indices.}
}
\value{
Updated \code{options}.
}
\description{
Hide columns
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.print_update_table}
\alias{.print_update_table}
\title{Pretty-print the update check table}
\usage{
.print_update_table(result)
}
\description{
Pretty-print the update check table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_proxy_search}
\alias{dt2_proxy_search}
\title{Global search (proxy)}
\usage{
dt2_proxy_search(
  proxy,
  value,
  regex = FALSE,
  smart = TRUE,
  caseInsensitive = TRUE
)
}
\arguments{
\item{proxy}{A \code{\link[=dt2_proxy]{dt2_proxy()}} object.}

\item{value}{Search string.}

\item{regex}{Logical; treat \code{value} as a regular expression? Default: \code{FALSE}.}

\item{smart}{Logical; use DataTables smart search? Default: \code{TRUE}.}

\item{caseInsensitive}{Logical; case-insensitive search? Default: \code{TRUE}.}
}
\value{
The proxy, invisibly.
}
\description{
Global search (proxy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_proxy}
\alias{dt2_proxy}
\title{Create a proxy for a DT2 table}
\usage{
dt2_proxy(id, session = shiny::getDefaultReactiveDomain())
}
\arguments{
\item{id}{Widget id used in \code{\link[=dt2_output]{dt2_output()}}.}

\item{session}{Shiny session.}
}
\value{
A \code{"DT2Proxy"} object.
}
\description{
Create a proxy for a DT2 table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_inputs.R
\name{dt2_col_button}
\alias{dt2_col_button}
\title{Action button per row}
\usage{
dt2_col_button(
  options = list(),
  col,
  label = "Action",
  input_id_prefix = "row_btn_"
)
}
\arguments{
\item{options}{Options list.}

\item{col}{Target column (name or 1-based index).}

\item{label}{Button label.}

\item{input_id_prefix}{Prefix for element ids (e.g., "row_btn_").}
}
\value{
Updated \code{options}.
}
\description{
Action button per row
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_extensions.R
\name{dt2-extensions}
\alias{dt2-extensions}
\alias{.dt2_extension_registry}
\title{DT2 Extension Registry}
\usage{
.dt2_extension_registry()
}
\description{
Declarative registry of all DataTables extensions supported by DT2.
Each extension specifies its version, JS/CSS files for core mode,
and optional bridge files for Bootstrap 5 mode.

Users select extensions via \code{dt2(extensions = c("Buttons", "Responsive"))}.
Only the selected extensions (plus required dependencies) are loaded.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_use_buttons}
\alias{dt2_use_buttons}
\title{Enable Buttons (extension) and define buttons}
\usage{
dt2_use_buttons(
  options = list(),
  buttons = c("copy", "csv", "excel", "print"),
  position = "topEnd",
  button_class = NULL
)
}
\arguments{
\item{options}{Options list.}

\item{buttons}{Vector of button ids (e.g., c("copy","csv","excel","print","colvis")).}

\item{position}{Where to place buttons in the layout.
One of \code{"topEnd"} (default), \code{"topStart"}, \code{"bottomEnd"},
\code{"bottomStart"}.}

\item{button_class}{CSS class for buttons (e.g., \code{"btn btn-sm btn-primary"}).
If \code{NULL}, uses the theme default (\code{"btn btn-sm btn-outline-secondary"}).
Applied per-button via \code{className}.}
}
\value{
Updated \code{options}.
}
\description{
Uses the modern DataTables 2.x \code{layout} API (not the deprecated \code{dom}).
}
\examples{
opts <- dt2_use_buttons(list(), buttons = c("copy", "csv", "excel"))
dt2(mtcars, options = opts)
}
\seealso{
\code{\link[=dt2_buttons]{dt2_buttons()}} for a lower-level variant that takes full button
objects and can relocate the buttons container to a custom CSS selector.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_server_processing.R
\name{dt2_bind_server}
\alias{dt2_bind_server}
\title{Bind a DataTables v2 server-side endpoint to a widget id}
\usage{
dt2_bind_server(
  id,
  data,
  session = shiny::getDefaultReactiveDomain(),
  handler = NULL
)
}
\arguments{
\item{id}{Output id of the widget (e.g., "tbl").}

\item{data}{A data.frame with the source data.}

\item{session}{Shiny session (default: current).}

\item{handler}{Optional custom handler function(data, req) -> list(...).}
}
\value{
No return value, called for side effects. Registers a Shiny
observer on \code{session} that responds to client-side server-processing
requests for the given widget \code{id}.
}
\description{
Bind a DataTables v2 server-side endpoint to a widget id
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_shiny.R
\name{observe_dt2_events}
\alias{observe_dt2_events}
\title{Observe DataTables events published by dt2.js}
\usage{
observe_dt2_events(input, id, handler)
}
\arguments{
\item{input}{Shiny input object.}

\item{id}{Widget ID.}

\item{handler}{Function with signature \verb{(event, type, indexes, rowData)}.}
}
\value{
No return value, called for side effects. Sets up a Shiny
observer that calls \code{handler} whenever the table emits an event.
}
\description{
Listen for table events (init, draw, order, search, page, select, deselect).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_server_processing.R
\name{.dt2_payload}
\alias{.dt2_payload}
\title{Internal: build DataTables JSON payload}
\usage{
.dt2_payload(draw, total, filtered, data_rows)
}
\description{
Internal: build DataTables JSON payload
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_extensions.R
\name{dt2_extensions}
\alias{dt2_extensions}
\title{List available DataTables extensions}
\usage{
dt2_extensions()
}
\value{
A data.frame with columns \code{name}, \code{version}, \code{dir}.
}
\description{
List available DataTables extensions
}
\examples{
dt2_extensions()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_language}
\alias{dt2_language}
\title{Language helper (either list or JSON url)}
\usage{
dt2_language(options = list(), lang_list = NULL, lang_url = NULL)
}
\arguments{
\item{options}{Options list.}

\item{lang_list}{Named list of language keys.}

\item{lang_url}{URL to a JSON translation file.}
}
\value{
Updated \code{options}.
}
\description{
Language helper (either list or JSON url)
}
\examples{
# Inline translation
opts <- dt2_language(list(), lang_list = list(search = "Buscar:"))
dt2(iris, options = opts)

# Or load a ready-made translation file from the DataTables CDN
opts <- dt2_language(list(),
  lang_url = "https://cdn.datatables.net/plug-ins/2.3.3/i18n/pt-BR.json")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_order}
\alias{dt2_order}
\title{Define initial ordering (option \code{order})}
\usage{
dt2_order(options = list(), ...)
}
\arguments{
\item{options}{Options list.}

\item{...}{Vectors like \code{c(col, "asc"/"desc")}. \code{col} may be name or 1-based index.}
}
\value{
Updated \code{options}.
}
\description{
Define initial ordering (option \code{order})
}
\examples{
opts <- list(columns = names(mtcars))
opts <- dt2_order(opts, c("mpg", "desc"))
dt2(mtcars, options = opts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2.R
\name{dt2_theme}
\alias{dt2_theme}
\title{Create a reusable DT2 theme}
\usage{
dt2_theme(
  preset = "default",
  striped = NULL,
  hover = NULL,
  compact = NULL,
  font_scale = NULL,
  style = NULL,
  button_class = NULL
)
}
\arguments{
\item{preset}{A named preset to start from: \code{"default"}, \code{"clean"},
\code{"minimal"}, or \code{"compact"}. Remaining arguments override the preset.}

\item{striped}{Logical; alternate row colours.}

\item{hover}{Logical; highlight rows on hover.}

\item{compact}{Logical; reduce cell padding.}

\item{font_scale}{Numeric; font-size multiplier (e.g., 0.85 = 85\%).}

\item{style}{Styling framework: \code{"bootstrap5"} (default) or \code{"core"}
(plain DataTables).}

\item{button_class}{CSS class string for Buttons extension buttons.
Default: \code{"btn btn-sm btn-outline-secondary"}. See Bootstrap 5
button classes for options (e.g., \code{"btn btn-sm btn-primary"}).}
}
\value{
A \code{dt2_theme} object (a named list).
}
\description{
Creates a theme object that can be passed to \code{\link[=dt2]{dt2()}} via the \code{theme}
parameter. Useful when you want the same look across many tables.

For quick one-off styling, you can also pass arguments directly to
\code{\link[=dt2]{dt2()}} (e.g., \code{dt2(iris, striped = FALSE)}).
}
\examples{
# Create and reuse
my_theme <- dt2_theme("clean", compact = TRUE)
\donttest{
dt2(iris, theme = my_theme)
dt2(mtcars, theme = my_theme)
}

# Custom button style
dt2_theme("default", button_class = "btn btn-sm btn-primary")

# Presets
dt2_theme("minimal")
dt2_theme("compact")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_cols_render_js}
\alias{dt2_cols_render_js}
\title{Attach a raw JS render function to columns}
\usage{
dt2_cols_render_js(options = list(), col_specs, js_render)
}
\arguments{
\item{options}{List returned, with \code{columnDefs} appended.}

\item{col_specs}{Column names or indices.}

\item{js_render}{A \code{htmlwidgets::JS()} function of signature
\code{function(data, type, row, meta) { ... }}.}
}
\value{
Modified \code{options}.
}
\description{
Provide a custom JS renderer for one or more columns. Use this when
you need fine control over \code{columns.render}, including returning
different outputs based on \code{type} (display/sort/filter/type).
}
\seealso{
\url{https://datatables.net/reference/option/columns.render}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_format_time_relative}
\alias{dt2_format_time_relative}
\title{Relative time using moment.fromNow(), with locale}
\usage{
dt2_format_time_relative(options = list(), col_specs, locale = "pt-br")
}
\arguments{
\item{options}{list of options (returned updated)}

\item{col_specs}{names or indices to format}

\item{locale}{e.g. "pt-br" (requires moment-with-locales)}
}
\value{
The modified \code{options} list with an updated \code{columnDefs} entry.
}
\description{
Relative time using moment.fromNow(), with locale
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_extensions.R
\name{.dt2_detect_extensions}
\alias{.dt2_detect_extensions}
\title{Auto-detect extensions from options}
\usage{
.dt2_detect_extensions(options)
}
\arguments{
\item{options}{DataTables options list}
}
\value{
character vector of detected extension names
}
\description{
Scans the options list for keys that imply certain extensions.
For example, \code{buttons} implies Buttons, \code{select} implies Select, etc.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.dt2_shell_var_map}
\alias{.dt2_shell_var_map}
\title{Shell script variable names for each library}
\usage{
.dt2_shell_var_map()
}
\description{
Shell script variable names for each library
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_utils.R
\name{dt2_use_renderer}
\alias{dt2_use_renderer}
\title{Use a named JS renderer on columns}
\usage{
dt2_use_renderer(options = list(), col_specs, name)
}
\arguments{
\item{options}{Options list (returned modified).}

\item{col_specs}{Column names or indices.}

\item{name}{Name used in \code{dt2_register_renderer()}.}
}
\value{
Modified \code{options}.
}
\description{
Use a named JS renderer on columns
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_format_number}
\alias{dt2_format_number}
\title{Format numeric columns (DataTables renderer: number)}
\usage{
dt2_format_number(
  options = list(),
  col_specs,
  thousands = NULL,
  decimal = NULL,
  digits = 0,
  prefix = "",
  prefix_right = ""
)
}
\arguments{
\item{options}{List of options (returned, with \code{columnDefs} updated).}

\item{col_specs}{Column names or 1-based indices to format.}

\item{thousands}{Thousands separator (character or \code{NULL} for auto).}

\item{decimal}{Decimal separator (character or \code{NULL} for auto).}

\item{digits}{Number of decimal places.}

\item{prefix, prefix_right}{String to prepend/append (e.g., currency symbol).}
}
\value{
Modified \code{options}.
}
\description{
Add a number renderer to one or more columns using
DataTables' built-in \code{DataTable.render.number}.
}
\examples{
opts <- list(columns = names(iris))
opts <- dt2_format_number(opts, "Sepal.Length", thousands = ".", decimal = ",",
                          digits = 2, prefix = "", prefix_right = "")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_server_processing.R
\name{dt2_ssp_handler}
\alias{dt2_ssp_handler}
\title{Default server-side handler (filter/order/page)}
\usage{
dt2_ssp_handler(names)
}
\arguments{
\item{names}{character() column names in display order.}
}
\value{
function(data, req) -> list(draw, recordsTotal, recordsFiltered, data)
}
\description{
Default server-side handler (filter/order/page)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{dt2_check_updates}
\alias{dt2_check_updates}
\title{Check for DataTables library updates}
\usage{
dt2_check_updates(quiet = FALSE)
}
\arguments{
\item{quiet}{Logical. If \code{TRUE}, returns the result invisibly without
printing. Default \code{FALSE}.}
}
\value{
A data.frame (invisibly) with columns:
\code{library}, \code{installed}, \code{latest}, \code{latest_ok},
\code{constraint}, \code{status}.

Status values:
\describe{
\item{\code{"ok"}}{Library is up to date.}
\item{\code{"UPDATE"}}{A compatible update is available.}
\item{\code{"PINNED"}}{A new major version exists, but is blocked by
the version constraint. The library is up to date within its
allowed range.}
\item{\code{"error"}}{Lookup failed (check your internet connection).}
}
}
\description{
Queries the npm registry to compare installed library versions against
the latest available versions. Version constraints are enforced to prevent
incompatible major version upgrades (e.g. jQuery 3.x will not jump to 4.x).
}
\examples{
\donttest{
dt2_check_updates()

# programmatic use
updates <- dt2_check_updates(quiet = TRUE)
updates[updates$status == "UPDATE", ]
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_cols_render_orthogonal}
\alias{dt2_cols_render_orthogonal}
\title{Orthogonal render (display/sort/filter/type) per column}
\usage{
dt2_cols_render_orthogonal(
  options = list(),
  col_specs,
  display = NULL,
  sort = NULL,
  filter = NULL,
  type = NULL
)
}
\arguments{
\item{options}{Options list to modify.}

\item{col_specs}{Column names or indices.}

\item{display}{Optional JS renderer for UI display.}

\item{sort}{Optional JS renderer used for ordering.}

\item{filter}{Optional JS renderer used for searching.}

\item{type}{Optional JS renderer used for type detection.}
}
\value{
Modified \code{options}.
}
\description{
Supply different renderers for each orthogonal data request. Pass an
object with keys \code{display}, \code{sort}, \code{filter}, \code{type}
(all optional). Each value must be a JS function.
}
\examples{
opts <- list(columns = names(iris))
opts <- dt2_cols_render_orthogonal(
  opts, "Sepal.Length",
  display = htmlwidgets::JS("function(d,t,row,meta){ return d + ' cm'; }"),
  sort    = htmlwidgets::JS("function(d,t,row,meta){ return parseFloat(d); }")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_format_time_format}
\alias{dt2_format_time_format}
\title{Format a date/time using DataTables' datetime renderer, with locale}
\usage{
dt2_format_time_format(
  options = list(),
  col_specs,
  from = NULL,
  to = "L",
  locale = "pt-br"
)
}
\arguments{
\item{options}{Options list (returned modified).}

\item{col_specs}{Column names or indices to format.}

\item{from}{Input format (e.g. \code{'YYYY-MM-DDTHH:mm:ssZ'} or \code{NULL} for ISO).}

\item{to}{Output format (e.g. \code{'L LTS'}). See moment.js docs.}

\item{locale}{Locale string, e.g. \code{'pt-br'}.}
}
\value{
Modified \code{options}.
}
\description{
Format a date/time using DataTables' datetime renderer, with locale
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_cols_escape}
\alias{dt2_cols_escape}
\title{Escape/unescape columns content}
\usage{
dt2_cols_escape(options = list(), cols, escape = TRUE)
}
\arguments{
\item{options}{Options list.}

\item{cols}{Names or indices.}

\item{escape}{If \code{TRUE} (default), HTML special characters are escaped so the
raw text is shown. If \code{FALSE}, the content is rendered as raw HTML
(use with care; only for trusted content).}
}
\value{
Updated \code{options}.
}
\description{
Controls whether cell content is HTML-escaped before display.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_deps.R
\name{dt2_deps}
\alias{dt2_deps}
\title{Build htmlwidgets dependencies for DT2}
\usage{
dt2_deps(
  bs = c("bootstrap5", "core"),
  include_bs = TRUE,
  extensions = character()
)
}
\arguments{
\item{bs}{\code{"bootstrap5"} (default) or \code{"core"} styling mode.}

\item{include_bs}{Logical; if TRUE and \code{bs="bootstrap5"}, include Bootstrap assets.
Default TRUE. Set FALSE only if your host page already provides Bootstrap.}

\item{extensions}{Character vector of extension names (e.g., \code{c("Buttons", "Select")}).
Use \code{dt2_extensions()} to see all available extensions.}
}
\value{
List of \code{htmlDependency()} objects in correct load order.
}
\description{
Constructs the dependency list dynamically based on the requested extensions.
Only loads CSS/JS for selected extensions, keeping the page lightweight.
}
\details{
All version numbers are read from \code{\link[=.dt2_lib_versions]{.dt2_lib_versions()}} (defined in
\code{dt2_check_updates.R}) so there is a single source of truth for versions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_cols_width}
\alias{dt2_cols_width}
\title{Column widths (CSS)}
\usage{
dt2_cols_width(options = list(), map_named)
}
\arguments{
\item{options}{Options list.}

\item{map_named}{Named character vector: c(Col="120px", ...).}
}
\value{
Updated \code{options}.
}
\description{
Column widths (CSS)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_replace_data}
\alias{dt2_replace_data}
\title{Replace all data in the table (proxy)}
\usage{
dt2_replace_data(proxy, data)
}
\arguments{
\item{proxy}{\code{\link[=dt2_proxy]{dt2_proxy()}}.}

\item{data}{New data.frame (will be serialized).}
}
\value{
The \code{proxy} object, returned invisibly.
}
\description{
Replace all data in the table (proxy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_utils.R
\name{dt2_register_renderer}
\alias{dt2_register_renderer}
\title{Register a named JS renderer}
\usage{
dt2_register_renderer(name, js)
}
\arguments{
\item{name}{Unique name (character scalar).}

\item{js}{A \code{htmlwidgets::JS()} function or a JSON helper expression.}
}
\value{
Invisibly, the name.
}
\description{
Register a named JS renderer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_format_number_abbrev}
\alias{dt2_format_number_abbrev}
\title{Abbreviate large numbers with fixed decimals (k / M / B)}
\usage{
dt2_format_number_abbrev(
  options = list(),
  col_specs,
  digits = 1,
  locale = NULL
)
}
\arguments{
\item{options}{A DataTables options list to be modified.}

\item{col_specs}{Column names or 1-based indices to format.}

\item{digits}{Integer, decimal places for the abbreviated display (default 1).}

\item{locale}{Optional BCP-47 locale string (e.g. "pt-BR"). If provided,
the non-abbreviated part uses \code{toLocaleString(locale)} for grouping.}
}
\value{
The modified \code{options} list.
}
\description{
Adds a \code{columns.render} function that displays numbers as 1.2k, 3.4M, etc.
This renderer \strong{lets you control} the number of decimal places via \code{digits}.
Use this when you want a fixed, compact style independent of locale rules.
}
\examples{
opts <- list(columns = names(mtcars))
opts <- dt2_format_number_abbrev(opts, c("hp","qsec"), digits = 1, locale = "pt-BR")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_proxy_order}
\alias{dt2_proxy_order}
\title{Order the table (proxy)}
\usage{
dt2_proxy_order(proxy, ..., columns = NULL)
}
\arguments{
\item{proxy}{\code{\link[=dt2_proxy]{dt2_proxy()}}.}

\item{...}{Vectors \code{c(col, "asc"/"desc")}. If you pass \code{columns}, names will be resolved to indices.}

\item{columns}{Optional character vector of column names to resolve names to indices.}
}
\value{
The \code{proxy} object, returned invisibly.
}
\description{
Order the table (proxy)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{dt2_update_libs}
\alias{dt2_update_libs}
\title{Update DataTables JS/CSS libraries (developer tool)}
\usage{
dt2_update_libs(pkg_dir = ".", download = TRUE, dry_run = FALSE)
}
\arguments{
\item{pkg_dir}{Path to the DT2 source root (the directory containing
\code{DESCRIPTION}). Defaults to the current working directory.}

\item{download}{Logical. If \code{TRUE}, runs the shell script after
patching version numbers. Default \code{TRUE}.}

\item{dry_run}{Logical. If \code{TRUE}, shows what would change without
modifying any files. Default \code{FALSE}.}
}
\value{
Invisibly, a data.frame with the update results.
}
\description{
Checks for updates (respecting version constraints), patches version
numbers in the source files, and optionally runs
\code{tools/get-dt2-libs.sh} to download the new files.
}
\details{
This function only works from the \strong{package source tree} (i.e.
during development). It will refuse to run from an installed package.

The workflow is:
\enumerate{
\item Query npm for the latest compatible version of every library.
\item Patch \code{tools/get-dt2-libs.sh} (version variables).
\item Patch \code{R/dt2_extensions.R} (extension registry).
\item Patch \code{R/dt2_check_updates.R} (core lib versions).
\item Patch \code{R/dt2_deps.R} (DataTables core version).
\item Run \code{bash tools/get-dt2-libs.sh} to download the files.
}

Version constraints prevent incompatible upgrades:
\itemize{
\item jQuery is pinned to 3.x (DataTables 2 requires jQuery 3).
\item pdfmake is pinned to 0.2.x (0.3.x has breaking changes and
is not available on cdnjs).
\item Bootstrap is pinned to 5.x.
}

Libraries marked as \code{"PINNED"} are skipped. Only \code{"UPDATE"}
items are applied.
}
\examples{
\dontrun{
# Developer-only tool: requires the DT2 package source tree
# (DESCRIPTION, tools/get-dt2-libs.sh, R/dt2_extensions.R, ...).
# It cannot run from an installed package, so it is not executable
# in CRAN check or from a regular user session.

# from the DT2 source root:
dt2_update_libs()

# preview changes without modifying anything:
dt2_update_libs(dry_run = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.dt2_lib_versions}
\alias{.dt2_lib_versions}
\title{Get all current library versions in DT2}
\usage{
.dt2_lib_versions()
}
\value{
Named character vector
}
\description{
Core library versions here must be kept in sync with
\code{R/dt2_deps.R} and \code{tools/get-dt2-libs.sh}.
Extension versions are read from the registry automatically.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_shiny.R
\name{dt2_state}
\alias{dt2_state}
\title{Access the current state snapshot of a DT2 table}
\usage{
dt2_state(input, id)
}
\arguments{
\item{input}{Shiny input object.}

\item{id}{Widget ID.}
}
\value{
A list with the current table state.
}
\description{
Returns a list with \code{reason}, \code{order}, \code{search}, \code{page}, \code{selected}, \code{state}
reflecting the current client-side table state.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_buttons.R
\name{dt2_buttons}
\alias{dt2_buttons}
\title{Configure DataTables Buttons and (optionally) move them to a custom container}
\usage{
dt2_buttons(
  options = list(),
  buttons = c("copyHtml5", "csvHtml5", "excelHtml5", "pdfHtml5", "print"),
  target = NULL
)
}
\arguments{
\item{options}{A DT2 \code{options} list you are building.}

\item{buttons}{Character vector with button names (e.g. \code{"copyHtml5"}, \code{"csvHtml5"}, \code{"excelHtml5"}, \code{"pdfHtml5"}, \code{"print"}).
You can also pass a list with full button objects.}

\item{target}{Optional CSS selector (e.g. \code{"#btn-slot"} or \code{".my-toolbar"}) to receive the
buttons container. If provided, DT2 will move the rendered buttons to that container after init.}
}
\value{
The modified \code{options} list.
}
\description{
Configure DataTables Buttons and (optionally) move them to a custom container
}
\details{
Requires the \strong{Buttons} extension. For CSV/Excel/PDF you also need \strong{JSZip} and \strong{pdfMake} (incl. \code{vfs_fonts}).

Prefer \code{\link[=dt2_use_buttons]{dt2_use_buttons()}} for the common case: it takes simple button ids,
styles them with a CSS class, and places them in the layout. Use
\code{dt2_buttons()} when you need full button objects or want to move the
rendered buttons into a custom container via \code{target}.
}
\seealso{
\code{\link[=dt2_use_buttons]{dt2_use_buttons()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.npm_latest_version}
\alias{.npm_latest_version}
\title{Fetch latest version of an npm package}
\usage{
.npm_latest_version(pkg)
}
\arguments{
\item{pkg}{npm package name}
}
\value{
version string or NA
}
\description{
Fetch latest version of an npm package
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_search_global}
\alias{dt2_search_global}
\title{Set global search (option \code{search})}
\usage{
dt2_search_global(
  options = list(),
  value,
  regex = FALSE,
  smart = TRUE,
  caseInsensitive = TRUE
)
}
\arguments{
\item{options}{Options list.}

\item{value}{Text.}

\item{regex, smart, caseInsensitive}{Search flags.}
}
\value{
Updated \code{options}.
}
\description{
Set global search (option \code{search})
}
\examples{
opts <- dt2_search_global(list(), value = "Toyota")
dt2(mtcars, options = opts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_html.R
\name{dt2_cols_html}
\alias{dt2_cols_html}
\title{Allow raw HTML rendering via \code{columns.render}}
\usage{
dt2_cols_html(options = list(), cols, js_render)
}
\arguments{
\item{options}{Options list.}

\item{cols}{Names or 1-based indices.}

\item{js_render}{JS function (via \link[htmlwidgets:JS]{htmlwidgets::JS}) with signature \verb{(data, type, row, meta)}
returning a string of HTML when \code{type == "display"}.}
}
\value{
Updated \code{options}.
}
\description{
Mark columns to render raw HTML using a JS render function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_proxy.R
\name{dt2_select_rows}
\alias{dt2_select_rows}
\title{Select rows (proxy; Select extension)}
\usage{
dt2_select_rows(proxy, indexes, reset = TRUE)
}
\arguments{
\item{proxy}{\code{\link[=dt2_proxy]{dt2_proxy()}}.}

\item{indexes}{1-based row indices.}

\item{reset}{If TRUE, clear selection before selecting.}
}
\value{
The \code{proxy} object, returned invisibly.
}
\description{
Select rows (proxy; Select extension)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.dt2_version_constraints}
\alias{.dt2_version_constraints}
\title{Version constraints for safe updates}
\usage{
.dt2_version_constraints()
}
\value{
Named character vector. Name = library name, value = version prefix.
Only libraries with constraints are included (NA = no constraint).
}
\description{
Defines the allowed version prefix for each library. Updates are only
applied within this prefix. For example, jQuery is constrained to
\code{"3."} so it will never auto-upgrade to 4.x.
}
\details{
Rationale for each constraint:
\describe{
\item{jQuery \code{"3."}}{DataTables 2.x requires jQuery 3.}
\item{PDFMake \code{"0.2."}}{0.3.x has breaking changes and is not
available on cdnjs.}
\item{Bootstrap \code{"5."}}{DataTables BS5 styling requires Bootstrap 5.}
\item{Moment \code{"2."}}{Major version pin for stability.}
\item{JSZip \code{"3."}}{Major version pin for stability.}
}

DataTables extensions do NOT have explicit constraints because their
major version tracks DataTables core compatibility (managed by the
DataTables project itself).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_shiny.R
\name{render_dt2}
\alias{render_dt2}
\title{Shiny render function for DT2}
\usage{
render_dt2(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{expr}{Expression returning a \code{\link[=dt2]{dt2()}} widget.}

\item{env, quoted}{Standard \code{shinyRenderWidget} arguments.}
}
\value{
A Shiny render function (closure produced by
\code{\link[htmlwidgets:shinyRenderWidget]{htmlwidgets::shinyRenderWidget()}}) that emits a DT2 widget.
}
\description{
Render a DT2 table in a Shiny server function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_options.R
\name{dt2_length_menu}
\alias{dt2_length_menu}
\title{Length menu helper}
\usage{
dt2_length_menu(options = list(), values = c(10, 25, 50, -1), labels = NULL)
}
\arguments{
\item{options}{Options list.}

\item{values}{Numeric vector of page lengths (e.g., \code{c(10, 25, 50, -1)}).
Use \code{-1} for "show all".}

\item{labels}{Optional character vector of labels. If \code{NULL}, numeric
values are used as-is and \code{-1} becomes \code{"All"} automatically via
\code{language.lengthLabels}.}
}
\value{
Updated \code{options}.
}
\description{
Configures the entries-per-page dropdown.
}
\examples{
opts <- dt2_length_menu(values = c(5, 10, 25, -1))
dt2(iris, options = opts)

opts <- dt2_length_menu(values = c(10, 50, 100), labels = c("10", "50", "100"))
dt2(iris, options = opts)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2.R
\name{dt2}
\alias{dt2}
\title{Create a DT2 DataTable widget}
\usage{
dt2(
  data,
  theme = "default",
  striped = NULL,
  hover = NULL,
  compact = NULL,
  font_scale = NULL,
  style = NULL,
  class = NULL,
  button_class = NULL,
  responsive = TRUE,
  options = list(),
  extensions = NULL,
  width = "100\%",
  height = NULL,
  elementId = NULL
)
}
\arguments{
\item{data}{A \code{data.frame}, \code{tibble}, or \code{matrix}.}

\item{theme}{A theme preset name (\code{"default"}, \code{"clean"}, \code{"minimal"},
\code{"compact"}) or a \code{\link[=dt2_theme]{dt2_theme()}} object. Default: \code{"default"}.}

\item{striped, hover, compact}{Logical; override the theme.
\code{NULL} (default) = use theme value.}

\item{font_scale}{Numeric; override the theme font-scale.
\code{NULL} (default) = use theme value.}

\item{style}{Styling framework: \code{"bootstrap5"} (default) or \code{"core"}.}

\item{class}{Optional CSS class string (e.g., \code{"table table-dark"}).
If provided, overrides all theme-generated classes.}

\item{button_class}{CSS class for Buttons extension buttons.
Default: \code{"btn btn-sm btn-outline-secondary"}.
Examples: \code{"btn btn-sm btn-primary"}, \code{"btn btn-sm btn-outline-dark"}.}

\item{responsive}{Logical; enable the Responsive extension so the table
fills 100\\% width and adapts to narrow screens by collapsing columns.
Default: \code{TRUE}. Set \code{FALSE} to disable.}

\item{options}{List of DataTables options. See
\url{https://datatables.net/reference/option/}.}

\item{extensions}{Character vector of extensions to load
(e.g., \code{c("Buttons", "Select")}). Auto-detected from \code{options}
when \code{NULL}.}

\item{width, height}{CSS dimensions.}

\item{elementId}{Optional HTML element ID.}
}
\value{
An \code{htmlwidget} object.
}
\description{
The main function for creating interactive DataTables.
Works standalone (R Markdown, Quarto, Viewer) and inside Shiny.

\strong{Styling} is controlled directly via \code{theme}, \code{striped}, \code{hover},
\code{compact}, \code{font_scale} -- or a CSS \code{class} string for full control.

\strong{DataTables configuration} goes in \code{options} (1:1 mapping to the
JavaScript API). The two concerns are cleanly separated.
}
\examples{
# Just works — beautiful defaults
dt2(iris)

\donttest{
# Override style inline
dt2(iris, striped = FALSE)
dt2(iris, font_scale = 0.85, compact = FALSE)

# Theme presets
dt2(iris, theme = "minimal")
dt2(iris, theme = "compact")

# Reusable theme
my_theme <- dt2_theme("clean", compact = TRUE)
dt2(iris, theme = my_theme)

# Override a preset
dt2(iris, theme = "minimal", striped = TRUE)

# CSS class override (power users)
dt2(iris, class = "table table-bordered table-dark")

# DataTables options (separate from styling)
dt2(iris, options = list(pageLength = 5, searching = FALSE))

# Disable responsive (fixed-width columns)
dt2(iris, responsive = FALSE)

# Everything composes
dt2(mtcars,
    theme = "clean",
    compact = TRUE,
    options = list(pageLength = 25))

# Buttons
dt2(mtcars, options = list(
  buttons = list("copy", "csv", "excel"),
  layout = list(topEnd = "buttons")
))

# Custom button style
dt2(mtcars,
    button_class = "btn btn-sm btn-primary",
    options = list(
      buttons = list("copy", "csv", "excel"),
      layout = list(topEnd = "buttons")
    ))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_formats.R
\name{dt2_format_datetime}
\alias{dt2_format_datetime}
\title{Format date/time columns (DataTables renderer: datetime)}
\usage{
dt2_format_datetime(
  options = list(),
  col_specs,
  from = NULL,
  to = "DD/MM/YYYY",
  locale = NULL,
  def = NULL
)
}
\arguments{
\item{options}{List of options (returned with \code{columnDefs} updated).}

\item{col_specs}{Column names or 1-based indices.}

\item{from}{Input format (e.g., \code{"YYYY-MM-DD"} or ISO8601 by default).}

\item{to}{Output format (e.g., \code{"DD/MM/YYYY"}).}

\item{locale}{Optional locale (e.g., \code{"pt-BR"}).}

\item{def}{Optional default output if input is invalid.}
}
\value{
Modified \code{options}.
}
\description{
Use DataTables' built-in \code{DataTable.render.datetime} to transform
date/time strings for display (and preserve sortability).
}
\examples{
opts <- list(columns = c("when", "val"))
opts <- dt2_format_datetime(opts, "when", from = "YYYY-MM-DD",
                            to = "DD/MM/YYYY", locale = "pt-BR")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dt2_check_updates.R
\name{.dt2_npm_map}
\alias{.dt2_npm_map}
\title{Mapping of DT2 library names to npm package names}
\usage{
.dt2_npm_map()
}
\description{
Mapping of DT2 library names to npm package names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.dt2_lock_mode}
\alias{.dt2_lock_mode}
\title{Lock Bootstrap mode per document (prevents mixing core/bootstrap5)}
\usage{
.dt2_lock_mode(bs)
}
\description{
Lock Bootstrap mode per document (prevents mixing core/bootstrap5)
}
\keyword{internal}
